#ifndef _KFbxMatrix_h
#define _KFbxMatrix_h

/**************************************************************************************

 Copyright  2000 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/


#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kaydara.h>

#include <kfbxmath/kfbxvector4.h>

#include <kfbxplugins/kfbxnamespacedef.h>
namespace FBXSDK_NAMESPACE {

class KFbxQuaternion;


/**	FBX SDK matrix class.
  * \nosubgrouping
  */
class KFBX_DLL KFbxMatrix
{

public:

	/**
	  * \name Constructors and Destructor
	  */
	//@{

	//! Constructor.
	KFbxMatrix ();

	//! Copy constructor.
	KFbxMatrix (const KFbxMatrix& pM);

	/** Constructor.
	  *	\param pT Translation vector.
	  *	\param pR Euler rotation vector.
	  *	\param pS Scale vector.
	  */
	KFbxMatrix (KFbxVector4& pT,
		        KFbxVector4& pR,
				KFbxVector4& pS);

	/** Constructor.
	  *	\param pT Translation vector.
	  *	\param pQ Quaternion.
	  *	\param pS Scale vector.
	  */
	KFbxMatrix (KFbxVector4& pT,
		        KFbxQuaternion& pQ,
				KFbxVector4& pS);
		
	//! Destructor.
	~KFbxMatrix ();
		
	//@}

	/**
	  * \name Access
	  */
	//@{

	/** Retrieve matrix element.
	  *	\param pY Row index.
	  *	\param pX Column index.
	  * \return Cell [ pX, pY ] value.
	  */
	double Get(int pY, int pX);

	/** Extract a row vector.
	  *	\param pY Row index.
	  * \return The row vector.
	  */
	KFbxVector4 GetRow(int pY);

	/** Extract a column vector.
	  *	\param pX Column index.
	  * \return The column vector.
	  */
	KFbxVector4 GetColumn(int pX);

	/** Set matrix element.
	  *	\param pY Row index.
	  *	\param pX Column index.
	  *	\param pValue New component value.
	  */
	void Set(int pY, int pX, double pValue);

	//! Set matrix to identity.
	void SetIdentity();
	
	/** Set matrix.
	  *	\param pT Translation vector.
	  *	\param pR Euler rotation vector.
	  *	\param pS Scale vector.
	  */
	void SetTRS(KFbxVector4& pT,
		        KFbxVector4& pR,
				KFbxVector4& pS);

	/** Set matrix.
	  *	\param pT Translation vector.
	  *	\param pQ Quaternion.
	  *	\param pS Scale vector.
	  */
	void SetTQS(KFbxVector4& pT,
		        KFbxQuaternion& pQ,
				KFbxVector4& pS);

	/** Set a matrix row.
	  *	\param pY Row index.
	  *	\param pRow	Row vector.
	  */
	void SetRow(int pY, KFbxVector4& pRow);

	/** Set a matrix column.
	  *	\param pX Column index.
	  *	\param pColumn Column vector.
	  */
	void SetColumn(int pX, KFbxVector4& pColumn);

	/** Assignment operator.
	  *	\param pMatrix Source matrix.
	  */
	KFbxMatrix& operator=(const KFbxMatrix& pMatrix);
	
	//@}

	/**
	  * \name Matrix Operations
	  */
	//@{	

	/**	Unary minus operator.
	  * \return A matrix where each element is multiplied by -1.
	  */
	KFbxMatrix operator-();
	
	/** Add two matrices together.
	  * \param pMatrix A Matrix.
	  * \return this + pMatrix.
	  */
	KFbxMatrix operator+(KFbxMatrix& pMatrix);

	/** Subtract a matrix from another matrix.
	  * \param pMatrix A Matrix.
	  * \return this - pMatrix.
	  */
	KFbxMatrix operator-(KFbxMatrix& pMatrix);

	/** Multiply two matrices together.
	  * \param pMatrix A Matrix.
	  * \return this * pMatrix.
	  */
	KFbxMatrix operator*(KFbxMatrix& pMatrix);

	/** Add two matrices together.
	  * \param pMatrix A Matrix.
	  * \return \e this updated with the result of the multiplication.
	  */
	KFbxMatrix& operator+=(KFbxMatrix& pMatrix);

	/** Subtract a matrix from another matrix.
	  * \param pMatrix A Matrix.
	  * \return \e this updated with the result of the multiplication.
	  */
	KFbxMatrix& operator-=(KFbxMatrix& pMatrix);

	/** Multiply two matrices together.
	  * \param pMatrix A Matrix.
	  * \return \e this updated with the result of the multiplication.
	  */
	KFbxMatrix& operator*=(KFbxMatrix& pMatrix);

	/** Calculate the matrix transpose.
	  * \return The transposed matrix of \e this.
	  */
	KFbxMatrix Transpose();

	//@}

	/**
	  * \name Boolean Operations
	  */
	//@{

	/**	Equivalence operator.
	  * \param pMatrix The matrix to be compared to \e this.
	  * \return \c true if the two matrices are equal (each element is within a 1.0e-6 tolerance)
	  * and \c false otherwise.
	  */
	bool operator==(KFbxMatrix& pM);

	/**	Non-equivalence operator.
	  * \param pMatrix The matrix to be compared to \e this.
	  * \return \c false if the two matrices are equal (each element is within a 1.0e-6 tolerance)
	  * and \c true otherwise.
	  */
	bool operator!=(KFbxMatrix& pM);
	
	//@}

	/**
	  * \name Casting
	  */
	//@{
	
	//! Cast the vector in a double pointer.
	operator double* ();

	//@}

	// Matrix data.
	double mData[4][4];

};

} // namespace
#endif // #ifndef _KFbxMatrix_h


