#ifndef _KFbxImageConverter_h
#define _KFbxImageConverter_h

/**************************************************************************************

 Copyright  2004 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/

#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kaydara.h>

#include <klib/kstring.h>

class KCharPtrSet;

#include <kfbxplugins/kfbxnamespacedef.h>
namespace FBXSDK_NAMESPACE {

class KFbxSdkManager;

const int ColorSpaceRGB = 0;
const int ColorSpaceYUV = 1;
const int FileToBuffer  = 0;
const int BufferToFile  = 1;

class ImageConverterBuffer
{
private:
	int     mWidth;			// nb of horizontal pixels
	int     mHeight;		// nb of vertical pixels
	int     mColorSpace;	// either RGB or YUV
	int     mPixelSize;		// 3 or 4 (case of RGBA)
	kByte  *mData;			// pointer to the array of pixels

	bool    mOriginalFormat;	// true if the data in this object has 
	KString mOriginalFileName;	// not been converted in any way

	bool    mValid;
	bool    mUseDataBuffer;    // if false, the data to use is an external file on disk
	                           // otherwise the data is the allocated memory buffer.

public:
	ImageConverterBuffer();
	~ImageConverterBuffer();

	/** Check if this object is correctly initialized.
	  * \return \c true if the object has been initialized with acceptable values.
	  */
	bool    IsValid()				{ return mValid; }

	/** Tells where the data to use is located.
	  * \return \c false if the data to use is an external file on disk
	  * otherwise returns \c true if the data is the allocated memory buffer.
	  */
	bool    UseDataBuffer()			{ return mUseDataBuffer; }

	/** Get the width of the image.
	  * \return The number of horizontal pixels.
	  */
	int     GetWidth()				{ return mWidth; }

	/** Get the height of the image.
	  * \return The number of vertical pixels.
	  */
	int     GetHeight()				{ return mHeight; }

	/** Get the color space of the image.
	  * \return Either ColorSpaceRGB or ColorSpaceYUV.
	  */
	int     GetColorSpace()			{ return mColorSpace; }

	/** Get the number of bytes per pixel.
	  * \return 3 for RGB images and 4 for RGBA images.
	  */
	char    GetPixelSize()			{ return mPixelSize; }

	/** Get access to the image data.
	  * \return Pointer to the array of pixels.
	  */
	kByte*  GetData()				{ return mData; }

	/** Tells if the image has not been converted in any way.
	  * \return \c true if the image is stored in its original format otherwise
	  * returns \c false.
	  */
	bool    GetOriginalFormat()		{ return mOriginalFormat; }

	/** Get the original filename of the image file (before any conversion).
	  * When a conversion to another format occurs, the filename of the converted
	  * image is different. The Original filename can be stored in the FBX file so
	  * the original file can be re-extracted from the converted one that has been 
	  * stored.
	  * \return The original filename.
	  */
	KString GetOriginalFileName()	{ return mOriginalFileName; }

	/** Initialize the object.
	  * \param pWidth Image width.
	  * \param pHeight Image height.
	  * \param pUseDataBuffer Pass \c true if the image buffer need to be allocated.
	  * \param pColorSpace The image color space.
	  * \param pPixelSize The number of bytes per pixel.
	  * \remark The total number of bytes allocated (if the pUseDataBuffer is \c true) is:\n
	  *       total = pWidth * pHeight * pPixelSize
	  */
	void Init(int pWidth, int pHeight, bool pUseDataBuffer, int pColorSpace = ColorSpaceRGB, char pPixelSize = 4);

	/** Set the original format flag.
	  * \param pState The value of the original format flag.
	  */
	void SetOriginalFormat(bool pState);

	/** Set the original filename string.
	  * \param pFilename The filename to use.
	  */
	void SetOriginalFileName(KString pFilename);
};

// This function should return 0 if successfull and any non zero value otherwise. And should
// init the pBuffer with the correct values.
typedef int (*ImageConverterFunction)(int pDirection, KString& pFileName, ImageConverterBuffer& pBuffer);


//! Provides a placeholder of functions to convert from a file format to "raw" data and vice et versa.
class KFBX_DLL KFbxImageConverter
{

public:

	/** Register a user converter function into the system.
	  * \param pFileExt The image file extension the registered function can handle.
	  * \param pFct The function that can convert the image file.
	  * \remark If the function can handle multiple image files, each file extension
	  * has to be registered individually (the same function can be used more than once
	  * in the RegisterConverterFunction).
	  */
	void RegisterConverterFunction(KString pFileExt, ImageConverterFunction pFct);

	/** Removes a user converter function from the system.
	  * \param pFct The function to be removed from the list of converters.
	  */
	void UnregisterConverterFunction(ImageConverterFunction pFct);

	/** Perform the actual conversion.
	  * \param pDirection Either FileToBuffer or BufferToFile.
	  * \param pFileName The destination filename (can be changed by the ImageConverterFunction).
	  * \param pBuffer The data placeholder.
	  * \return \c true if the conversion succeeded and \c false otherwise.
	  */
	bool Convert(int pDirection, KString& pFileName, ImageConverterBuffer& pBuffer);

protected:
///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS

	KFbxImageConverter(KFbxSdkManager& pManager);
	~KFbxImageConverter();

	KCharPtrSet*    mConverterFunctions;
	KFbxSdkManager* mManager;

	friend class KFbxSdkManager;

#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

};

// global utilities
bool IsCompressedTif(KString pFile);
bool ReadFromTif(KString pFile, ImageConverterBuffer& pBuffer, bool deleteFile);
bool WriteToTif(KString pFile, bool pCompressed, ImageConverterBuffer& pBuffer);

} // namespace

#endif // #ifndef _KFbxImageConverter_h


