#ifndef _KFbxLayer_h
#define _KFbxLayer_h

/**************************************************************************************

 Copyright  2001 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/

#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kaydara.h>

#include <klib/karrayul.h>
#include <klib/kdebug.h>

#include <kfbxmath/kfbxvector2.h>
#include <kfbxmath/kfbxvector4.h>
#include <kfbxplugins/kfbxcolor.h>

#include <kfbxplugins/kfbxnamespacedef.h>
namespace FBXSDK_NAMESPACE {

class KFbxTexture;
class KFbxMaterial;


/** KFbxLayerElement is the base class for Layer Elements. 
  * It describes how a Layer Element is mapped on a geometry surface and how the 
  * mapping information is arranged in memory.
  * \nosubgrouping
  */
class KFBX_DLL KFbxLayerElement
{

public:

	/** \enum ELayerElementType Type identifier for Layer Elements.
	  */
	typedef enum 
	{
		eUNDEFINED,         /**< Undefined Layer Element class.                      */
		eNORMAL,            /**< Layer Element of type KFbxLayerElementNormal.       */
		eMATERIAL,          /**< Layer Element of type KFbxLayerElementMaterial.     */
		eTEXTURE,           /**< Layer Element of type KFbxLayerElementTexture.      */
		ePOLYGON_GROUP,     /**< Layer Element of type KFbxLayerElementPolygonGroup. */
		eUV,                /**< Layer Element of type KFbxLayerElementUV.           */
		eVERTEX_COLOR       /**< Layer Element of type KFbxLayerElementVertexColor.  */
	} ELayerElementType;

	/**	\enum EMappingMode Determine how the element are mapped on a surface.
	  */
	typedef enum 
	{
		eNONE,              /**< The mapping is undetermined.                                                            */
		eBY_CONTROL_POINT,  /**< There will be one mapping coordinate for each surface control point/vertex.             */
		eBY_POLYGON_VERTEX, /**< There will be one mapping coordinate for each vertex, for each polygon it is part of.
	  	                         This means that a vertex will have as many mapping coordinates as polygons it is part of. */
		eBY_POLYGON,        /**< There can be only one mapping coordinate for the whole polygon.                          */
		eALL_SAME           /**< There can be only one mapping coordinate for the whole surface.                           */
	} EMappingMode;

	/** \enum EReferenceMode Determine how the mapping information is stored in the array of coordinate.
	  */
	typedef enum 
	{
		eDIRECT,           /**< This indicates that the mapping information for the n'th element
	  		                    is found in the n'th place of KFbxLayerElementTemplate::mDirectArray.                   */
		eINDEX,            /**< This indicates that the KFbxLayerElementTemplate::mIndexArray
	  		                    contains, for the n'th element, an index. This index does not refer to
	  		                    an element in the KFbxLayerElementTemplate::mDirectArray; but only
	  		                    refers to an index in an external table. \e eINDEX is usually useful to store
	  		                    material indices or texture indices that maybe stored elsewhere in a array
	  		                    of materials/textures.                                                                  */
		eINDEX_TO_DIRECT   /**< This indicates that the KFbxLayerElementTemplate::mIndexArray
	  		                    contains, for the n'th element, an index in the KFbxLayerElementTemplate::mDirectArray
	  		                    array of mapping elements. \e eINDEX_TO_DIRECT is usually useful to store coordinates
	  		                    for \e eBY_POLYGON_VERTEX mapping mode elements. Since the same coordinates are usually
	  		                    repeated a large number of times, it saves spaces to store the coordinate only one time
	  		                    and refer to them with an index.                                                        */
	} EReferenceMode;

	
	/** Set the Mapping Mode
	  * \param pMappingMode Specify the way the layer element is mapped on a surface.
	  */
	void SetMappingMode(EMappingMode pMappingMode) { mMappingMode = pMappingMode; }

	/** Set the Reference Mode
	  * \param pMappingMode Specify how the mapping information is arraged in memory.
	  */
	void SetReferenceMode(EReferenceMode pReferenceMode) { mReferenceMode = pReferenceMode; }

	/** Get the Mapping Mode
	  * \return Return the current Mapping Mode.
	  */
	EMappingMode GetMappingMode() { return mMappingMode; }

	/** Get the Reference Mode
	  * \return Return the current Reference Mode.
	  */
	EReferenceMode GetReferenceMode() { return mReferenceMode; }

///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS

	KFbxLayerElement() : mMappingMode(eNONE), mReferenceMode(eDIRECT) {}

protected:

	EMappingMode mMappingMode;
	EReferenceMode mReferenceMode;
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS
};


/** This class complement the KFbxLayerElement class.
  * \nosubgrouping
  */
template <class Type> class KFbxLayerElementTemplate : public KFbxLayerElement
{

public:
	/** Access the array of Layer Elements.
	  * \return Return a reference to the array.
	  * \remarks It is not legal to put elements in the Direct array when
	  *          the mapping mode is \e eINDEX.
	  */
	KArrayTemplate<Type>& GetDirectArray() 
	{ 
		K_ASSERT(mReferenceMode == KFbxLayerElement::eDIRECT || mReferenceMode == KFbxLayerElement::eINDEX_TO_DIRECT);
		return mDirectArray; 
	}

	/** Access the array of index.
	  * \return Return a reference to the array.
	  * \remarks It is not legal to put elements in the Index array when
	  *          the mapping mode is \e eDIRECT.
	  */
	KArrayTemplate<int>& GetIndexArray() 
	{ 
		K_ASSERT(mReferenceMode == KFbxLayerElement::eINDEX || mReferenceMode == KFbxLayerElement::eINDEX_TO_DIRECT);
		return mIndexArray; 
	}

	/** Remove the elements from the Direct and the Index array.
	  */
	void Clear()
	{
		mDirectArray.Clear();
		mIndexArray.Clear();
	}

///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS
protected:

	KArrayTemplate<Type> mDirectArray;
	KArrayTemplate<int> mIndexArray;
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS
};

/** Layer to map Normals on a geometry.
  * \nosubgrouping
  */
typedef class KFBX_DLL KFbxLayerElementTemplate<KFbxVector4>   KFbxLayerElementNormal;

/** Layer to map Materials on a geometry.
  * \nosubgrouping
  */
typedef class KFBX_DLL KFbxLayerElementTemplate<KFbxMaterial*> KFbxLayerElementMaterial;

/** Layer to group related polygons together.
  * \nosubgrouping
  */
typedef class KFBX_DLL KFbxLayerElementTemplate<int>          KFbxLayerElementPolygonGroup;

/** Layer to map UVs on a geometry.
  * \nosubgrouping
  */
typedef class KFBX_DLL KFbxLayerElementTemplate<KFbxVector2>   KFbxLayerElementUV;

/** Layer to map Vertex Colors on a geometry.
  * \nosubgrouping
  */
typedef class KFBX_DLL KFbxLayerElementTemplate<KFbxColor>	   KFbxLayerElementVertexColor;

typedef class KFBX_DLL KFbxLayerElementTemplate<KFbxTexture*>  KFbxLayerElementTextureBase;

/** Layer to map Textures on a geometry.
  * \nosubgrouping
  */
class KFBX_DLL KFbxLayerElementTexture : public KFbxLayerElementTextureBase
{

public:

	/** \enum EBlendMode Lets you control how textures are combined
	  * when multiple layers of texture are applied to an surface.
	  */
	typedef enum 
	{
		eTRANSLUCENT, /**< The new texture layer is transparent depending on the Alpha value. */
		eADD,         /**< The color of the new texture is added to the previous texture.     */
		eMODULATE,    /**< The color value of the new texture is multiplied by the color
	                       values of all previous layers of texture.                          */
		eMODULATE2,   /**< The color value of the new texure is multiplied by two and then
	                       multiplied by the color values of all previous layers of texture.  */
		eMAXBLEND     /**< Marks the end of the blend mode enum.                               */
	} EBlendMode;

	/** Constructor
	  * \remarks By default, textures have a Blend Mode of \e eTRANSLUCENT,
	  * a Reference Mode of \e eINDEX_TO_DIRECT and an Alpha value of 1.0.
	  */
	KFbxLayerElementTexture() : mBlendMode(eTRANSLUCENT)
	{
		mReferenceMode = eINDEX_TO_DIRECT;
		mAlpha         = 1.0;
	}

	/** Set the way Textures are blended between layers.
	  * \param pBlendMode one of the valid blend mode.
	  */
	void       SetBlendMode(EBlendMode pBlendMode) { mBlendMode = pBlendMode; }

	/** Set the level of transparency between multiple level of textures.
	  * \param pAlpha 0.0 is totally transparent, 1.0 is totally opaque.
	  * \remarks Values smaller than 0.0 are clipped to 0.0 and values larger than
	  * 1.0 are clipped to 1.0.
	  */
	void       SetAlpha(double pAlpha)             
	{ 
		mAlpha = pAlpha > 1.0 ? 1.0 : pAlpha;
		mAlpha = pAlpha < 0.0 ? 0.0 : pAlpha;
	}

	/** Get the way Textures are blended between layers.
	  * \return Return the current Blend Mode.
	  */
	EBlendMode GetBlendMode()                      { return mBlendMode; } 

	/** Get the level of transparency between multiple levels of textures.
	  * \return Return Alpha value. 0.0 is totally transparent, 1.0 is totally opaque.
	  */
	double     GetAlpha()                          { return mAlpha; }

///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS

private:

	EBlendMode mBlendMode;
	double     mAlpha;
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS
};


/** KFbxLayer class provides the base for the layering mecanism.
  * <p>
  * A layer can contain one or more of the following layer elements:
  * Normals, UVs, Textures, Materials, Polygon Groups and Vertex Colors.
  * A typical layer for a Mesh will contain Normals, UVs and Materials. 
  * A typical layer for a Nurbs will contain only Materials. If a texture
  * is applied to a Mesh, a typical layer will contain the Textures along
  * with the UVs. In the case of the Nurbs, the Nurbs' parametrisation is
  * used for the UVs; there should be no UVs specified.  The mapping of 
  * a texture is completely defined within the layer; there is no 
  * cross-layer management.
  * <p>
  * In most cases, a single layer is sufficient to describe a geometry.
  * Many applications will only support what is defined on the first layer. 
  * This should be taken into account when filling the layer. For example,
  * it is totaly legal to define the Layer 0 with the Textures and UVs and
  * define the model's Normals on the layer 1. However a file constructed
  * this way may not be imported correctly in other applications. The user
  * should put the Normals in Layer 0.
  * <p>
  * Texture layering is achieved by defining more than one layer containing
  * Textures and UVs elements. For example, a Mesh may have Textures and
  * the corresponding UVs elements on Layer 0 for the primary effect, and 
  * an other set of Textures and UVs on Layer 1. The way the texture blending
  * is done is defined in the Texture layer element.
  * \nosubgrouping
  */
class KFBX_DLL KFbxLayer
{

public:

	/**
	  * \name Layer Element Management
	  */
	//@{

	/** Get the Normals description for this layer.
	  * \return Pointer to the Normals layer element or \c NULL if no Normal are 
	  * defined on this layer.
	  * \remarks A geometry of type KFbxNurb or KFbxPatch should not have Normals
	  * defined.
	  */
	KFbxLayerElementNormal* GetNormals();	

	/** Get the Materials description for this layer.
	  * \return Pointer to the Materials layer element or \c NULL if no Materials are 
	  * defined on this layer.
	  */
	KFbxLayerElementMaterial* GetMaterials();

	/** Get the Textures description for this layer.
	  * \return Pointer to the Textures layer element or \c NULL if no Textures are 
	  * defined on this layer.
	  */
	KFbxLayerElementTexture* GetTextures();

	/** Get the Polygon Groups description for this layer.
	  * \return Pointer to the Polygon Groups layer element or \c NULL if no Polygon Groups are 
	  * defined on this layer.
	  */
	KFbxLayerElementPolygonGroup* GetPolygonGroups();

	/** Get the UVs description for this layer.
	  * \return Pointer to the UVs layer element or \c NULL if no UV are 
	  * defined on this layer.
	  * \remarks A geometry of type KFbxNurb or KFbxPatch should not have UVs
	  * defined. The Nurbs/Patch parametrization is used as UV parameters to map
	  * a texture.
	  */
	KFbxLayerElementUV* GetUVs();

	/** Get the Vertex Colors description for this layer.
	  * \return Pointer to the Vertex Colors layer element or \c NULL if no Vertex Color are 
	  * defined on this layer.
	  * \remarks A geometry of type KFbxNurb or KFbxPatch should not have Vertex Colors 
	  * defined, since no vertex exists.
	  */
	KFbxLayerElementVertexColor* GetVertexColors();

	/** Get the layer element description of the specified type for this layer.
	  * \param pType The Layer element type required. Supported values are \e KFbxLayerElement::eNORMAL,
	  * \e KFbxLayerElement::eMATERIAL, \e KFbxLayerElement::eTEXTURE, \e KFbxLayerElement::ePOLYGON_GROUP, 
	  * \e KFbxLayerElement::eUV and \e KFbxLayerElement::eVERTEX_COLOR. 
	  * <ul><li>Calling with \e eNORMAL is equivalent as calling \e GetNormals().
	  *     <li>Calling with \e eMATERIAL is equivalent as calling \e GetMaterials().
	  *     <li>Calling with \e eTEXTURE is equivalent as calling \e GetTextures().
	  *     <li>Calling with \e ePOLYGON_GROUP is equivalent as calling \e GetPolygonGroups().
	  *     <li>Calling with \e eUV is equivalent as calling \e GetUVs().
	  *     <li>Calling with \e eVERTEX_COLOR is equivalent as calling \e GetVertexColors().
	  * \return Pointer to the requested layer element or \e NULL if the layer element is not defined
	  * on this layer.
	  */
	KFbxLayerElement* GetLayerElementOfType(KFbxLayerElement::ELayerElementType pType);

	/** Set the Normals description for this layer.
	  * \param pNormals Pointer to the Normals layer element or \c NULL to remove the Normals definition.
	  * \remarks A geometry of type KFbxNurb or KFbxPatch should not have Normals defined.
	  */
	void SetNormals(KFbxLayerElementNormal* pNormals);

	/** Set the Materials description for this layer.
	  * \param pMaterials Pointer to the Materials layer element or \c NULL to remove the Material definition.
	  */
	void SetMaterials(KFbxLayerElementMaterial* pMaterials);

	/** Set the Textures description for this layer.
	  * \param pTextures Pointer to the Textures layer element or \c NULL to remove the Texture definition.
	  */
	void SetTextures(KFbxLayerElementTexture* pTextures);

	/** Set the Polygon Groups description for this layer.
	  * \param pPolygonGroups Pointer to the Polygon Groups layer element or \c NULL to remove the Polygon Group 
	  * definition.
	  */
	void SetPolygonGroups(KFbxLayerElementPolygonGroup* pPolygonGroups);

	/** Set the UVs description for this layer.
	  * \param pUVs Pointer to the UVs layer element or \c NULL to remove the UV definition.
	  * \remarks A geometry of type KFbxNurb or KFbxPatch should not have UVs
	  * defined. The Nurbs/Patch parametrization is used as UV parameters to map
	  * a texture.
	  */
	void SetUVs(KFbxLayerElementUV* pUVs);

	/** Set the Vertex Colors description for this layer.
	  * \param pVertexColors Pointer to the Vertex Colors layer element or \c NULL to remove the Vertex Color definition.
	  * \remarks A geometry of type KFbxNurb or KFbxPatch should not have Vertex Colors 
	  * defined, since no vertex exists.
	  */
	void SetVertexColors (KFbxLayerElementVertexColor* pVertexColors);
	
	//! Assignment operator.
	KFbxLayer& operator=(KFbxLayer& pSrcLayer);	
	//@}

///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS

private:

	KFbxLayer();
	virtual ~KFbxLayer();

	void Clear();

	KFbxLayerElementNormal*       mNormals;
	KFbxLayerElementMaterial*     mMaterials;
	KFbxLayerElementTexture*      mTextures;
	KFbxLayerElementPolygonGroup* mPolygonGroups;
	KFbxLayerElementUV*           mUVs;	
	KFbxLayerElementVertexColor*  mVertexColors;

	friend class KFbxLayerContainer;

#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS
};

} // namespace
#endif // #ifndef _KFbxLayer_h



