#ifndef _KFbxMesh_h
#define _KFbxMesh_h

/**************************************************************************************

 Copyright  2001 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/

#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kaydara.h>

#include <kfbxplugins/kfbxgeometry.h>

#include <kfbxmath/kfbxvector4.h>
#include <kfbxmath/kfbxvector2.h>

#include <klib/karrayul.h>
	   
#include <kfbxplugins/kfbxnamespacedef.h>
namespace FBXSDK_NAMESPACE {

class KFbxMesh;
class KFbxSdkManager;

/** A mesh is a geometry made of polygons.
  * Functions to initialize, set and access vertices are provided in the
  * KFbxGeometry base class. A vertex is referred as a control point in the 
  * KFbxGeometry base class. Though a control point is made of four elements,
  * meshes only use the first tree to store the XYZ coordinates.
  * <p>
  * Since the mesh-related terminology of the FBX SDK differs a little from 
  * the standard, here are some definitions:
  * <ul><li>A control point is a XYZ coordinate, it is synonym of vertex.
  *     <li>A polygon vertex is an index to a control point, it is synonym of vertex index.
  *     <li>A polygon is a group of polygon vertex.</ul>
  * \nosubgrouping
  */
class KFBX_DLL KFbxMesh : public KFbxGeometry 
{

public:

	/** Return the type of node attribute.
	  * \return Return the type of this node attribute which is \e EAttributeType::eMESH.
	  */
	virtual EAttributeType GetAttributeType();

	/**
	  * \name Polygon Management
	  */
	//@{

	/** Begin writing a polygon.
	  * \param pMaterial Index of material to assign to this polygon if material mapping 
	  * type is \e eBY_POLYGON. Otherwise it must be \c -1.
	  * \param pTexture Index of texture to assign to this polygon if texture mapping 
	  * type is \e eBY_POLYGON. Otherwise it must be \c -1.
	  * \param pGroup Group index assigned to polygon.
      */
	void BeginPolygon(int pMaterial = -1, int pTexture = -1, int pGroup = -1);

	/** Add an index to a control point (i.e. a polygon vertex) to the current polygon.
	  * \param pIndex Index to a control point (i.e. a polygon vertex).
	  * \param pTextureUVIndex Index of texture UV coordinates to assign to this polygon 
	  * if texture UV mapping type is \e eBY_POLYGON_VERTEX. Otherwise it must be \c -1.
      */
	void AddPolygon(int pIndex, int pTextureUVIndex = -1);

	//! End writing a polygon.
	void EndPolygon();

	/** Get the polygon count of this mesh.
	  * \return Return the number of polygons in the mesh.
	  */
	int GetPolygonCount();

	/** Get the number of polygon vertices in a polygon.
	  * \param pPolygonIndex Index of the polygon.
	  * \return The number of polygon vertices in the indexed polygon.
	  * If the polygon index is out of bounds, return -1.
	  */
	int GetPolygonSize(int pPolygonIndex);

	/** Get the group index assigned to a polygon.
	  * \param pPolygonIndex Index of the polygon.
	  * \return Group index assigned to a polygon.
	  * If the polygon index is out of bounds, return -1.
	  */
	int GetPolygonGroup(int pPolygonIndex);

	/** Get a polygon vertex (i.e. an index to a control point).
      * \param pPolygonIndex Index of polygon.
	  * The valid range for this parameter is 0 to \c KFbxMesh::GetPolygonCount().
	  * \param pPositionInPolygon Position of polygon vertex in indexed polygon.
	  * The valid range for this parameter is 0 to \c KFbxMesh::GetPolygonSize(pPolygonIndex).
      * \return Return the polygon vertex indexed or -1 if the requested vertex does not exists.
      */
	int GetPolygonVertex(int pPolygonIndex, int pPositionInPolygon);

	/** Get the normal associated with a polygon/vertex.
	  * \param pPolyIndex Index of the polygon.
	  * \param pVertexIndex Index of the vertex in the polygon space.
	  * \param pNormal The returned normal.
	  * \remarks \c pNormal remain unchanged if the requested vertex does
	  * not exists.
	  */
	void GetPolygonVertexNormal(int &pPolyIndex, int &pVertexIndex, KFbxVector4 &pNormal);
	
	/** Get the array of polygon vertices (i.e. index to control points).
	  * This array is a concatenation of the list of polygon vertices
	  * of all the polygons. Example: a mesh made of 2 triangles [1,2,3]
	  * and [2,3,4] results in [1,2,3,2,3,4]. The first polygon starts at 
	  * position 0 and the second at position 3.
      */
	int* GetPolygonVertices();
	
	//@}

	/**
	  * \name Texture UV Utility Functions.
	  * <p>
	  * The functions you will find in this section are utility functions
	  * to handle UV coordinates quickly. Internaly they refer to \c KFbxLayer
	  * and \c KFbxLayerElementUV methods to do the work.
	  * \remarks Those functions are only working on Layer 0. Use the \c KFbxLayer
	  * methods directly to access other layers.
	  */
	//@{

	/** Init texture UV coordinates.
	  * \param pCount Number of texture UV elements.
	  * \remarks \c pCount must equal the number of control points of the Mesh if
	  * the UV mapping mode is \e KFbxLayerElement::eBY_CONTROL_POINT.
      */
	void InitTextureUV(int pCount);

	/** Add texture UV coordinates.
	  * Appends a new element at the end of the array of texture UV coordinates.
	  * \param pUV Texture UV coordinates, ranging between \c 0 and \c 1.
	  * \remarks The final number of texture UV elements must equal the number of control 
	  * points if the UV mapping mode is \e KFbxLayerElement::eBY_CONTROL_POINT.
      */
	void AddTextureUV(KFbxVector2 pUV);

	//! Get the number of texture UV coordinates.
	int GetTextureUVCount();

	/** Get the array of texture UV coordinates.
	  * \return Pointer to array of texture UV coordinates or \c NULL if the array
	  * hasn't been allocated.
	  */
	KFbxVector2* GetTextureUV();

	//@}

	/**
	  * \name Material, Texture and UV Indices Utility Functions.
	  * <p>
	  * The functions you will find in this section are utility functions
	  * to handle Material, Texture and UV indices. Internaly they refer to \c KFbxLayer
	  * and \c KFbxLayerElement methods to do the work.
	  * \remarks Those functions are only working on Layer 0. Use the \c KFbxLayer
	  * methods directly to access other layers.
	  */
	//@{
	
	/** Initialize material indices.
	  * \param pMappingMode The mapping mode.
	  * This function must be called after function KFbxGeometryBase::InitControlPoints().
	  * The material indices refer to the position of a material in the KFbxLayerElementMaterial's direct array.
	  * See KFbxLayerElementMaterial for more details. Supported mapping types are \e eBY_CONTROL_POINT,
	  * \e eBY_POLYGON and \e eALL_SAME.
	  *		- If mapping mode is \e eBY_CONTROL_POINT, there will be as many indices in the material index array
	  *       as there are control points.
	  *		- If mapping mode is \e eBY_POLYGON, there will be as many indices in the material index array
	  *       as there are polygons in the mesh.
	  *		- If mapping mode is \e eALL_SAME, there will be only one index in the material index array.  
	  * \remarks This function will set the Reference mode of the KFbxLayerElementMaterial on layer 0 to \e eINDEX_TO_DIRECT.
      */
	void InitMaterialIndices(KFbxLayerElement::EMappingMode pMappingMode);

	/** Get material indices mapping mode.
	  * \return The mapping mode for the Material Layer Element on Layer 0, or
	  * \e eNONE if no mapping is defined.
	  */
	KFbxLayerElement::EMappingMode GetMaterialMappingMode();

	/** Get the array of material indices.
	  * \return Pointer to array of material indices or \c NULL. The function will
	  * return \c NULL in the following cases:
	  *		- There is no Material Layer Element (KFbxLayerElementMaterial) on Layer 0.
	  *		- There is a Material Layer Element, but the reference mode is \e eDIRECT.
      */
	int* GetMaterialIndices();

	/** Initialize texture indices.
	  * \param pMappingMode The mapping mode.
	  * The texture indices refer to the position of a texture in the KFbxLayerElementTexture's direct array.
	  * See KFbxLayerElementTexture for more details. Supported mapping modes are \e eBY_POLYGON 
	  * and \e eALL_SAME.
	  *		- If mapping mode is \e eBY_POLYGON, there will be as many indices in the texture index array
	  *       as there are polygons in the mesh.
	  *		- If mapping mode is \e eALL_SAME, there will be only one index in the texture index array.
	  * \remarks This function will set the Reference mode of the KFbxLayerElementTexture on layer 0 to \e eINDEX_TO_DIRECT.
      */
	void InitTextureIndices(KFbxLayerElement::EMappingMode pMappingMode);

	/** Get texture indices mapping mode.
	  * \return The mapping mode for the Texture Layer Element on Layer 0, or
	  * \e eNONE if no mapping is defined.
	  */
	KFbxLayerElement::EMappingMode GetTextureMappingMode();

	/** Get the array of texture indices.
	  * \return Pointer to array of texture indices or \c NULL. The function will
	  * return \c NULL in the following cases:
	  *		- There is no Texture Layer Element (KFbxLayerElementTexture) on Layer 0.
	  *		- There is a Texture Layer Element, but the reference mode is \e eDIRECT.
      */
	int* GetTextureIndices();

	/** Initialize texture UV indices.
	  * \param pMappingMode The mapping mode.
	  * The texture UV indices refer to the index of an element in the KFbxLayerElementTexture's direct array.
	  * See KFbxLayerElementTexture for more details. Supported mapping types are \e eBY_CONTROL_POINT , \e eBY_POLYGON_VERTEX 
	  * and \e eALL_SAME.
	  *		- If mapping mode is \e eBY_CONTROL_POINT, there will be as many indices in the UV index array
	  *       as there are control points. This will also set the Reference mode of the KFbxLayerElementUV on
	  *       layer 0 to \e eDIRECT.
	  *		- If mapping mode is \e eBY_POLYGON_VERTEX, there will be an index in the UV index array
	  *       for each vertex, for each polygon it is part of. This will also set the Reference mode of the KFbxLayerElementUV on
	  *       layer 0 to \e eINDEX_TO_DIRECT.
	  *     - If mapping mode is \e eALL_SAME, there will be no index in the UV index array. This will also set the Reference 
	  *       mode of the KFbxLayerElementUV on layer 0 to \e eDIRECT.
      */
	void InitTextureUVIndices(KFbxLayerElement::EMappingMode pMappingMode);

	/** Get texture UV indices mapping mode.
	  * \return The mapping mode for the UV Layer Element on Layer 0, or
	  * \e eNONE if no mapping is defined.
	  */
	KFbxLayerElement::EMappingMode GetTextureUVMappingMode();

	/** Get a texture UV index associated with a polygon vertex (i.e. an index to a control point).
      * \param pPolygonIndex Index of polygon.
	  * The valid range for this parameter is 0 to KFbxMesh::GetPolygonCount().
      * \param pPositionInPolygon Position of polygon vertex in indexed polygon.
	  * The valid range for this parameter is 0 to KFbxMesh::GetPolygonSize(pPolygonIndex).
      * \return Return a texture UV index.
	  * \remarks This function only works if the texture UV mapping mode is set to \e eBY_POLYGON_VERTEX,
	  * otherwise it returns -1. 
      */
	int GetTextureUVIndex(int pPolygonIndex, int pPositionInPolygon);

	/** Set a texture UV index associated with a polygon vertex (i.e. an index to a control point).
      * \param pPolygonIndex Index of polygon.
	  * The valid range for this parameter is 0 to KFbxMesh::GetPolygonCount().
      * \param pPositionInPolygon Position of polygon vertex in indexed polygon.
	  * The valid range for this parameter is 0 to KFbxMesh::GetPolygonSize(pPolygonIndex).
      * \param pIndex The index of the texture UV we want to assign to the polygon vertex.
	  * \remarks This function only works if the texture UV mapping type is set to \e eBY_POLYGON_VERTEX.
      */
	void SetTextureUVIndex(int pPolygonIndex, int pPositionInPolygon, int pIndex);
	
	/** Get the normal mapping mode.
	  * \return The mapping mode for the Normal Layer Element on Layer 0. Supported mapping types are \e eBY_POLYGON_VERTEX
	  * and \e eBY_CONTROL_POINT.
	  */
	KFbxLayerElement::EMappingMode GetNormalsMappingMode();

	/** Set the normal mapping mode.
	  * \param pMappingMode The mapping mode.
	  * Supported mapping modes are \e eBY_POLYGON_VERTEX and \e eBY_CONTROL_POINT.
	  */
	void SetNormalsMappingMode(KFbxLayerElement::EMappingMode pMappingMode);

	//@}

	/**
	  * \name Utility functions
	  */
	//@{

	/** Reset the mesh to default values.
	  *	Frees and set to \c NULL all layers and clear the polygon and the control point array.
      */
	void Reset();

	/** Compute the vertex normals on the mesh. 
	  * The normals are per vertex and are the average of all the polygon normals 
	  * associated with each vertex.
	  * \param pCW True if the normals are calculated clockwise, false otherwise (counter-clockwise).
	  */
	void ComputeVertexNormals(bool pCW =  false);

	/** Compares the normals calculated by doing cross-products between the polygon vertex and by the ones
	  * stored in the normal array. 
	  * \returns \c false if ALL of them are Clockwise. Returns \c true otherwise.
	  */
	bool CheckIfVertexNormalsCCW();

	/** \enum ESplitObject Object of interest when spliting.
	  */
	typedef enum 
	{
		eBY_NORMAL  /**< Each splited point will have a different normal for polygon/vertex. 
					     This is for normal mapping emulation. */
	} ESplitObject;

	//! Internal structure used to keep the duplicate vertex information.
	class KDuplicateVertex
	{
	public:
		KDuplicateVertex() :
		  lVertexPolyIndex(0),
		  lNewVertexIndex(0),
		  lNormal(0,0,0)
		  {
		  };

		int lVertexPolyIndex ; //!< Index in mPolygonsVertex where the vertex is found.
		int lNewVertexIndex;   //!< The new index of the vertex.
		KFbxVector4 lNormal;    //!< The normal associated with this duplicate control point.
		KFbxVector2 lUV;        //!< The UV associated with this duplicate control point.
	};

	//! Internal structure used to compute the normals on a mesh
	class KVertexNormalInfo
	{
	public:
		KVertexNormalInfo():
		  mTotalNormal(0,0,0),
		  mNumNormal(0)
		  {
		  };

		  KFbxVector4 mTotalNormal; //!< Sum of all the normals found.
		  int mNumNormal;          //!< Number of normals added.
	};

	typedef KArrayTemplate< KDuplicateVertex > KArrayOfDuplicateVertex;

	//! Assigment operator.
    KFbxMesh& operator= (KFbxMesh& pMesh);

	//@}

	/**
	  * \name Point Spliting/Merging utility functions
	  */
	//@{

	/** Insert the DuplicateVertex information in the pSplitList for the points that have to be splited
	  * depending on pObject value.
	  * \param pSlitList The list containing the KDuplicateVertex information of the points that will be splited.
	  * \param pObject The object of interest of the split.
	  */
	void BuildSplitList(KArrayTemplate<KArrayOfDuplicateVertex* >&pSplitList, ESplitObject pObject);

	/** Split the points specified in the list.
	  * \param pSplitList The List containing the information on the points that will be splited.
	  */
	void SplitPointsForHardEdge(KArrayTemplate<KArrayOfDuplicateVertex* > &pSplitList); 

	/** Insert the new indexes of the object that have to be merged.
	  * \param pMergeList The list that will contain the indexes of the objects to merge.
	  * \param pObject The object of interest of the merge.
	  * \param pExport If set to \c true, include the duplicate indexes in the merge list.
	  */
	bool BuildMergeList(KArrayTemplate<int> &pMergeList,ESplitObject pObject , bool pExport = false);

	/** Merge the points specified in the list.
	  * \param pMergeList list containing the information on the points that will be merge
	  */
	void MergePointsForPolygonVerteNormals(KArrayTemplate<int> &pMergeList);

	//@}

///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS

protected:

	KFbxMesh(KFbxSdkManager& pManager);
	~KFbxMesh();

	KString		GetTypeName();
	KStringList	GetTypeFlags();

	int    mUCount, mVCount;
	int    mUCountExtent, mVCountExtent;
	int    mUStep, mVStep;
	bool    mUClosed, mVClosed;
	bool    mUCapped, mVCapped;

	struct KFbxPolygon 
	{
		int mIndex;
		int mSize;
		int mGroup;
	};

	KArrayTemplate<KFbxPolygon> mPolygons;
	KArrayTemplate<int> mPolygonVertices;

	friend class KFbxReaderFbx;
	friend class KFbxWriterFbx;
	friend class KFbxWriterFbx6;
	friend class KFbxSdkManager;
	friend class KFbxGeometryConverter;
	friend class KFbxWriter3DS;

#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

};

typedef KFbxMesh* HKFbxMesh;

} // namespace
#endif // #ifndef _KFbxMesh_h


