#ifndef _KFbxScene_h
#define _KFbxScene_h

/**************************************************************************************

 Copyright  2001 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/

#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kaydara.h>

#include <kfbxplugins/kfbxdeformer.h>

#include <klib/karrayul.h>
#include <klib/kerror.h>
#include <klib/kstring.h>

#include <kfbxplugins/kfbxnamespacedef.h>
namespace FBXSDK_NAMESPACE {

class KFbxSdkManager;
class KFbxEntity;
class KFbxObject;
class KFbxNode;
class KFbxVideo;
class KFbxTexture;
class KFbxMaterial;
class KFbxMarkerSet;
class KFbxActor;
class KFbxCharacter;
class KFbxControlSetPlug;
class KFbxCharacterPose;
class KFbxPose;
class KFbxSceneInfo;
class KFbxTakeInfo;
class KFbxGlobalLightSettings;
class KFbxGlobalCameraSettings;
class KFbxGlobalTimeSettings;
class KFbxGenericNode;
class KFbxConstraint;


/** This class contains the description of a complete 3D scene.
  * \nosubgrouping
  * The FBX SDK requires unique names for nodes, textures, materials, generic nodes, videos and gobos contained 
  * in a given scene. To build a valid node tree, new nodes must be inserted under the 
  * scene's root node. Call function KFbxScene::ValidateNewName() with the object to 
  * insert in the following cases:
  *		- before inserting a node in the node tree under the scene's root node
  *		- before inserting a texture in a node attribute of type KFbxGeometry 
  *     - before inserting a video in the scene
  *		- before inserting a material in a node attribute of type KFbxGeometry 
  *     - before inserting a gobo in a node attribute of type KFbxLight 
  *		- before inserting a generic node in the scene
  *
  * \par
  * This class also provides access to global settings and take information.
  *
  */
class KFBX_DLL KFbxScene
{

public:

	//! Delete the node tree below the root node and restore default settings.
	void Clear();

	/** 
	  * \name Node Tree Access
	  */
	//@{

	/** Get the root node.
	  *	\return Pointer to the root node.
	  *	\remarks This node is not saved. Do not use it to apply a global transformation
	  * to the node hierarchy. If a global transformation must be applied, insert a 
	  * new node below this one.
	  */
    KFbxNode* GetRootNode();

	/** Get the camera switcher node.
	  *	\return Pointer to the camera switcher node.
	  *	\remarks This node has a node attribute of type \c KFbxNodeAttribute::eCAMERA_SWITCHER.
	  * This node isn't saved if the scene contains no camera.
	  *	Nodes inserted below are never saved.
	  *
	  * \par 
	  * Camera indices start at 1. Out of range indices are clamped between 1 and the 
	  * number of cameras in the scene. The index of a camera refers to its order of 
	  * appearance when searching the node tree depth first. 
	  *
	  * \par 
	  * Use function KFbxTakeNode::GetCameraIndex() to get and set the camera index.
	  *	If a camera is added or removed after camera indices have been set, the camera 
	  * indices must be updated. It's much simpler to set the camera indices once all 
	  * cameras have been set. 
	  *
	  * \par
	  * Camera index keys must be set with constant interpolation to make sure camera 
	  * switches occur exaclty at key time.
	  */
    KFbxNode* GetCameraSwitcher();

	/** Check if the new name is valid and can be assigned.
	  *	\param pObject May be a node, a texture, a material, a video or a gobo.
	  *	\param pNewName Name to be assigned to the object.
	  *	\return \c true if the name is accepted by the entity, \c false otherwise.
	  *	\return Will return \c false if called for anything else than a node, a texture, a material, a video or a gobo.
	  */
    bool ValidateNewName(KFbxObject& pObject, char* pNewName);

	//@}

	/** 
	  * \name Texture Material and Video Access
	  */
	//@{

	/** Clear then fill a video array with all existing videos included in the scene.
	  *	\param pVideoArray An array of video pointers.
	  */
	void FillVideoArray(KArrayTemplate<KFbxVideo*>& pVideoArray);

	/** Clear then fill a texture array with all existing textures included in the scene.
	  *	\param pTextureArray An array of texture pointers.
	  */
	void FillTextureArray(KArrayTemplate<KFbxTexture*>& pTextureArray);

	/** Clear then fill a material array with all existing materials included in the scene.
	  *	\param pMaterialArray An array of material pointers.
	  */
	void FillMaterialArray(KArrayTemplate<KFbxMaterial*>& pMaterialArray);
	
	//@}

	/** 
	  * \name Generic Node Access
	  */
	//@{

	/** Get number of generic nodes.
	  * \return Number of Generic Nodes in this scene.
	  */
	int             GetGenericNodeCount();

	/** Get generic node at given index.
	  * \param pIndex Position in the list of the generic nodes.
	  * \return Pointer to the generic node or \c NULL if the index is out of bounds.
	  */
	KFbxGenericNode* GetGenericNode(int pIndex);

	/** Add the generic node to this scene.
	  * \param pGenericNode Pointer to the generic node to be added.
	  * \return If the passed parameter is \c NULL, this method will return \c false otherwise \c true.
	  */
	bool             AddGenericNode(KFbxGenericNode* pGenericNode);

	/** Remove the generic node from this scene.
	  * \param pGenericNode Pointer to the generic node to be removed.
	  * \remark The pointed object is not referenced by the scene anymore but is not deleted from the system.
	  * \return If the passed parameter is \c NULL, this method will return \c false otherwise \c true.
	  */
	bool             RemoveGenericNode(KFbxGenericNode* pGenericNode);
	
	//@}

	/** 
	  * \name Constraints Access
	  */
	//@{

	/** Get the number of constraints.
	  * \return Number of Constraints in this scene.
	  */
	int				GetConstraintCount();

	/** Get the number of constraints of the specified type.
	  * \param pType Specify the type of constraints we are interested in.
	  * \return Number of Constraints in this scene.
	  */
	int				GetConstraintCount(int pType);

	/** Get the constraint at the given index.
	  * \param pIndex Position in the list of the constraints.
	  * \return Pointer to the constraint or \c NULL if the index is out of bounds.
	  * \remark pIndex must be between 0 and GetConstraintCount().
	  */
	KFbxConstraint*	GetConstraint(int pIndex);

	/** Get the Position constraint at the given index.
	  * \param pIndex Occurence of Position constraint in the list of the constraints.
	  * \return Pointer to the constraint or \c NULL if the index is out of bounds.
	  * \remark pIndex must be between 0 and GetConstraintCount(KFbxConstraint::ePOSITION).
	  */
	KFbxConstraint*	GetConstraintPosition(int pIndex);

	/** Get the Rotation constraint at the given index.
	  * \param pIndex Occurence of Rotation constraint in the list of the constraints.
	  * \return Pointer to the constraint or \c NULL if the index is out of bounds.
	  * \remark pIndex must be between 0 and GetConstraintCount(KFbxConstraint::eROTATION).
	  */
	KFbxConstraint*	GetConstraintRotate(int pIndex);

	/** Get the Scale constraint at the given index.
	  * \param pIndex Occurence of Scale constraint in the list of the constraints.
	  * \return Pointer to the constraint or \c NULL if the index is out of bounds.
	  * \remark pIndex must be between 0 and GetConstraintCount(KFbxConstraint::eSCALE).
	  */
	KFbxConstraint*	GetConstraintScale(int pIndex);

	/** Get the Single Chain IK constraint at the given index.
	  * \param pIndex Occurence of Single Chain IK constraint in the list of the constraints.
	  * \return Pointer to the constraint or \c NULL if the index is out of bounds.
	  * \remark pIndex must be between 0 and GetConstraintCount(KFbxConstraint::eSINGLECHAIN_IK).
	  */
	KFbxConstraint*	GetConstraintSingleChainIK(int pIndex);

	/** Get the Aim constraint at the given index.
	  * \param pIndex Occurence of Aim constraint in the list of the constraints.
	  * \return Pointer to the constraint or \c NULL if the index is out of bounds.
	  * \remark pIndex must be between 0 and GetConstraintCount(KFbxConstraint::eAIM).
	  */
	KFbxConstraint*	GetConstraintAim(int pIndex);

	/** Get the Character constraint at the given index.
	  * \param pIndex Occurence of Character constraint in the list of the constraints.
	  * \return Pointer to the constraint or \c NULL if the index is out of bounds.
	  * \remark pIndex must be between 0 and GetConstraintCount(KFbxConstraint::eCHARACTER_CONSTRAINT).
	  */
	KFbxConstraint*	GetConstraintCharacter(int pIndex);

	/** Add the constraint to this scene.
	  * \param pConstraint Pointer to the constraint to be added.
	  * \return If the passed parameter is \c NULL, this method will return \c false otherwise \c true.
	  */
	bool			AddConstraint(KFbxConstraint* pConstraint);

	//@}


	/** 
	  * \name Character Management
	  */
	//@{

	/** Get number of characters.
      * \return Number of characters in this scene.
	  */
	int GetCharacterCount();

	/** Get character at given index.
	  * \param pIndex Position in the list of the characters.
	  * \return Pointer to the character or \c NULL if index is out of bounds.
	  */
	KFbxCharacter* GetCharacter(int pIndex);

	/** Create a new character.
	  * \param pName Name given to character.
	  * \return Index of the created character.
	  */
	int CreateCharacter(char* pName);

	/** Destroy character.
	  * \param pIndex Specify which character to destroy.
	  * \remark After the destruction  of the requested element the list is resized.
	  */
	void DestroyCharacter(int pIndex);

	//@}

	/** 
	  * \name ControlSetPlug Management
	  */
	//@{

	/** Get number of ControlSetPlugs.
	  * \return Number of ControlSet plugs in this scene.
	  */
	int GetControlSetPlugCount();

	/** Get ControlSetPlug at given index.
	  * \param pIndex Position in the list of the ControlSetPlug
	  * \return Pointer to ControlSetPlug or \c NULL if index is out of bounds.
	  */
	KFbxControlSetPlug* GetControlSetPlug(int pIndex);

	/** Create a new ControlSetPlug.
	  * \param pName Name given to ControlSetPlug.
	  * \return Index of created ControlSetPlug.
	  */
	int CreateControlSetPlug(char* pName);

	/** Destroy ControlSetPlug.
      * \param pIndex Specify which ControlSetPlug to destroy.
	  * \remark After the destruction of the requested element the list is resized.
	  */
	void DestroyControlSetPlug(int pIndex);

	//@}

	/** 
	  * \name Character Pose Management
	  */
	//@{

	/** Get number of character poses.
      * \return Number of character poses in this scene.
	  * \remark Character Poses and Poses are two distinct entities having their own lists.
	  */
	int GetCharacterPoseCount();

	/** Get character pose at given index.
	  * \param pIndex Position in the list of character poses.
	  * \return Pointer to the character pose or \c NULL if index is out of bounds.
	  */
	KFbxCharacterPose* GetCharacterPose(int pIndex);

	/** Create a new character pose.
	  * \param pName Name given to character pose.
	  * \return Index of created character pose.
	  */
	int CreateCharacterPose(char* pName);

	/** Destroy character pose.
      * \param pIndex Specify which character pose to destroy.
	  * \remark After the destruction of the requested element the list is resized.
	  */
	void DestroyCharacterPose(int pIndex);

	//@}

	/** 
	  * \name Pose Management
	  */
	//@{

	/** Get number of poses.
	  * \return Number of poses in the scene.
	  * \remark Poses and Character Poses are two distinct entities having their own lists.
	  */
	int GetPoseCount();

	/** Get pose at given index.
	  * \param pIndex Position in the list of poses.
	  * \return Pointer to the pose or \c NULL if index is out of bounds.
	  */
	KFbxPose* GetPose(int pIndex);

	/**	Add a pose to this scene.
	  * \param pPose The pose to be added to the scene.
	  * \return If the pose is correctly added to the scene, return \true otherwise, if the pose is 
	  * already in the scene, returns \c false.
	  */
	bool AddPose(KFbxPose* pPose);
	
	/** Remove the specified pose from the scene.
	  * \param pPose The pose to be removed from the scene.
	  * \return If the pose was successfully removed from the scene, return \c true otherwise, if the 
	  * pose could not be found returns \c false.
	  */
	bool RemovePose(KFbxPose* pPose);

	/** Remove the pose at the given index from the scene.
	  *	\param pIndex The zero-based index of the pose to be removed.
	  * \return If the pose was successfully removed from the scene, return \c true otherwise, if the 
	  * pose could not be found returns \c false.
	  */
	bool RemovePose(int pIndex);


	//@}
	/** 
	  * \name Scene information
	  */
	//@{

	/**	Get the scene information.
	  * \return Pointer to the scene information object.
	  */
	KFbxSceneInfo* GetSceneInfo();

	/** Set the scene information.
	  * \param pSceneInfo Pointer to the scene information object.
	  */
	void SetSceneInfo(KFbxSceneInfo* pSceneInfo);

	//@}

	/** 
	  * \name Global Settings
	  */
	//@{

	/** Access global light settings.
	  * \return Reference to the Global Light Settings.
	  */
	KFbxGlobalLightSettings& GetGlobalLightSettings();

	/** Access global camera settings.
	  * \return Reference to the Global Camera Settings.
	  */
	KFbxGlobalCameraSettings& GetGlobalCameraSettings();

	/** Access global time settings.
	  * \return Reference to the Global Time Settings.
	  */
	KFbxGlobalTimeSettings& GetGlobalTimeSettings();

	//@}

	/** 
	  * \name Take Management
	  */
	//@{

	/**	Create a take.
	  *	\param pName Created take name.
	  *	\return \c true if not a single node, texture or material in the 
	  * hierarchy had a take with this name before.
	  *	\return \c false if at least one node, texture or material in the 
	  * hierarchy had a take with this name before.
	  *	\return In the last case, KFbxScene::GetLastErrorID() will return 
	  * \c eTAKE_ERROR.
	  *	\remarks This function will create a new take node for every node, 
	  * texture and material in the hierarchy. It may be more efficient to call
	  * KFbxTakeNodeContainer::CreateTakeNode() on the relevant nodes, textures
	  * and materials if a take only has a few of them with animation data.
	  */
	bool CreateTake(char* pName);

	/**	Remove a take.
	  *	\param pName Name of the take to remove.
	  *	\return \c true if every node, texture and material in the hierarchy 
	  * have a take with this name.
	  *	\return \c false if at least one node, texture or material in the 
	  * hierarchy don't have a take with this name.
	  *	\return In the last case, KFbxScene::GetLastErrorID() will return 
	  * \c eTAKE_ERROR.
	  * \remarks Scans the node hierarchy, the texture list and the material 
	  * list to remove all take nodes found with that name.
	  */
	bool RemoveTake(char* pName);

	/** Set the current take.
	  *	\param pName Name of the take to set.
	  *	\return \c true if every node, texture and material in the hierarchy 
	  * have a take with this name.
	  *	\return \c false if at least one node, texture or material in the 
	  * hierarchy don't have a take with this name.
	  *	\return In the last case, KFbxScene::GetLastErrorID() will return 
	  * \c eTAKE_ERROR.
	  * \remarks Scans the node hierarchy, the texture list and the material 
	  * list to set all take nodes found with that name.
	  * \remarks All nodes, textures and materials without a take node of the 
	  * requested name are set to default take node. It means that, if a node,
	  * texture or material does not have the requested take, it is assumed 
	  * that this node is not animated in this take.
	  */
	bool SetCurrentTake(char* pName);

	/**	Get current take name.
	  *	\return Current take name. 
	  * \return An empty string if the scene has not been imported from a file 
	  * and function KFbxScene::SetCurrentTake() has not been called previously 
	  * at least once.
	  */
	char* GetCurrentTakeName();

	/** Fill a string array with all existing take names.
	  *	\param pNameArray An array of string objects.
	  *	\remarks Scans the node hierarchy, the texture list and the material 
	  * list to find all existing take node names. 
	  * \remarks The array of string is cleared before scanning the node 
	  * hierarchy.
	  */
	void FillTakeNameArray(KArrayTemplate<KString*>& pNameArray);

	//@}

	/** 
	  * \name Take Information Management
	  */
	//@{

	/** Set take information about an available take.
	  *	\param pTakeInfo Take information, field KFbxTakeInfo::mName specifies 
	  * the targeted take.
	  *	\return \c true if take is found and take information set.
	  */
	bool SetTakeInfo(const KFbxTakeInfo& pTakeInfo);

	/** Get take information about an available take.
	  *	\param pTakeName Take name.
	  *	\return Pointer to take information or \c NULL if take isn't found or 
	  * has no information set.
	  */
	KFbxTakeInfo* GetTakeInfo(const KString& pTakeName);

	//@}

	/**
	  * \name Error Management
	  * The same error object is shared among instances of this class.
	  */
	//@{

	/** Retrieve error object.
	  *	\return Reference to error object.
	  */
	KError& GetError();
	
	/** Error identifiers.
	  *	\remarks Most of these are only used internally.
	  */
	typedef enum 
	{
		eTAKE_ERROR,
		eERROR_COUNT
	} EError;
	
	/** Get last error code.
	  *	\return Last error code.
	  */
	EError GetLastErrorID();

	/** Get last error string.
	  *	\return Textual description of the last error.
	  */
	char* GetLastErrorString();

	//@}

///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS

	int          GetDeformerCount(KFbxDeformer::EDeformerType pType = KFbxDeformer::eUNDEFINED);
	KFbxDeformer* GetDeformer(char* pName);
	KFbxDeformer* GetDeformer(int pIndex, KFbxDeformer::EDeformerType pType = KFbxDeformer::eUNDEFINED);
	bool          AddDeformer(KFbxDeformer* pDeformer);
	bool          RemoveDeformer(KFbxDeformer* pDeformer);
	void          ReplaceCameraSwitcher(KFbxNode* pNode);

private:

	KFbxScene(KFbxSdkManager& pManager);
	~KFbxScene();
	const KFbxScene& operator=(const KFbxScene& pScene); 

	bool FindTakeName(const KString& pTakeName);
	KFbxEntity& GetEntity();

	void ConnectMaterials();
	void ConnectTextures();
	void ConnectVideos();

	void BuildMaterialLayersDirectArray();
	void BuildTextureLayersDirectArray();
	void BuildKFbxLinkFromKFbxDeformer();
	void BuildKFbxDeformerFromKFbxLink();
	void RemoveKFbxDeformers();
	void FixInheritType(KFbxNode *pNode);

	KFbxSdkManager* mSdkManager;
	KFbxEntity* mEntity;
	KFbxNode* mCameraSwitcher;

	KArrayTemplate<KFbxControlSetPlug*> mControlSetPlugArray;
	KArrayTemplate<KFbxCharacterPose*> mCharacterPoseArray;

	KArrayTemplate<KFbxPose*> mPoseArray;

	KFbxGlobalLightSettings* mGlobalLightSettings;
	KFbxGlobalCameraSettings* mGlobalCameraSettings;
	KFbxGlobalTimeSettings* mGlobalTimeSettings;

	KString mCurrentTakeName;	
	KArrayTemplate<KFbxTakeInfo *> mTakeInfoArray;
	KFbxSceneInfo* mSceneInfo;

	KError mError;

	friend class KFbxSdkManager;
	friend class KFbxWriterFbx;
	friend class KFbxWriterFbx6;
	friend class KFbxReaderFbx;	

#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS 

};

typedef  class KFBX_DLL KArrayTemplate<KFbxTakeInfo *>			KArrayKFbxTakeInfo;
typedef KFbxScene* HKFbxScene;

} // namespace
#endif // #ifndef _KFbxScene_h


