#ifndef NX_PHYSICS_NXCAPSULECONTROLLER
#define NX_PHYSICS_NXCAPSULECONTROLLER
/*----------------------------------------------------------------------------*\
|
|						Public Interface to NovodeX Technology
|
|							     www.novodex.com
|
\*----------------------------------------------------------------------------*/

#include "NxCharacter.h"
#include "NxController.h"

/**
\brief A descriptor for a capsule character controller.

@see NxCapsuleController NxControllerDesc
*/
class NxCapsuleControllerDesc : public NxControllerDesc
	{
	public:
	/**
	\brief constructor sets to default.
	*/
	NX_INLINE								NxCapsuleControllerDesc ();
	NX_INLINE virtual						~NxCapsuleControllerDesc ();

	/**
	\brief (re)sets the structure to the default.
	*/
	NX_INLINE virtual	void				setToDefault();
	/**
	\brief returns true if the current settings are valid

	\return True if the descriptor is valid.
	*/
	NX_INLINE virtual	bool				isValid()		const;

						/**
						\brief The radius of the capsule

						<b>Default:</b> 0.0

						@see NxCapsuleController
						*/
						NxF32				radius;

						/**
						\brief The height of the controller

						<b>Default:</b> 0.0

						@see NxCapsuleController
						*/
						NxF32				height;
	};

NX_INLINE NxCapsuleControllerDesc::NxCapsuleControllerDesc () : NxControllerDesc(NX_CONTROLLER_CAPSULE)
	{
	setToDefault();
	}

NX_INLINE NxCapsuleControllerDesc::~NxCapsuleControllerDesc()
	{
	}

NX_INLINE void NxCapsuleControllerDesc::setToDefault()
	{
	NxControllerDesc::setToDefault();
	radius = height = 0.0f;
	}

NX_INLINE bool NxCapsuleControllerDesc::isValid() const
	{
	if(!NxControllerDesc::isValid())	return false;
	if(radius<=0.0f)					return false;
	if(height<=0.0f)					return false;
	return true;
	}
/**
\brief A capsule character controller.

	The capsule is defined as a position, a vertical height, and a radius.
	The height is the same height as for NxCapsuleShape objects, i.e. the distance
	between the two sphere centers at the end of the capsule. In other words:

	p = pos (returned by controller)
	h = height
	r = radius

	p = center of capsule
	top sphere center = p.y + h*0.5
	bottom sphere center = p.y - h*0.5
	top capsule point = p.y + h*0.5 + r
	bottom capsule point = p.y - h*0.5 - r
*/
class NxCapsuleController : public NxController
	{
	protected:
	NX_INLINE					NxCapsuleController()	{}
	virtual						~NxCapsuleController()	{}

	public:

	/**
	\brief Gets controller's radius.

	\return The radius of the controller.

	@see NxCapsuleControllerDesc.radius setRadius()
	*/
	virtual		NxF32			getRadius() const = 0;

	/**
	\brief Sets controller's radius.

	\warning this doesn't check for collisions.

	\param[in] radius The new radius for the controller.
	\return Currently always true.

	@see NxCapsuleControllerDesc.radius getRadius()
	*/
	virtual		bool			setRadius(NxF32 radius) = 0;

	/**
	\brief Gets controller's height.

	\return The height of the capsule controller.

	@see NxCapsuleControllerDesc.height setHeight()
	*/
	virtual		NxF32			getHeight() const = 0;

	/**
	\brief Resets controller's height.

	\warning this doesn't check for collisions.

	\param[in] height The new height for the controller.
	\return Currently always true.

	@see NxCapsuleControllerDesc.height getHeight()
	*/
	virtual		bool			setHeight(NxF32 height) = 0;

	/**
	\brief Sets the step height/offset for the controller.

	\param[in] offset The new step offset for the controller.

	@see NxControllerDesc.stepOffset NxController.setStepOffset()
	*/
	virtual	    void			setStepOffset(const float offset) =0;
	};

#endif
