#ifndef NX_PHYSICS_NXCONTROLLER
#define NX_PHYSICS_NXCONTROLLER
/*----------------------------------------------------------------------------*\
|
|						Public Interface to NovodeX Technology
|
|							     www.novodex.com
|
\*----------------------------------------------------------------------------*/

#include "NxCharacter.h"
#include "Nxp.h"
#include "NxExtended.h"

/**
\brief The type of controller, eg box, sphere or capsule.
*/
enum NxControllerType
	{
	/**
	\brief A box controller.

	@see NxBoxController NxBoxControllerDesc
	*/
	NX_CONTROLLER_BOX,

	/**
	\brief A capsule controller

	@see NxCapsuleController NxCapsuleControllerDesc
	*/
	NX_CONTROLLER_CAPSULE,

	NX_CONTROLLER_FORCE_DWORD = 0x7fffffff
	};

#include "NxTriangleMesh.h"

class NxShape;
class NxScene;
class NxController;
class NxActor;

/**
\brief specifies which sides a character is colliding with.
*/
enum NxControllerFlag
	{
	NXCC_COLLISION_SIDES	= (1<<0),	//!< Character is colliding to the sides.
	NXCC_COLLISION_UP		= (1<<1),	//!< Character has collision above.
	NXCC_COLLISION_DOWN		= (1<<2),	//!< Character has collision below.
	};

/**
\brief Controller actions.
*/
enum NxControllerAction
	{
	NX_ACTION_NONE,				//!< Don't apply forces to touched actor
	NX_ACTION_PUSH,				//!< Automatically compute & apply forces to touched actor (push)
	};

/**
\brief Describes a controller shape hit. Passed to onShapeHit()

@see NxUserControllerHitReport.onShapeHit()
*/
struct NxControllerShapeHit
	{
	NxController*	controller;		//!< Current controller
	NxShape*		shape;			//!< Touched shape
	NxExtendedVec3	worldPos;		//!< Contact position in world space
	NxVec3			worldNormal;	//!< Contact normal in world space
	NxU32			id;				//!< Feature identifier (e.g. triangle index for meshes)
	NxVec3			dir;			//!< Motion direction
	NxF32			length;			//!< Motion length
	};

/**
\brief Describe a controller hit. Passed to onControllerHit().

@see NxUserControllerHitReport.onControllerHit()
*/
struct NxControllersHit
	{
	NxController*	controller;		//!< Current controller
	NxController*	other;			//!< Touched controller
	};

/**
\brief User callback class for character controller events.

@see NxControllerDesc.callback
*/
class NxUserControllerHitReport
	{
	public:

	/**
	\brief Called when current controller hits a shape.

	\param[in] hit Provides information about the contact with the touched shape.
	\return Specifies if to automatically apply forces to the touched actor.

	@see NxControllerAction NxControllerShapeHit
	*/
	virtual NxControllerAction  onShapeHit(const NxControllerShapeHit& hit) = 0;

	/**
	\brief Called when current controller hits another controller.

	\param[in] hit Provides information about the touched controller.
	\return Specifies if to automatically apply forces to the touched controller.

	@see NxControllerAction NxControllersHit
	*/
	virtual NxControllerAction  onControllerHit(const NxControllersHit& hit) = 0;
	};

/**
\brief Descriptor class for a character controller.

@see NxBoxController NxCapsuleController
*/
class NxControllerDesc
	{
	protected:
	
	/**
	\brief Not used.
	*/
	NxU32						version;
	const NxControllerType		type;		//!< The type of the controller. This gets set by the derived class' ctor, the user should not have to change it.
	
	/**
	\brief constructor sets to default.
	*/
	NX_INLINE										NxControllerDesc(NxControllerType);
	NX_INLINE virtual								~NxControllerDesc();
	public:

	/**
	\brief (re)sets the structure to the default.
	*/
	NX_INLINE virtual	void						setToDefault();
	/**
	\brief returns true if the current settings are valid

	\return True if the descriptor is valid.
	*/
	NX_INLINE virtual	bool						isValid()		const;

	/**
	\brief Not used.

	\return The version number.
	*/
	NX_INLINE			NxU32						getVersion()	const	{ return version;	}

	/**
	\brief Returns the character controller type

	\return The controllers type.

	@see NxControllerType NxCapsuleControllerDesc NxBoxControllerDesc
	*/
	NX_INLINE			NxControllerType			getType()		const	{ return type;		}

						/**
						\brief The position of the character

						<b>Default:</b> Zero
						*/
						NxExtendedVec3				position;

						/**
						\brief Specifies the 'up' direction
						
						In order to provide stepping functionality the SDK must be informed about the up direction.

						\li NX_X => (1, 0, 0)
						\li NX_Y => (0, 1, 0)
						\li NX_Z => (0, 0, 1)

						<b>Default:</b> NX_Y

						@see NxHeightFieldAxis
						*/
						NxHeightFieldAxis			upDirection;

						/**
						\brief The maximum slope which the character can walk up.

						In general it is desirable to limit where the character can walk, in particular it is unrealistic
						for the character to be able to climb arbitary slopes.

						The limit is expressed as the cosine of desired limit angle. A value of 0 disables this feature.

						<b>Default:</b> 0.707

						@see upDirection
						*/
						NxF32						slopeLimit;

						/**
						\brief The skin width used by the controller.

						A "skin" around the controller is necessary to avoid numerical precision issues.

						This is dependant on the scale of the users world, but should be a small, positive 
						non zero value.

						<b>Default:</b> 0.1
						*/
						NxF32						skinWidth;

						/**
						\brief Defines the maximum height of an obstacle which the character can climb.

						A small value will mean that the character gets stuck and cannot walk up stairs etc, 
						a value which is too large will mean that the character can climb over unrealistically 
						high obstacles.

						<b>Default:</b> 0.5

						@see upDirection 
						*/
						NxF32						stepOffset;

						/**
						\brief Specifies a user callback interface.

						This callback interface is called when the character collides with shapes and other characters.

						Setting this to NULL disables callbacks.

						<b>Default:</b> NULL

						@see NxUserControllerReport
						*/
						NxUserControllerHitReport*	callback;

						/**
						\brief User specified data associated with the controller.

						<b>Default:</b> NULL
						*/
						void*						userData;
	};

NX_INLINE NxControllerDesc::NxControllerDesc(NxControllerType t) : type(t)
	{
	setToDefault();
	}

NX_INLINE NxControllerDesc::~NxControllerDesc()
	{
	}

NX_INLINE void NxControllerDesc::setToDefault()
	{
	version		= 0;
	upDirection	= NX_Y;
	slopeLimit	= 0.707f;
	skinWidth	= 0.1f;
	stepOffset	= 0.5f;
	callback	= NULL;
	userData	= NULL;
	position.zero();
	}

NX_INLINE bool NxControllerDesc::isValid() const
	{
	return true;
	}


/**
\brief Base class for character controllers.

@see NxCapsuleController NxBoxController
*/
class NxController
	{
	protected:
	NX_INLINE							NxController() : appData(NULL)	{}
	virtual								~NxController()					{}

	public:

	/**
	\brief Moves the character using a "collide-and-slide" algorithm.

	\param disp	a displacement vector
	\param activeGroups	a filtering mask for collision groups. If a bit is set, corresponding group is active.
	\param minDist the minimum travelled distance to consider. If travelled distance is smaller, the character doesn't move. 
	This is used to stop the recursive motion algorithm when remaining distance to travel is small.
	\param collisionFlags returned collision flags, collection of ::NxControllerFlag
	\param sharpness to prevent sudden height changes due to the autostep feature, the motion can be smoothed using a feedback filter.
	This coefficient defines the amount of smoothing. The smaller, the smoother. (1.0 means no smoothing).
	*/
	virtual		void					move(const NxVec3& disp, NxU32 activeGroups, NxF32 minDist, NxU32& collisionFlags, NxF32 sharpness=1.0f)	= 0;

	/**
	\brief Resets controller's position.

	\warning this is a 'teleport' function, it doesn't check for collisions.

	To move the character under normal conditions use the #move() function.

	\param[in] position The new positon for the controller.
	\return Currently always returns true.

	@see NxControllerDesc.position getPosition() move()
	*/
	virtual		bool					setPosition(const NxExtendedVec3& position) = 0;
	/**
	\brief Retrieve the raw position of the controller.

	\return The controllers position

	@see NxControllerDesc.position setPositon()
	*/
	virtual		const NxExtendedVec3&	getPosition()			const	= 0;
	/**
	\brief Retrieve the filtered position of the controller.

	The filtered position is a smoothed version of the position. Smoothing of the position is useful
	to prevent sudden changes in position when the character steps over an obstacle. See the sharpness parameter
	of #move()

	\return The filtered position.

	@see move() getPosition()
	*/
	virtual		const NxExtendedVec3&	getFilteredPosition()	const	= 0;

	/**
	\brief Currently returns the position of the character.

	\return The characters position.

	@see getPosition()
	*/
	virtual		const NxExtendedVec3&	getDebugPosition()		const	= 0;

	/**
	\brief Get the actor associated with this controller.

	\return the actor associated with the controller.
	*/
	virtual		NxActor*				getActor()				const	= 0;

	/**
	\brief The step height.

	\param[in] offset The new step offset for the controller.

	@see NxControllerDesc.stepOffset
	*/
	virtual	    void					setStepOffset(const float offset) =0;

	/**
	\brief Enable/Disable collisions for this controller/actor.

	Disables collision between the scene the the kinematic actor associated with the controller.

	\param[in] enabled True to enable collision with the controller.
	*/
	virtual		void					setCollision(bool enabled)		= 0;

	/**
	\brief Returns the application data associated with this controller.

	\return The user pointer associated with the controller.

	/**
	\brief Returns the application data associated with this controller.

	\return The user pointer associated with the controller.

	@see NxControllerDesc.userData
	*/
	NX_INLINE	void*					getAppData()	{ return appData;	}

	protected:
				void*					appData;
	};

#endif
