#ifndef NX_PHYSICS_NX_PHYSICS_SDK
#define NX_PHYSICS_NX_PHYSICS_SDK
/*----------------------------------------------------------------------------*\
|
|						Public Interface to NovodeX Technology
|
|							     www.novodex.com
|
\*----------------------------------------------------------------------------*/
/** \addtogroup physics
  @{
*/

#include "Nxp.h"
#include "NxInterface.h"
#include "NxUserAllocator.h"

class NxScene;
class NxSceneDesc;
class NxUserDebugRenderer;
class NxTriangleMesh;
class NxTriangleMeshDesc;
class NxConvexMesh;
class NxConvexMeshDesc;
class NxUserOutputStream;
class NxUserAllocator;
class NxActor;
class NxJoint;
class NxStream;
class NxFoundationSDK;
class NxCCDSkeleton;			//this class doesn't actually exist.
class NxSimpleTriangleMesh;


enum NxCookingValue
	{
	/**
	Version numbers follow this format:

		Version = 16bit|16bit

	The high part is increased each time the format changes so much that
	pre-cooked files become incompatible with the system (and hence must
	be re-cooked)

	The low part is increased each time the format changes but the code
	can still read old files. You don't need to re-cook the data in that
	case, unless you want to make sure cooked files are optimal.
	*/
	NX_COOKING_CONVEX_VERSION_PC,
	NX_COOKING_MESH_VERSION_PC,
	NX_COOKING_CONVEX_VERSION_XENON,
	NX_COOKING_MESH_VERSION_XENON,
	};

/**
\brief Reads an internal value.

\param[in] cookValue See #NxCookingValue
*/
NX_C_EXPORT NXP_DLL_EXPORT NxU32 NxGetValue(NxCookingValue cookValue);

/**
\brief Descriptor class for NxPhysicsSDK, primarily used for defining PhysX hardware limits
       for data shared between scenes.
*/


class NxPhysicsSDKDesc
{
public:
	NxU32 hwPageSize;		//!< size of hardware mesh pages. Currently only the value 65536 is supported.
	NxU32 hwPageMax;		//!< maximum number of hardware pages supported concurrently on hardware
	NxU32 hwConvexMax;		//!< maximum number of convex meshes which will be resident on hardware.

	/**
	\brief (re)sets the structure to the default.	
	*/

	NX_INLINE void setToDefault()
	{
		hwPageSize = 65536;
		hwConvexMax = 2048;
		hwPageMax = 256;
	}

	/**
	\brief Returns true if the descriptor is valid.

	\return return true if the current settings are valid
	*/

	NX_INLINE bool isValid() const
	{
		return hwPageSize == 65536;
	}

	NxPhysicsSDKDesc()
	{
		setToDefault();
	}
};


/**
	\brief Abstract singleton factory class used for instancing objects in the Physics SDK.

	In addition you can use NxPhysicsSDK to set global parameters which will effect all scenes,
	create triangle meshes and CCD skeletons.
	
	You can get an instance of this class by calling NxCreatePhysicsSDK().

	@see NxCreatePhysicsSDK() NxScene NxParameter NxTriangleMesh NxConvexMesh NxPhysicsSDK.createCCDSkeleton()
*/

class NxPhysicsSDK
	{
	protected:
			NxPhysicsSDK()	{}
	virtual	~NxPhysicsSDK()	{}

	public:

	/**
	\brief Destroys the instance it is called on.

	Use this release method to destroy an instance of this class. Be sure
	to not keep a reference to this object after calling release.

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : Yes
	\li PS3  : Yes
	\li XB360: Yes

	@see NxCreatePhysicsSDK()
	*/
	virtual	void release() = 0;

	/**
	\brief Function that lets you set global simulation parameters.

	Returns false if the value passed is out of range for usage specified by the enum.

	<b>Sleeping:</b> Does <b>NOT</b> wake any actors which may be affected.

	\param[in] paramEnum Parameter to set. See #NxParameter
	\param[in] paramValue The value to set, see #NxParameter for allowable values.
	\return False if the parameter is out of range.

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : Partial
	\li PS3  : Yes
	\li XB360: Yes

	@see NxParameter getParameter
	*/
	virtual bool setParameter(NxParameter paramEnum, NxReal paramValue) = 0;

	/**
	\brief Function that lets you query global simulation parameters.

	\param[in] paramEnum The Parameter to retrieve.
	\return The value of the parameter.

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : Partial
	\li PS3  : Yes
	\li XB360: Yes

	@see setParameter NxParameter
	*/
	virtual NxReal getParameter(NxParameter paramEnum) const = 0;

	/**
	\brief Creates a scene.

	The scene can then create its contained entities.

	\param[in] sceneDesc Scene descriptor. See #NxSceneDesc
	\return The new scene object.

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : Yes
	\li PS3  : Yes
	\li XB360: Yes

	@see NxScene NxSceneDesc releaseScene()
	*/
	virtual NxScene* createScene(const NxSceneDesc& sceneDesc) = 0;

	/**
	\brief Deletes the instance passed.

	Be sure	to not keep a reference to this object after calling release.

	\param[in] scene The scene to release.

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : Yes
	\li PS3  : Yes
	\li XB360: Yes

	@see NxScene createScene()
	*/
	virtual void releaseScene(NxScene& scene) = 0;

	/**
	\brief Gets number of created scenes.

	\return The numbewr of scenes created.

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : Yes
	\li PS3  : Yes
	\li XB360: Yes

	@see getScene()
	*/
	virtual NxU32 getNbScenes()			const	= 0;

	/**
	\brief Retrieves pointer to created scenes.

	\param[in] i The index for the scene.
	\return The scene at index i.

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : Yes
	\li PS3  : Yes
	\li XB360: Yes

	@see getNbScenes() NxScene
	*/
	virtual NxScene* getScene(NxU32 i)			= 0;

	/**
	\brief Creates a triangle mesh object.
	
	This can then be instanced into #NxTriangleMeshShape objects.

	\param[in] stream The triangle mesh stream.
	\return The new triangle mesh.

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : Yes
	\li PS3  : Yes
	\li XB360: Yes

	@see NxTriangleMesh NxStream releaseTriangleMesh() createConvexMesh()
	*/
	virtual NxTriangleMesh* createTriangleMesh(const NxStream& stream) = 0;

	/**
	\brief Destroys the instance passed.

	Be sure	to not keep a reference to this object after calling release.
	Do not release the triangle mesh before all its instances are released first!

	\param[in] mesh Triangle mesh to release.

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : Yes
	\li PS3  : Yes
	\li XB360: Yes

	@see createTriangleMesh NxTriangleMesh
	*/
	virtual	void	releaseTriangleMesh(NxTriangleMesh& mesh) = 0;


	/**
	\brief Creates a CCD Skeleton mesh object.  

	CCD is performed with a skeleton(mesh) embedded within the object, this can be simpler than the geometry 
	used for discrete collision detection. 

	NxShape::setCCDSkeleton() should be used to associate a CCD skeleton with shapes.

	\note that stray vertices are permitted (in other words, vertices not referenced by any triangles), 
	but degenerate triangles (triangles that have a triangle index 2 or 3 times) are not. Stray vertices 
	are supported so that you can use a single vertex to do a raycast style CCD test. 

	\note The CCD skeleton should be scaled so that it is smaller than the geometry it is embedded within.
	This allows regular discrete collision detection to handle resting contact. Making the CCD skeleton too
	large in comparison to the shape it is embedded within can cause eratic behavior.
	
	\note CCDSkeletons currently must contain at most 64 vertices!

	<b>Limitations</b>
	<p>
	\li 64 vertex limit on CCD Skeletons (restriction probably lifted in a future version)
	\li Doesnt work when the static shape is a NxSphereShape, NxCapsuleShape, NxPlaneShape,NxBoxShape.
	</p>

	<h3>Visulizations:</h3>
	\li #NX_VISUALIZE_COLLISION_CCD
	\li #NX_VISUALIZE_COLLISION_SKELETONS

	\param[in] mesh The traingle mesh from which to create the CCD skeleton.
	\return The new CCD skeleton.

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : No
	\li PS3  : Yes
	\li XB360: Yes

	@see NxSimpleTriangleMesh releaseCCDSkeleton() NxShape.setCCDSkeleton()
	*/
	virtual NxCCDSkeleton * createCCDSkeleton(const NxSimpleTriangleMesh& mesh) = 0;

	/**
	\brief Destroys the instance passed.

	Be sure	to not keep a reference to this object after calling release.
	Do not release the object before all its users are released first!

	\param[in] skel The CCD Skeleton to release.

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : No
	\li PS3  : Yes
	\li XB360: Yes

	@see createCCDSkeleton() NxShape.setCCDSkeleton()
	*/
	virtual	void	releaseCCDSkeleton(NxCCDSkeleton& skel) = 0;


	/**
	\brief Creates a convex mesh object.
	
	This can then be instanced into #NxConvexShape objects.

	\param[in] mesh The stream to load the convex mesh from.
	\return The new convex mesh.

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : Yes
	\li PS3  : Yes
	\li XB360: Yes

	@see releaseConvexMesh() NxConvexMesh NxStream createTriangleMesh() NxConvexShape
	*/
	virtual NxConvexMesh* createConvexMesh(const NxStream& mesh) = 0;

	/**
	\brief Destroys the instance passed.

	Be sure	to not keep a reference to this object after calling release.
	Do not release the convex mesh before all its instances are released first!

	\param[in] mesh The convex mesh to release.

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : Yes
	\li PS3  : Yes
	\li XB360: Yes

	@see createConvexMesh() NxConvexMesh NxConvexShape
	*/
	virtual	void	releaseConvexMesh(NxConvexMesh& mesh) = 0;


#if NX_USE_FLUID_API

	/**
	Not available in the current release.
	Fluids can be assigned to fluid groups by using NxFluid::setFluidGroup(). This method sets
	fluid actor group pairs to configure collision and collision reports between
	fluids and actors using the flags:

	NX_IGNORE_PAIR
	NX_NOTIFY_ON_COLLISION

	\param[in] actorGroup The actor group to pair the fluid group with.
	\param[in] fluidGroup The fluid group.
	\param[in] flags Flags to control the pairing. See #NxContactPairFlag

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  :
	\li PS3  : No
	\li XB360: 

	@see NxContactPairFlag.
	*/
	virtual void setFluidGroupPairFlags(NxActorGroup actorGroup, NxFluidGroup fluidGroup, NxU32 flags) = 0;

	/**
	Not available in the current release.
	This reads the value set with the above call.

	\param[in] actorGroup The actor group.
	\param[in] fluidGroup The fluid group.
	\return The fluid pair flags.
	
	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  :
	\li PS3  : No
	\li XB360: 
	*/
	virtual NxU32 getFluidGroupPairFlags(NxActorGroup actorGroup, NxFluidGroup fluidGroup) const = 0;

#endif

	/**
	\brief Reports the internal API version number of the SDK

	\return The inernal API version information.

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : Yes
	\li PS3  : Yes
	\li XB360: Yes

	*/
	virtual NxU32 getInternalVersion(NxU32 &apiRev,NxU32 &descRev,NxU32 &branchId)			const	= 0;
	virtual NxInterface *getInterface(NxInterfaceType type,int versionNumber) = 0;

    /** 
	\brief Reports the available revision of the PhysX Hardware

	\return 0 if there is no hardware present in the machine, 1 for the PhysX Athena revision 1.0 card. 
	*/

    virtual NxHWVersion getHWVersion() const = 0;

	/**
	\brief retrieves the FoundationSDK instance.
	\return The foundation SDK refrence.

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : Yes
	\li PS3  : Yes
	\li XB360: Yes
	*/
	virtual NxFoundationSDK& getFoundationSDK() const = 0;

	};

#endif


//AGCOPYRIGHTBEGIN
///////////////////////////////////////////////////////////////////////////
// Copyright  2005 AGEIA Technologies.
// All rights reserved. www.ageia.com
///////////////////////////////////////////////////////////////////////////
//AGCOPYRIGHTEND

