#ifndef NX_COLLISION_NXUSERCONTACTREPORT
#define NX_COLLISION_NXUSERCONTACTREPORT
/*----------------------------------------------------------------------------*\
|
|						Public Interface to NovodeX Technology
|
|							     www.novodex.com

\*----------------------------------------------------------------------------*/
/** \addtogroup physics
  @{
*/

#include "Nxp.h"
#include "NxShape.h"
#include "NxContactStreamIterator.h"

class NxActor;

#if NX_USE_FLUID_API
class NxFluid;
#endif

/**
\brief Contact pair flags.

@see NxUserContactReport.onContactNotify()
*/
enum NxContactPairFlag
	{
	NX_IGNORE_PAIR				= (1<<0),	//!< Disable contact generation for this pair

	NX_NOTIFY_ON_START_TOUCH	= (1<<1),	//!< Pair callback will be called when the pair starts to be in contact
	NX_NOTIFY_ON_END_TOUCH		= (1<<2),	//!< Pair callback will be called when the pair stops to be in contact
	NX_NOTIFY_ON_TOUCH			= (1<<3),	//!< Pair callback will keep getting called while the pair is in contact
	NX_NOTIFY_ON_IMPACT			= (1<<4),	//!< [Not yet implemented] pair callback will be called when it may be appropriate for the pair to play an impact sound
	NX_NOTIFY_ON_ROLL			= (1<<5),	//!< [Not yet implemented] pair callback will be called when the pair is in contact and rolling.
	NX_NOTIFY_ON_SLIDE			= (1<<6),	//!< [Not yet implemented] pair callback will be called when the pair is in contact and sliding (and not rolling).

	NX_NOTIFY_ALL				= (NX_NOTIFY_ON_START_TOUCH|NX_NOTIFY_ON_END_TOUCH|NX_NOTIFY_ON_TOUCH|NX_NOTIFY_ON_IMPACT|NX_NOTIFY_ON_ROLL|NX_NOTIFY_ON_SLIDE)
	};

/**
\brief An instance of this class is passed to NxUserContactReport::onContactNotify().
It contains a contact stream which may be parsed using the class NxContactStreamIterator.

@see NxUserContactReport.onContactNotify()
*/
class NxContactPair
	{
	public:
	NX_INLINE	NxContactPair() : stream(NULL)	{}

	/**
	\brief the two actors that make up the pair.

	@see NxActor
	*/
	NxActor*				actors[2];

	/**
	\brief use this to create stream iter. See #NxContactStreamIterator.

	@see NxConstContactStream
	*/
	NxConstContactStream	stream;

	/**
	\brief the total contact normal force that was applied for this pair, to maintain nonpenetration constraints.
	*/
	NxVec3					sumNormalForce;

	/**
	\brief the total tangential force that was applied for this pair.
	*/
	NxVec3					sumFrictionForce;
	};

/**
\brief The user needs to implement this interface class in order to be notified when
certain contact events occur.

Once you pass an instance of this class to #NxScene::setUserContactReport(), 
its  #onContactNotify() method will be called for each pair of actors which comes into contact, 
for which this behavior was enabled.

You request which events are reported using NxScene::setActorPairFlags(), 
#NxScene::setShapePairFlags(), or #NxScene::getActorGroupPairFlags()

 <b>Threading:</b> It is not necessary to make this class thread safe as it will only be called in the context of the
 user thread.

<h3>Example</h3>

\include NxUserContactReport_Example.cpp

<h3>Visulizations:</h3>
\li #NX_VISUALIZE_CONTACT_POINT
\li #NX_VISUALIZE_CONTACT_NORMAL
\li #NX_VISUALIZE_CONTACT_ERROR
\li #NX_VISUALIZE_CONTACT_FORCE

@see NxScene.setUserContactReport() NxScene.getUserNotify()
*/
class NxUserContactReport
	{
	public:
 	/**
	Called for a pair in contact. The events parameter is a combination of:

	<ul>
	<li>NX_NOTIFY_ON_START_TOUCH,</li>
	<li>NX_NOTIFY_ON_END_TOUCH,</li>
	<li>NX_NOTIFY_ON_TOUCH,</li>
	<li>NX_NOTIFY_ON_IMPACT,	//unimplemented!</li>
	<li>NX_NOTIFY_ON_ROLL,		//unimplemented!</li>
	<li>NX_NOTIFY_ON_SLIDE,		//unimplemented!</li>
	</ul>

	See the documentation of #NxContactPairFlag for an explanation of each. You request which events 
	are reported using #NxScene::setActorPairFlags() or 
	#NxScene::setActorGroupPairFlags(). Do not keep a reference to the passed object, as it will
	be invalid after this function returns.

	\param[in] pair The contact pair we are being notified of. See #NxContactPair.
	\param[in] events Flags raised due to the contact. See #NxContactPairFlag.

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  :
	\li PS3  : Yes
	\li XB360: Yes

	@see NxContactPair NxContactPairFlag
	*/
	virtual void  onContactNotify(NxContactPair& pair, NxU32 events) = 0;
	};

#if NX_USE_FLUID_API

enum NxFluidContactPairFlag
	{
	NX_FPF_IGNORE_PAIR				= (1<<0),	//!< Not available in the current release. Disable contact generation for this pair
	NX_FPF_NOTIFY_ON_COLLISION		= (1<<1),	//!< Not available in the current release. Pair callback will be called when the pair collides
	};

/**
\warning <b>Not available in the current release. </b>
\brief An instance of this class is passed to NxUserFluidContactReport::onContactNotify().

It contains a contact stream which may be parsed using the class NxContactStreamIterator.
*/
class NxFluidContactPair
	{
	public:
	NX_INLINE	NxFluidContactPair() : stream(NULL)	{}

	NxFluid*				fluid;
	NxActor*				actor;
	NxConstContactStream	stream;
	};

/**
\brief The user needs to implement this interface class in order to be notified when
certain fluid actor contact events occur.

Once you pass an instance of this class to NxScene::setUserFluidContactReport(), 
its  onContactNotify() method will be called for each pair of actors which comes 
into contact, for which this behavior was enabled.

You request which events are reported using NxPhysicsSDK::getFluidGroupPairFlags()

 <b>Threading:</b> It is not necessary to make this class thread safe as it will only be called in the context of the
 user thread.
*/
class NxUserFluidContactReport
	{
	public:
 	/**
	\brief Called for a pair in contact.
	
	The events parameter is at the moment always NX_NOTIFY_ON_COLLISION:

	NX_NOTIFY_ON_COLLISION,

	See the documentation of NxFluidContactPairFlag for an explanation of each. You request which events 
	are reported using NxScene::setActorPairFlags(), NxScene::setShapePairFlags(), or 
	NxPhysicsSDK::getActorGroupPairFlags(). Do not keep a reference to the passed object, as it will
	be invalid after this function returns.

	\param[in] pair Fluid contact pair.
	\param[in] events See #NxContactPairFlag.

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  :
	\li PS3  : No
	\li XB360: 
	*/
	virtual void  onContactNotify(NxFluidContactPair& pair, NxU32 events) = 0;
	};

#endif

/**
\brief The user needs to implement this interface class in order to be notified when trigger events
occur. 

Once you pass an instance of this class to #NxScene::setUserTriggerReport(), shapes
which have been marked as triggers using NxShape::setFlag(NX_TRIGGER_ENABLE,true) will call the
#onTrigger() method when their trigger status changes.

<b>Threading:</b> It is not necessary to make this class thread safe as it will only be called in the context of the
user thread.

Example:

\include NxUserTriggerReport_Usage.cpp

<h3>Visualizations</h3>
\li NX_VISUALIZE_COLLISION_SHAPES

@see NxScene.setUserTriggerReport() NxScene.getUserTriggerReport() NxShapeFlag NxShape.setFlag()
*/
class NxUserTriggerReport
	{
	public:
	/**
	\brief Called when a trigger shape reports a trigger event.
	
	\param[in] triggerShape is the shape that has been marked as a trigger.
	\param[in] otherShape is the shape causing the trigger event.
	\param[in] status is the type of trigger event.

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  :
	\li PS3  : Yes
	\li XB360: Yes

	@see NxTriggerFlag
	*/
	virtual void onTrigger(NxShape& triggerShape, NxShape& otherShape, NxTriggerFlag status) = 0;
	};

/** @} */
#endif


//AGCOPYRIGHTBEGIN
///////////////////////////////////////////////////////////////////////////
// Copyright  2005 AGEIA Technologies.
// All rights reserved. www.ageia.com
///////////////////////////////////////////////////////////////////////////
//AGCOPYRIGHTEND

