#ifndef NX_FLUIDS_NXFLUIDEMITTERDESC
#define NX_FLUIDS_NXFLUIDEMITTERDESC
/** \addtogroup fluids
  @{
*/
/*----------------------------------------------------------------------------*\
|
|						Public Interface to NovodeX Technology
|
|							     www.novodex.com
|
\*----------------------------------------------------------------------------*/

/**
\brief Flags which control the behavior of fluid emitters.

\warning <b>Preliminary API, subject to change</b>

@see NxFluidEmitter
*/
enum NxFluidEmitterFlag
	{
	/**
	\brief Flags whether the emitter should be visualized for debugging or not.
	*/
	NX_FEF_VISUALIZATION		= (1<<0),

	/**
	\brief The emitter may reference an NxActor instance in order to maintain a relative coordinate 
	frame.

	If this NxActor instance is released, the NX_FEF_BROKEN_ACTOR_REF flag is set.
	*/
	NX_FEF_BROKEN_ACTOR_REF		= (1<<1),
	
	/**
	\brief This flag specifies whether the emission should cause a force on 
	the actor that the emitter is attached to.
	*/
	NX_FEF_FORCE_ON_ACTOR		= (1<<2),
	
	/**
	\brief If set, the velocity of the emitter is added to the emitted particle velocity.
	
	This is the default behaviour.
	*/
	NX_FEF_ADD_ACTOR_VELOCITY	= (1<<3),
	
	/**
	\brief Flag to start and stop the emission. On default the emission is enabled.
	*/
	NX_FEF_ENABLED				= (1<<4),
	};

/**
\brief Flags to specify the shape of the area of emission.

Exactly one flag should be set at any time.

\warning <b>Preliminary API, subject to change</b>
*/
enum NxEmitterShape
	{
	NX_FE_RECTANGULAR		= (1<<0),
	NX_FE_ELLIPSE			= (1<<1)
	};

/**
\brief Flags to specify the emitter's type of operation.
Exactly one flag should be set at any time.

\warning <b>Preliminary API, subject to change</b>

@see NxFluidEmitter
*/
enum NxEmitterType
	{
	NX_FE_CONSTANT_PRESSURE		= (1<<0),
	NX_FE_CONSTANT_FLOW_RATE	= (1<<1)
	};

#include "fluids/NxFluidEmitter.h"

/**
\brief Descriptor for NxFluidEmitter class.  Used for saving and loading the emitter state.

\warning <b>Preliminary API, subject to change</b>
*/
class NxFluidEmitterDesc
	{
	public:

	/**
	\brief The emitter's pose relative to the frame actor's coordinate frame, or the world, if the frame 
	actor is set to NULL.
	
	The third axis of the orientation frame sets the flow direction.

	\warning <b>Preliminary API, subject to change</b>
	*/
    NxMat34					relPose;
	
	/**
	\brief A pointer to the NxActor to which the emitter is attached to.
	
	If this pointer is set to NULL, the emitter is attached to the world frame. The actor 
	must be in the same scene as the emitter. 

	\warning <b>Preliminary API, subject to change</b>
	*/
	NxActor*				frameActor;

	/**
	\brief The emitter's mode of operation.

	Either the simulation enforces constant pressure or constant flow rate at the emission site, 
	given the velocity of emitted particles.

	\warning <b>Preliminary API, subject to change</b>

	@see NxEmitterType
	*/
	NxU32					type;

	/**
	\brief The maximum number of particles which are emitted from this emitter.
	
	If the total number of particles in the fluid already hit the maxParticles parameter of the fluid, 
	this maximal values can't be reached.
	
	If set to 0, the number of emitted particles is unrestricted.

	\warning <b>Preliminary API, subject to change</b>
	*/
	NxU32					maxParticles;

	/**
	\brief The emitter's shape can either be rectangular or elliptical.

	\warning <b>Preliminary API, subject to change</b>

	@see NxEmitterShape
	*/
	NxU32					shape;

	/**
	\brief The sizes of the emitter in the directions of the first and the second axis of its orientation 
	frame (relPose).

	The dimensions are actually the radii of the size.

	\warning <b>Preliminary API, subject to change</b>

	*/
	NxReal					dimensionX;
	NxReal					dimensionY;

	/**
	\brief Random vector with values for each axis direction of the emitter orientation.
	
	The values have to be positive and describe the maximal random particle displacement in each dimension.
	
	The z value describes the randomization in emission direction. The emission direction 
	is specified by the third orientation axis of relPose.

	\warning <b>Preliminary API, subject to change</b>
	*/
    NxVec3					randomPos;

	/**
	\brief Random angle deviation from emission direction.
	
	The emission direction is specified by the third orientation axis of relPose.
	
	<b>Unit:</b> Radians

	\warning <b>Preliminary API, subject to change</b>
	*/
	NxReal					randomAngle;

	/**
	\brief The velocity magnitude of the emitted fluid particles.

	\warning <b>Preliminary API, subject to change</b>
	*/
	NxReal					fluidVelocityMagnitude;

	/**
	\brief The rate specifies how many particles are emitted per second.
	
	The rate is only considered in the simulation if the type is set to NX_FE_CONSTANT_FLOW_RATE.

	\warning <b>Preliminary API, subject to change</b>

	@see NxEmitterType
	*/
	NxReal					rate;
	
	/**
	\brief This specifies the time in seconds an emitted particle lives.

	If set to 0, each particle will live until it collides with a drain.

	\warning <b>Preliminary API, subject to change</b>
	*/
	NxReal					particleLifetime;

	/**
	\brief A combination of NxFluidEmitterFlags.

	\warning <b>Preliminary API, subject to change</b>

	@see NxFluidEmitterFlags
	*/
	NxU32					flags;

	void*					userData;		//!< Will be copied to NxShape::userData.
	const char*				name;			//!< Possible debug name. The string is not copied by the SDK, only the pointer is stored.

	NX_INLINE ~NxFluidEmitterDesc();
	/**
	\brief (Re)sets the structure to the default.	

	\warning <b>Preliminary API, subject to change</b>
	*/
	NX_INLINE void setToDefault();
	/**
	\brief Returns true if the current settings are valid

	\warning <b>Preliminary API, subject to change</b>
	*/
	NX_INLINE bool isValid() const;

	/**
	\brief Constructor sets to default.

	\warning <b>Preliminary API, subject to change</b>
	*/
	NX_INLINE	NxFluidEmitterDesc();
	};


NX_INLINE NxFluidEmitterDesc::NxFluidEmitterDesc()
	{
	setToDefault();
	}

NX_INLINE NxFluidEmitterDesc::~NxFluidEmitterDesc()
	{
	}

NX_INLINE void NxFluidEmitterDesc::setToDefault()
	{
    relPose							.id();
    frameActor						= NULL;
	type							= NX_FE_CONSTANT_PRESSURE;
	maxParticles					= 0;
	shape							= NX_FE_RECTANGULAR;
	dimensionX						= 1.0f;
	dimensionY						= 1.0f;
	randomPos						.zero();
	randomAngle						= 0.0f;
	fluidVelocityMagnitude			= 1.0f;
	rate							= 100.0f;
	particleLifetime				= 0.0f;
	flags							= NX_FEF_ENABLED|NX_FEF_VISUALIZATION|NX_FEF_ADD_ACTOR_VELOCITY;
	
	userData						= NULL;
	name							= NULL;
	}

NX_INLINE bool NxFluidEmitterDesc::isValid() const
	{
	if (!relPose.isFinite()) return false;
	
	if (dimensionX < 0.0f) return false;
	if (dimensionY < 0.0f) return false;

	if (randomPos.x < 0.0f) return false;
	if (randomPos.y < 0.0f) return false;
	if (randomPos.z < 0.0f) return false;
	if (!randomPos.isFinite()) return false;

	if (randomAngle < 0.0f) return false;

	if (flags & NX_FEF_BROKEN_ACTOR_REF) return false;
	if (!((shape & NX_FE_ELLIPSE) ^ (shape & NX_FE_RECTANGULAR))) return false;
	if (!((type & NX_FE_CONSTANT_FLOW_RATE) ^ (type & NX_FE_CONSTANT_PRESSURE))) return false;

	if (rate < 0.0f) return false;
	if (fluidVelocityMagnitude < 0.0f) return false;
	if (particleLifetime < 0.0f) return false;

	return true;
	}

/** @} */
#endif


//AGCOPYRIGHTBEGIN
///////////////////////////////////////////////////////////////////////////
// Copyright  2005 AGEIA Technologies.
// All rights reserved. www.ageia.com
///////////////////////////////////////////////////////////////////////////
//AGCOPYRIGHTEND

