/****************************************************************************\
*                                                                            *
* 10.05.2006                                                                 *
* CPhysXMgr - Source File													 *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>
#include <NxPhysics.h>
#include <NxCooking.h>
#include <NxStream.h>
#include <ControllerManager.h>
#include <NxCapsuleController.h>
#include <NxBoxController.h>
#include "CPhysXMgr.h"
#include "CEngineFactory.h"
#include <FileSystem/CStdFile.h>

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/
#pragma comment(lib, "PhysXLoader.lib")
#pragma comment(lib, "NxCooking.lib")
#pragma comment(lib, "NxCharacter.lib")

class MyOutputStream : public NxUserOutputStream
{
    void reportError(NxErrorCode code, const char *message, const char* file, int line)
    {
        LOG(TL_ERROR, "Error: PhysX reported: \"%s\" (%s, %d)", message, file, line);
    }

    NxAssertResponse reportAssertViolation (const char *message, const char *file,int line)
    {
        //this should not get hit by
        // a properly debugged SDK!
        assert(0 && "Error: reportAssertViolation hit in MyOutputStream!");
        return NX_AR_CONTINUE;
    }

    void print (const char *message)
    {
        LOG(TL_DEBUG, "PhysX said: \"%s\"", message);
    }

} g_MyOutputStream;

class MyContactReport : public NxUserContactReport
{
    virtual void onContactNotify(NxContactPair& pair, NxU32 events)
    {
		IPhysXObj *pFirst = (IPhysXObj *) pair.actors[0]->userData;
		IPhysXObj *pSecond = (IPhysXObj *) pair.actors[1]->userData;
        //Verify that the userData has been set (... with pointers to the corresponding IPhysXObj)
        if(pFirst && pSecond)
        {
            //LOG(TL_DEBUG, "Notifying on contact");
            //Tell the actors that they "met" each other :-)
            pFirst->CollidedWith(pSecond);
            pSecond->CollidedWith(pFirst);
        }
    }
} g_MyContactReport;

class MyControllerHitReport : public NxUserControllerHitReport
{
    virtual NxControllerAction  onShapeHit(const NxControllerShapeHit& hit)
    {
		IPhysXObj *pFirst = (IPhysXObj *) hit.controller->getActor()->userData;
		IPhysXObj *pSecond = (IPhysXObj *) hit.shape->getActor().userData;
		//Verify that the userData has been set (... with pointers to the corresponding IPhysXObj)
		if(pFirst && pSecond)
		{
			//LOG(TL_DEBUG, "Notifying on shape hit");
			//Tell the actors that they "met" each other :-)
			pFirst->CollidedWith(pSecond);
			pSecond->CollidedWith(pFirst);
		}

        return NX_ACTION_NONE;
    }

    virtual NxControllerAction  onControllerHit(const NxControllersHit& hit)
    {
		if(hit.other) //Is sometimes null... don't know when and why
		{
			IPhysXObj *pFirst = (IPhysXObj *) hit.controller->getActor()->userData;
			IPhysXObj *pSecond = (IPhysXObj *) hit.other->getActor()->userData;
			//Verify that the userData has been set (... with pointers to the corresponding IPhysXObj)
			if(pFirst && pSecond)
			{
				//LOG(TL_DEBUG, "Notifying on controller hit");
				//Tell the actors that they "met" each other :-)
				pFirst->CollidedWith(pSecond);
				pSecond->CollidedWith(pFirst);
			}
		}

		return NX_ACTION_NONE;
    }
} g_MyControllerHitReport;

class MyFileStream : public NxStream
{
    public:
        MyFileStream(const std::string &sFilename, bool load) {bool bRet = m_File.Create(sFilename, (load ? ACCESS_READ_ONLY : ACCESS_WRITE_ONLY), (load ? CREATION_OPEN_EXISTING : CREATION_OPEN_ALWAYS));
                                                               assert(bRet && "Error: Couldn't create file in MyFileStream::MyFileStream");}
        virtual ~MyFileStream() {m_File.Destroy();}

        virtual	NxU8    readByte() const {NxU8 buffer; readBuffer(&buffer, sizeof(buffer)); return buffer;}
        virtual	NxU16   readWord() const {NxU16 buffer; readBuffer(&buffer, sizeof(buffer)); return buffer;}
        virtual NxU32   readDword() const {NxU32 buffer; readBuffer(&buffer, sizeof(buffer)); return buffer;}
        virtual float   readFloat() const {float buffer; readBuffer(&buffer, sizeof(buffer)); return buffer;}
        virtual double  readDouble() const {double buffer; readBuffer(&buffer, sizeof(buffer)); return buffer;}
        virtual void    readBuffer(void* buffer, NxU32 size) const {m_File.Read(buffer, size);}

        virtual NxStream& storeByte(NxU8 data) {storeBuffer(&data, sizeof(data)); return *this;}
        virtual NxStream& storeWord(NxU16 data) {storeBuffer(&data, sizeof(data)); return *this;}
        virtual NxStream& storeDword(NxU32 data) {storeBuffer(&data, sizeof(data)); return *this;}
        virtual NxStream& storeFloat(NxReal data) {storeBuffer(&data, sizeof(data)); return *this;}
        virtual NxStream& storeDouble(NxF64 data) {storeBuffer(&data, sizeof(data)); return *this;}
        virtual NxStream& storeBuffer(const void* buffer, NxU32 size) {m_File.Write(buffer, size); return *this;}

    private:
        CStdFile m_File;
};

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
CPhysXMgr::CPhysXMgr() : m_pPhysicsSDK(NULL), m_pScene(NULL), m_pControllerMgr(NULL), m_pActorDesc(NULL),
    m_pBodyDesc(NULL), m_pGroundPlane(NULL)
{
}

bool CPhysXMgr::Create()
{
    if(m_pPhysicsSDK || m_pScene)
    {
        LOG(TL_DEBUG, "Warning: CPhysXMgr::Create called but the object has been created before");
        return true;
    }

    m_pPhysicsSDK = NxCreatePhysicsSDK(NX_PHYSICS_SDK_VERSION, NULL, &g_MyOutputStream);
    if(!m_pPhysicsSDK)
    {
        LOG(TL_ERROR, "Error: NxCreatePhysicsSDK failed in CPhysXMgr::Create");
        return false;
    }

    //Set gravity:
    m_vGravity.Set(0.0f, -9.8f, 0.0f);
    //Set skin depth
    m_pPhysicsSDK->setParameter(NX_SKIN_WIDTH, 0.01f);

    // Create the scene
    NxSceneDesc sceneDesc;
    sceneDesc.gravity.x             = m_vGravity.m_X;
    sceneDesc.gravity.y             = m_vGravity.m_Y;
    sceneDesc.gravity.z             = m_vGravity.m_Z;
    sceneDesc.broadPhase            = NX_BROADPHASE_COHERENT;
    sceneDesc.collisionDetection    = true;
    m_pScene = m_pPhysicsSDK->createScene(sceneDesc);
    if(!m_pScene)
    {
        LOG(TL_ERROR, "Error: m_pPhysicsSDK->createScene failed in CPhysXMgr::Create");
        return false;
    }

    // Create the default material
    NxMaterial* defaultMaterial = m_pScene->getMaterialFromIndex(0); 
    defaultMaterial->setRestitution(0.0f);
    defaultMaterial->setStaticFriction(0.7f);
    defaultMaterial->setDynamicFriction(0.7f);

    //Register our contact callback object
    m_pScene->setUserContactReport(&g_MyContactReport);
	//Enable all contact reports (for the first 32 groups... won't use more)
	for(word i = 0; i < 32; ++i)
	{
		for(word j = i; j < 32; ++j)
		{
			m_pScene->setActorGroupPairFlags(i, j, NX_NOTIFY_ON_START_TOUCH | NX_NOTIFY_ON_END_TOUCH | NX_NOTIFY_ON_TOUCH);
		}
	}

    //Create the controller manager
    m_pControllerMgr = new ControllerManager;

    //Initialise cooking:
    if(!NxInitCooking(NULL, &g_MyOutputStream))
    {
        LOG(TL_ERROR, "Error: NxInitCooking failed in CPhysXMgr::Create");
        return false;
    }

    //Debug:
    //EnableDebugRendering(true);

    return true;
}

void CPhysXMgr::Update(float dt)
{
    assert(m_pScene && "Error: m_pScene is NULL in CPhysXMgr::Update");
    m_pScene->simulate(dt);
    m_pScene->flushStream();
    //We could do something useful between those 2... like rendering the skybox and other physic-less stuff :-)
    m_pScene->fetchResults(NX_RIGID_BODY_FINISHED, true);

    m_pControllerMgr->updateControllers();
}

void CPhysXMgr::EnableDebugRendering(bool bEnable)
{
    if(bEnable)
    {
        m_pPhysicsSDK->setParameter(NX_VISUALIZATION_SCALE, 1.0f);
        m_pPhysicsSDK->setParameter(NX_VISUALIZE_BODY_AXES, 1.0f);
        m_pPhysicsSDK->setParameter(NX_VISUALIZE_BODY_LIN_VELOCITY, 1.0f);
        m_pPhysicsSDK->setParameter(NX_VISUALIZE_BODY_CONTACT_LIST, 1.0f);
        m_pPhysicsSDK->setParameter(NX_VISUALIZE_CONTACT_POINT, 1.0f);
        m_pPhysicsSDK->setParameter(NX_VISUALIZE_CONTACT_FORCE, 1.0f);
        m_pPhysicsSDK->setParameter(NX_VISUALIZE_ACTOR_AXES, 1.0f);
        m_pPhysicsSDK->setParameter(NX_VISUALIZE_COLLISION_SHAPES, 1.0f);
        m_pPhysicsSDK->setParameter(NX_VISUALIZE_COLLISION_COMPOUNDS, 1.0f);
        m_pPhysicsSDK->setParameter(NX_VISUALIZE_COLLISION_VNORMALS, 1.0f);
        m_pPhysicsSDK->setParameter(NX_VISUALIZE_COLLISION_FNORMALS, 1.0f);
        m_pPhysicsSDK->setParameter(NX_VISUALIZE_COLLISION_EDGES, 1.0f);
        m_pPhysicsSDK->setParameter(NX_VISUALIZE_COLLISION_SPHERES, 1.0f);
    }
    else
    {
        m_pPhysicsSDK->setParameter(NX_VISUALIZATION_SCALE, 0.0f);
    }
}

//Debug
#include <gl/GL.h>

void CPhysXMgr::RenderDebugData()
{
    const NxDebugRenderable *dbgRenderable = m_pScene->getDebugRenderable();

    NxU32 NbPoints = dbgRenderable->getNbPoints();
    const NxDebugPoint* Points = dbgRenderable->getPoints();
    glBegin(GL_POINTS);
    while(NbPoints--)
    {
        SetDebugColor((dword) Points->color);
        glVertex3fv(&Points->p.x);
        ++Points;
    }
    glEnd();
    NxU32 NbLines = dbgRenderable->getNbLines();
    const NxDebugLine* Lines = dbgRenderable->getLines();
    glBegin(GL_LINES);
    while(NbLines--)
    {
        SetDebugColor((dword) Lines->color);
        glVertex3fv(&Lines->p0.x);
        glVertex3fv(&Lines->p1.x);
        ++Lines;
    }
    glEnd();
    NxU32 NbTris = dbgRenderable->getNbTriangles();
    const NxDebugTriangle* Triangles = dbgRenderable->getTriangles();
    glBegin(GL_TRIANGLES);
    while(NbTris--)
    {
        SetDebugColor((dword) Triangles->color);
        glVertex3fv(&Triangles->p0.x);
        glVertex3fv(&Triangles->p1.x);
        glVertex3fv(&Triangles->p2.x);
        ++Triangles;
    }
    glEnd();
}

const TSmartPointer<CPhysXMesh> CPhysXMgr::CookTerrain(dword dwNumVertices, dword dwNumIndices,
        const CVector3f *pVertexBuffer, const dword *pIndexBuffer, float fTerrainDepth)
{
    //Set mesh data
    NxTriangleMeshDesc desc;
    desc.numVertices				= dwNumVertices;
    desc.numTriangles				= dwNumIndices / 3;
    desc.pointStrideBytes			= sizeof(CVector3f);
    desc.triangleStrideBytes		= 3*sizeof(dword);
    desc.points						= pVertexBuffer;
    desc.triangles					= pIndexBuffer;
    desc.flags						= 0;
    desc.heightFieldVerticalAxis    = NX_Y;
    desc.heightFieldVerticalExtent	= -fTerrainDepth;

    //Knnten das Ganze in ein Hash-File cachen und nur cooken, wenn ntig
    if(!NxCookTriangleMesh(desc, MyFileStream("cookedmesh.tmp", false)))
    {
        LOG(TL_ERROR, "Error: NxCookTriangleMesh failed in CPhysXMgr::CookTerrain");
        return NULL;
    }

    NxTriangleMesh *pMeshData = m_pPhysicsSDK->createTriangleMesh(MyFileStream("cookedmesh.tmp", true));
/*
    MemoryWriteBuffer buf;

    //Knnten das Ganze in ein Hash-File cachen und nur cooken, wenn ntig
    if(!NxCookTriangleMesh(desc, buf))
    {
        LOG(TL_ERROR, "Error: NxCookTriangleMesh failed in CPhysXMgr::CookTerrain");
        return NULL;
    }

    NxTriangleMesh *pMeshData = m_pPhysicsSDK->createTriangleMesh(MemoryReadBuffer(buf.data));
*/
    if(!pMeshData)
    {
        LOG(TL_ERROR, "Error: m_pPhysicsSDK->createTriangleMesh failed in CPhysXMgr::CookTerrain");
        return NULL;
    }

    TSmartPointer<CPhysXMesh> pMesh = new CPhysXMesh(pMeshData);
    m_Meshes.push_back(pMesh);

    return pMesh;
}

void CPhysXMgr::BeginActorCreation(const CMatrix44 &orientation, byte bCollisionGroup, bool bDynamicActor, float fDensity)
{
    assert((!m_pActorDesc || !m_pBodyDesc || m_ShapeDescs.size()) && "Error: m_pActorDesc or m_pBodyDesc is NOT NULL or m_ShapeDescs NOT empty in CPhysXMgr::BeginActorCreation");
	m_bActorGroup = bCollisionGroup;
    m_pActorDesc = new NxActorDesc;
    m_pActorDesc->globalPose.setColumnMajor44(&orientation._11);
	m_pActorDesc->group = m_bActorGroup;
    if(bDynamicActor)
    {
        m_pBodyDesc = new NxBodyDesc;
        m_pActorDesc->body = m_pBodyDesc;
        m_pActorDesc->density = fDensity;
    }
}

CPhysXActor *CPhysXMgr::FinishActorCreation()
{
    assert(m_pActorDesc && "Error: m_pActorDesc is NULL in CPhysXMgr::FinishActorCreation");
    NxActor *pActor = m_pScene->createActor(*m_pActorDesc);
    if(!pActor)
    {
        LOG(TL_ERROR, "Error: m_pScene->createActor returned NULL in CPhysXMgr::FinishActorCreation");
        return NULL;
    }
    CPhysXActor *pPhysXActor = new CPhysXActor(pActor);

	//Bind this pPhysXActor to the NxController to allow the Manager to notify it
	pActor->userData = pPhysXActor;

    SAFE_DELETE(m_pActorDesc);
    SAFE_DELETE(m_pBodyDesc);
    for(ShapeList::iterator it = m_ShapeDescs.begin(); it != m_ShapeDescs.end(); ++it)
    {
        SAFE_DELETE((*it));
    }
    m_ShapeDescs.clear();

    return pPhysXActor;
}

void CPhysXMgr::AddMesh(const CMatrix44 &orientation, const TSmartPointer<CPhysXMesh> &pMesh)
{
    assert(m_pActorDesc && "Error: m_pActorDesc is NULL in CPhysXMgr::AddMesh");
    NxTriangleMeshShapeDesc *pDesc = new NxTriangleMeshShapeDesc;
    pDesc->meshData = pMesh->m_pMesh;
    pDesc->localPose.setColumnMajor44(&orientation._11);
	pDesc->group = m_bActorGroup;

    m_ShapeDescs.push_back(pDesc); //Save shape pointer to delete it later
    m_pActorDesc->shapes.push_back(pDesc); //Add shape to actor description
}

void CPhysXMgr::AddSphere(const CMatrix44 &orientation, float fRadius)
{
    assert(m_pActorDesc && "Error: m_pActorDesc is NULL in CPhysXMgr::AddSphere");
    NxSphereShapeDesc *pDesc = new NxSphereShapeDesc;
    pDesc->radius = fRadius;
    pDesc->localPose.setColumnMajor44(&orientation._11);
	pDesc->group = m_bActorGroup;

    m_ShapeDescs.push_back(pDesc); //Save shape pointer to delete it later
    m_pActorDesc->shapes.push_back(pDesc); //Add shape to actor description
}

void CPhysXMgr::AddCube(const CMatrix44 &orientation, const CVector3f &vSize)
{
    assert(m_pActorDesc && "Error: m_pActorDesc is NULL in CPhysXMgr::AddCube");
    NxBoxShapeDesc *pDesc = new NxBoxShapeDesc;
    pDesc->dimensions.set(vSize.m_X / 2.0f, vSize.m_Y / 2.0f, vSize.m_Z / 2.0f);
    pDesc->localPose.setColumnMajor44(&orientation._11);
	pDesc->group = m_bActorGroup;

    m_ShapeDescs.push_back(pDesc); //Save shape pointer to delete it later
    m_pActorDesc->shapes.push_back(pDesc); //Add shape to actor description
}

void CPhysXMgr::AddCapsule(const CMatrix44 &orientation, float fRadius, float fHeight)
{
    assert(m_pActorDesc && "Error: m_pActorDesc is NULL in CPhysXMgr::AddCapsule");
    NxCapsuleShapeDesc *pDesc = new NxCapsuleShapeDesc;
    pDesc->radius = fRadius;
    pDesc->height = fHeight;
    pDesc->localPose.setColumnMajor44(&orientation._11);
	pDesc->group = m_bActorGroup;

    m_ShapeDescs.push_back(pDesc); //Save shape pointer to delete it later
    m_pActorDesc->shapes.push_back(pDesc); //Add shape to actor description
}

void CPhysXMgr::AddPlane(const CVector4f &plane)
{
	assert(m_pActorDesc && "Error: m_pActorDesc is NULL in CPhysXMgr::AddPlane");
	NxPlaneShapeDesc *pDesc = new NxPlaneShapeDesc;
	pDesc->normal = NxVec3(plane.m_X, plane.m_Y, plane.m_Z);
	pDesc->d = plane.m_W;
	pDesc->group = m_bActorGroup;

	m_ShapeDescs.push_back(pDesc); //Save shape pointer to delete it later
	m_pActorDesc->shapes.push_back(pDesc); //Add shape to actor description
}


void CPhysXMgr::DeleteActor(CPhysXActor *pActor)
{
    m_pScene->releaseActor(*(pActor->m_pActor));
    SAFE_DELETE(pActor);
}

CPhysXCharacter *CPhysXMgr::CreateCapsuleCharacter(const CVector3f &vPos, float fRadius, float fHeight, byte bCollisionGroup)
{
    NxCapsuleControllerDesc desc;
    desc.position.x     = vPos.m_X;
    desc.position.y		= vPos.m_Y;
    desc.position.z		= vPos.m_Z;
    desc.height = fHeight;
    desc.radius = fRadius;
    desc.upDirection	= NX_Y;
    desc.slopeLimit		= cosf(45.0f * MY_PI / 180.0f); //45 Grad maximale Neigung
    desc.skinWidth		= 0.05f; //Ausprobieren...
    desc.stepOffset		= 0.5f; //Ausprobieren...
    desc.callback		= &g_MyControllerHitReport;

    NxController *pController = m_pControllerMgr->createController(m_pScene, desc);
    if(!pController)
    {
        LOG(TL_ERROR, "Error: m_pControllerMgr->createController failed in CPhysXMgr::CreateAABoxCharacter");
        return NULL;
    }

	CPhysXCharacter *pCharacter = new CPhysXCharacter(pController);

	//Bind this pCharacter to the NxController to allow the Manager to notify it
	pController->getActor()->userData = pCharacter;

	//CollisionGroup setzen
	pController->getActor()->setGroup(bCollisionGroup);
	NxU32 numShapes = pController->getActor()->getNbShapes();
	NxShape * const *pShapes = pController->getActor()->getShapes();
	for(NxU32 i = 0; i < numShapes; ++i)
	{
		pShapes[i]->setGroup(bCollisionGroup);
	}

    pCharacter->m_pController = pController;

    return pCharacter;
}

CPhysXCharacter *CPhysXMgr::CreateAABoxCharacter(const CVector3f &vPos, const CVector3f &vSize, byte bCollisionGroup)
{
    NxBoxControllerDesc desc;
    desc.position.x     = vPos.m_X;
    desc.position.y		= vPos.m_Y;
    desc.position.z		= vPos.m_Z;
    desc.extents.x      = vSize.m_X;
    desc.extents.y		= vSize.m_Y;
    desc.extents.z		= vSize.m_Z;
    desc.upDirection	= NX_Y;
    desc.slopeLimit		= cosf(45.0f * MY_PI / 180.0f); //45 Grad maximale Neigung
    desc.skinWidth		= 0.05f; //Ausprobieren...
    desc.stepOffset		= 0.5f; //Ausprobieren...
    desc.callback		= &g_MyControllerHitReport;

    NxController *pController = m_pControllerMgr->createController(m_pScene, desc);
    if(!pController)
    {
        LOG(TL_ERROR, "Error: m_pControllerMgr->createController failed in CPhysXMgr::CreateAABoxCharacter");
        return NULL;
    }

	CPhysXCharacter *pCharacter = new CPhysXCharacter(pController);

	//Bind this pCharacter to the NxController to allow the Manager to notify it
	pController->getActor()->userData = pCharacter;

	//CollisionGroup setzen
	pController->getActor()->setGroup(bCollisionGroup);
	NxU32 numShapes = pController->getActor()->getNbShapes();
	NxShape * const *pShapes = pController->getActor()->getShapes();
	for(NxU32 i = 0; i < numShapes; ++i)
	{
		pShapes[i]->setGroup(bCollisionGroup);
	}

    pCharacter->m_pController = pController;

    return pCharacter;
}

void CPhysXMgr::DeleteCharacter(CPhysXCharacter *pCharacter)
{
    m_pControllerMgr->releaseController(*(pCharacter->m_pController));
    SAFE_DELETE(pCharacter);
}

bool CPhysXMgr::CheckOverlapping(const CAABox3f &box, dword dwGroupMask)
{
	NxBounds3 bounds;
	bounds.set(box.m_MinX, box.m_MinY, box.m_MinZ, box.m_MaxX, box.m_MaxY, box.m_MaxZ);
	return m_pScene->checkOverlapAABB(bounds, NX_ALL_SHAPES, dwGroupMask);
}

void CPhysXMgr::SetGroupCollisionTesting(byte group1, byte group2, bool enable)
{
	m_pScene->setGroupCollisionFlag(group1, group2, enable);
}

CPhysXMgr::~CPhysXMgr()
{
    SAFE_DELETE(m_pBodyDesc);
    SAFE_DELETE(m_pActorDesc);
    for(ShapeList::iterator it = m_ShapeDescs.begin(); it != m_ShapeDescs.end(); ++it)
    {
        SAFE_DELETE((*it));
    }
    m_ShapeDescs.clear();

    if(m_pControllerMgr)
    {
        m_pControllerMgr->purgeControllers();
        SAFE_DELETE(m_pControllerMgr);
    }

    if(m_pScene)
    {
        m_pScene->releaseActor(*m_pGroundPlane);        
        m_pScene->setUserContactReport(NULL);
        m_pPhysicsSDK->releaseScene(*m_pScene);
    }

    NxCloseCooking();

    if(m_pPhysicsSDK)
    {
        //Delete all cooked meshes
        for(MeshList::iterator it = m_Meshes.begin(); it != m_Meshes.end(); ++it)
        {
            m_pPhysicsSDK->releaseTriangleMesh(*((*it)->m_pMesh));
        }
        m_Meshes.clear();

        m_pPhysicsSDK->release();
    }
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/

void CPhysXMgr::SetDebugColor(dword dwColor) const
{
    float b = float((dwColor)&0xff)/255.0f;
    float g = float((dwColor>>8)&0xff)/255.0f;
    float r = float((dwColor>>16)&0xff)/255.0f;
    glColor3f(r, g, b);    
}

