/****************************************************************************\
*                                                                            *
* 30.03.2006                                                                 *
* CCamera - Source File                                                      *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CCamera.h"
#include "CEngineFactory.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
void CCamera::SetPerspective(const float &fFov, const float &fAspectRatio, const float &fNearZ, const float &fFarZ)
{
    m_Projection.Perspective(fFov, fAspectRatio, fNearZ, fFarZ);
    m_bFrustumValid = false;
}

void CCamera::SetPos(const CVector3f &vPos)
{
    m_vPos = -vPos;
    m_bViewValid = m_bFrustumValid = false;
}

void CCamera::SetRot(const CVector3f &vRot)
{
    m_vRot = -vRot;
    m_bVectorsValid = m_bViewValid = m_bFrustumValid = false;
}

void CCamera::LookAt(const CVector3f &vPoint, const CVector3f &vUp)
{
    m_vIn = vPoint - m_vPos;
    m_vIn.Normalize();

    m_vUp = vUp;
    m_vUp.Normalize();

    m_vRight = m_vUp * m_vIn;
    m_vRight.Normalize();
    

    //vRot??? Math goes in here :-(
    LOG(TL_ERROR, "Error: CCamera::LookAt not implemented");

    m_bVectorsValid = m_bViewValid = m_bFrustumValid = false;
}

void CCamera::MoveIn(float length)
{
    if(!m_bVectorsValid)
        Update();

    m_vPos -= m_vIn * length;

    m_bViewValid = m_bFrustumValid = false;
}

void CCamera::MoveUp(float length)
{
    if(!m_bVectorsValid)
        Update();

    m_vPos -= m_vUp * length;

    m_bViewValid = m_bFrustumValid = false;
}

void CCamera::MoveRight(float length)
{
    if(!m_bVectorsValid)
        Update();

    m_vPos -= m_vRight * length;

    m_bViewValid = m_bFrustumValid = false;
}

void CCamera::MoveX(float length)
{
    m_vPos.m_X -= length;

    m_bViewValid = m_bFrustumValid = false;
}

void CCamera::MoveY(float length)
{
    m_vPos.m_Y -= length;

    m_bViewValid = m_bFrustumValid = false;
}

void CCamera::MoveZ(float length)
{
    m_vPos.m_Z -= length;

    m_bViewValid = m_bFrustumValid = false;
}

void CCamera::Move(const CVector3f &vDistance)
{
    m_vPos -= vDistance;

    m_bViewValid = m_bFrustumValid = false;
}

void CCamera::RotateX(float angle)
{
    m_vRot.m_X -= angle;

    if(m_vRot.m_X > MY_PI / 2)
        m_vRot.m_X = MY_PI / 2;
    else if(m_vRot.m_X < -MY_PI / 2)
        m_vRot.m_X = -MY_PI / 2;

    m_bVectorsValid = m_bViewValid = m_bFrustumValid = false;
}

void CCamera::RotateY(float angle)
{
    m_vRot.m_Y -= angle;

    if(m_vRot.m_Y >= MY_PI * 2)
        m_vRot.m_Y -= MY_PI * 2;
    else if(m_vRot.m_Y <= -MY_PI * 2)
        m_vRot.m_Y += MY_PI * 2;

    m_bVectorsValid = m_bViewValid = m_bFrustumValid = false;
}

void CCamera::RotateZ(float angle)
{
    m_vRot.m_Z -= angle;

    if(m_vRot.m_Z >= MY_PI * 2)
        m_vRot.m_Z -= MY_PI * 2;
    else if(m_vRot.m_Z <= -MY_PI * 2)
        m_vRot.m_Z += MY_PI * 2;

    m_bVectorsValid = m_bViewValid = m_bFrustumValid = false;
}

const CVector3f &CCamera::GetIn()
{
    if(!m_bVectorsValid)
        Update();
    return m_vIn;
}

const CVector3f &CCamera::GetUp()
{
    if(!m_bVectorsValid)
        Update();
    return m_vUp;
}

const CVector3f &CCamera::GetRight()
{
    if(!m_bVectorsValid)
        Update();
    return m_vRight;
}

void CCamera::Log(word wTargetLog)
{
    if(!m_bVectorsValid || !m_bViewValid)
    {
        Update();
    }

    LOG(wTargetLog, "Pos:");
    (-m_vPos).Log(wTargetLog);
    LOG(wTargetLog, "Rot:");
    (m_vRot*180.0f/MY_PI).Log(wTargetLog);
    LOG(wTargetLog, "In:");
    m_vIn.Log(wTargetLog);
    LOG(wTargetLog, "Up:");
    m_vUp.Log(wTargetLog);
    LOG(wTargetLog, "Right:");
    m_vRight.Log(wTargetLog);
    LOG(wTargetLog, "View:");
    m_View.Log(wTargetLog);
}

void CCamera::UseForRendering()
{
    if(!m_bViewValid)
	{
        Update();
    }

    CEngineFactory::InstancePtr()->GetRenderer()->SetMatrixMode(EMM_PROJECTION);
	CEngineFactory::InstancePtr()->GetRenderer()->SetMatrix(m_Projection);
	CEngineFactory::InstancePtr()->GetRenderer()->SetMatrixMode(EMM_MODELVIEW);
	CEngineFactory::InstancePtr()->GetRenderer()->SetMatrix(m_View);
}

const CFrustum &CCamera::GetFrustum()
{
    if(!m_bFrustumValid)
        m_Frustum.CalculateFrustum(m_Projection, m_View);

    return m_Frustum;
}

const CVector3f CCamera::GetRay(float fX, float fY)
{
    if(!m_bViewValid || !m_bVectorsValid)
        Update();

    //Transform relative cursor position onto the near plane (assuming NearZ = 1.0f?)
    fX =  ((2.0f * fX) - 1.0f) / m_Projection._11;
    fY = -((2.0f * fY) - 1.0f) / m_Projection._22;

    CVector3f vec(fX * m_vRight.m_X + fY * m_vRight.m_Y + m_vRight.m_Z,
        fX * m_vUp.m_X + fY * m_vUp.m_Y + m_vUp.m_Z,
        fX * m_vIn.m_X + fY * m_vIn.m_Y + m_vIn.m_Z);

    vec.Normalize();

    return vec;
}

const CMatrix44 &CCamera::GetTransformation()
{
    if(!m_bViewValid)
        Update();

    return m_View;
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
void CCamera::Clear()
{
    m_vPos.Set(0.0f, 0.0f, 0.0f);
    m_vRot.Set(0.0f, 0.0f, 0.0f);
    m_bVectorsValid = m_bViewValid = m_bFrustumValid = false;
}

void CCamera::Update()
{
    if(!m_bViewValid || !m_bVectorsValid)
    {
        CMatrix44 trans, rot;

        rot.RotationZYX(m_vRot);
        trans.Translation(m_vPos);
        m_View.Multiply(rot, trans);    //first translation, then rotatoin

        //Get the transposed view vectors
        m_vRight.m_X = m_View._11;
        m_vRight.m_Y = m_View._12;
        m_vRight.m_Z = m_View._13;

        m_vUp.m_X = m_View._21;
        m_vUp.m_Y = m_View._22;
        m_vUp.m_Z = m_View._23;

        //...and invert the in vector
        m_vIn.m_X = -m_View._31;
        m_vIn.m_Y = -m_View._32;
        m_vIn.m_Z = -m_View._33;

/*  //Forget it...
        m_vRight.m_X = m_View._11;
        m_vRight.m_Y = m_View._21;
        m_vRight.m_Z = m_View._31;

        m_vUp.m_X = m_View._12;
        m_vUp.m_Y = m_View._22;
        m_vUp.m_Z = m_View._32;

        m_vIn.m_X = m_View._13;
        m_vIn.m_Y = m_View._23;
        m_vIn.m_Z = m_View._33;
*/

        m_bViewValid = m_bVectorsValid = true;
        m_bFrustumValid = false;
    }
}
