/****************************************************************************\
*                                                                            *
* 18.03.2006                                                                 *
* CGuiMgr - Source File                                                      *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CGuiMgr.h"
#include "CEngineFactory.h"
#include <Math/TVector4.h>

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/
const float cg_fBaseFontSize = 22.0f;

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
bool CGuiMgr::Create(const TSmartPointer<IWindow> &pWindow, const TSmartPointer<IInputMgr> &pInputMgr,
                     const TSmartPointer<CTimer> &pTimer, word wVScreenWidth, word wVScreenHeight)
{
    assert((bool) pWindow && "Error: pWindow is NULL in CGuiMgr::Create");
    assert((bool) pInputMgr && "Error: pInputMgr is NULL in CGuiMgr::Create");

    m_pWindow = pWindow;
    m_pWindow->RegisterHandler(this);

    m_pInputMgr = pInputMgr;
    m_pInputMgr->RegisterHandler(this);

    m_pTimer = pTimer;

    //Grenfaktoren fr virtuelles Koordinatensystem ausrechnen
    m_pWindow->GetSize(m_wVScreenWidth, m_wVScreenHeight);
    m_fWidthScale = ((float) wVScreenWidth) / m_wVScreenWidth;
    m_fHeightScale = ((float) wVScreenHeight) / m_wVScreenHeight;
    m_wVScreenWidth = wVScreenWidth;
    m_wVScreenHeight = wVScreenHeight;

    //Projektionsmatrix setzen
    m_Projection.Orthogonal(m_wVScreenWidth, m_wVScreenHeight, 1.0f, 1000.0f);

    m_pRenderer = CEngineFactory::InstancePtr()->GetRenderer();

    //Passende Font erstellen
    m_pGuiFont = m_pRenderer->GetFontObject("Verdana", (int)(cg_fBaseFontSize / m_fHeightScale), true, false);

    //GUI sounds erstellen
    m_pButtonRolloverSound = CEngineFactory::InstancePtr()->GetSoundMgr()->LoadSound(eDECOMPRESSED_SAMPLE,"Data\\sounds\\button_rollover.wav");
    m_pButtonClickSound = CEngineFactory::InstancePtr()->GetSoundMgr()->LoadSound(eDECOMPRESSED_SAMPLE,"Data\\sounds\\button_select.wav");
    m_pButtonRolloverSound->SetLoopCount(4);

    m_pFpsText = CreateText();
    m_pFpsText->SetColor(0xFFFFFFFF);
    m_pFpsText->SetPos(CVector3f(5.0f, 970.0f, 800.0f)); //980.0f ist im Gui nicht mehr sichtbar ?!?

    m_pRepsRendered = CreateText();
    m_pRepsRendered->SetColor(0xFFFFFFFF);
    m_pRepsRendered->SetPos(CVector3f(5.0f, 950.0f, 800.0f));

    UpdateCursorPos();

    return true;
}

//Debug
#include <Display/IRepresentation.h>

void CGuiMgr::Update()
{
    if(m_pRootObject)
    {
        m_pRootObject->Update((float) m_pTimer->GetFrameDuration());
    }

	if(m_bRenderInfo){

		//Debug: Update FPS
		m_newFpsUpdateTime = m_pTimer->GetTime();

		if(m_newFpsUpdateTime-m_oldFpsUpdateTime > 0.2){
			sprintf(m_sFps, "FPS: %0.1f", m_pTimer->GetFrameRate());
			m_pFpsText->SetText(m_sFps);
			m_oldFpsUpdateTime = m_newFpsUpdateTime;
		}
		//End debug

		//Debug: Update Reps
		sprintf(m_sRendered, "Rendered: %lu (%lu/%lu)", IRepresentation::ms_dwRendered, IRepresentation::ms_dwVerified, IRepresentation::ms_dwBothTests);
		IRepresentation::ms_dwRendered = 0;
		IRepresentation::ms_dwVerified = 0;
		IRepresentation::ms_dwBothTests = 0;
		m_pRepsRendered->SetText(m_sRendered);
		//End debug
	}

}

void CGuiMgr::Render()
{
    m_pRenderer->SetMatrixMode(EMM_PROJECTION);
    m_pRenderer->SetMatrix(m_Projection);
    //Debug
    m_pRenderer->SetMatrixMode(EMM_MODELVIEW);
    CMatrix44 m_ModelView;
    m_ModelView.Identity();
    m_pRenderer->SetMatrix(m_ModelView);
    //EndDebug
    m_pRenderer->EnableZTest(false);
    if(m_pRootObject)
    {
        m_pRootObject->Render();
    }

    if(m_bRenderInfo){
		m_pFpsText->Render();
		m_pRepsRendered->Render();
	}

    m_pRenderer->EnableZTest(true);
}

void CGuiMgr::SetRootObject(const TSmartPointer<IGuiObject> &pObject)
{
    if(m_pRootObject != pObject)
    {
        SetFocus(NULL); //Fokus entreien, auch wenn das neue Objekt ihn nicht will
        m_InputGrabbers.clear();
        m_pRootObject = pObject;
        if(m_pRootObject)
        {
            m_pRootObject->SetParent(NULL);
//            SetFocus(m_pRootObject);
        }
    }
}

void CGuiMgr::GrabInput(const TSmartPointer<IGuiObject> &pObject)
{
    m_InputGrabbers.push_back(pObject);
    if((bool) m_pFocusObject && !pObject->Contains(m_pFocusObject))
    {
        SetFocus(NULL); //Fokus entreien, auch wenn das neue Objekt ihn nicht will
        SetFocus(pObject);
    }
}

void CGuiMgr::ReleaseInput(const TSmartPointer<IGuiObject> &pObject)
{
    GuiObjectList::iterator i(m_InputGrabbers.begin());

    while(i != m_InputGrabbers.end() && !((*i) == pObject))
    {
        ++i;
    }
    if(i != m_InputGrabbers.end())
        m_InputGrabbers.erase(i);
}

void CGuiMgr::SetFocus(const TSmartPointer<IGuiObject> &pObject)
{
    //Nur etwas ndern falls der Fokus verschwindet oder niemand den Input beansprucht oder pObject Teil des beanspruchenden Objekts ist
    if(m_pFocusObject != pObject && (!pObject || !m_InputGrabbers.size() || m_InputGrabbers.back()->Contains(pObject)))
    {
        //Will das neue Objekt berhaupt den Fokus?
        if(!pObject || pObject->OnGetFocus())
        {
            if(m_pFocusObject)
                m_pFocusObject->OnLoseFocus();
            m_pFocusObject = pObject;
        }
    }
}

void CGuiMgr::Destroy()
{
    if(m_pWindow)
    {
        m_pWindow->UnregisterHandler(this);
    }
    if(m_pInputMgr)
    {
        m_pInputMgr->UnregisterHandler(this);
    }
    Clear();
}

TSmartPointer<CGuiContainer> CGuiMgr::CreateContainer() const
{
    return new CGuiContainer;
}

TSmartPointer<CGuiWnd> CGuiMgr::CreateWnd(const std::string &sTexture, const CVector2f &uv1, const CVector2f &uv2, const CVector4f &color, EBlendingMode blendingMode) const
{
	CGuiWnd *pWnd = new CGuiWnd;
	pWnd->SetRep(CEngineFactory::InstancePtr()->GetModelFactory()->getStockObject(eQUAD, sTexture, uv1.m_X, uv1.m_Y, uv2.m_X, uv2.m_Y, &color, blendingMode));
    return pWnd;
}

TSmartPointer<CGuiButton> CGuiMgr::CreateButton(const std::string &sTexture) const
{
	CGuiButton *newButtton = new CGuiButton(m_pButtonRolloverSound, m_pButtonClickSound);
	CVector4f col(1.0f,1.0f,1.0f,1.0f);

	newButtton->setNormalRep(CEngineFactory::InstancePtr()->GetModelFactory()->getStockObject(eQUAD, sTexture,0.0f,0.0f,1.0f,0.25f,&col, EBM_NONE));
	newButtton->setOverRep(CEngineFactory::InstancePtr()->GetModelFactory()->getStockObject(eQUAD, sTexture,0.0f,0.25f,1.0f,0.5f,&col, EBM_NONE));

    col.Set(0.8f,0.8f,0.8f,1.0f);
    newButtton->setDownRep(CEngineFactory::InstancePtr()->GetModelFactory()->getStockObject(eQUAD, sTexture,0.0f,0.5f,1.0f,0.75f,&col, EBM_NONE));
	newButtton->setDisabledRep(CEngineFactory::InstancePtr()->GetModelFactory()->getStockObject(eQUAD, sTexture,0.0f,0.75f,1.0f,1.0f,&col, EBM_NONE));


	return newButtton;
}

TSmartPointer<CGuiText> CGuiMgr::CreateText() const
{
    return new CGuiText(m_pGuiFont);
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
void CGuiMgr::Clear()
{
    m_wVScreenWidth = m_wVScreenHeight = 0;
    m_fWidthScale = m_fHeightScale = 0;
    m_vCursorPos.Set(0.0f, 0.0f);

    m_pRootObject = NULL;
    m_pFocusObject = NULL;
    m_InputGrabbers.clear();

    m_pGuiFont = NULL;

    m_pWindow = NULL;
    m_pInputMgr = NULL;
    m_pRenderer = NULL;
}

void CGuiMgr::UpdateCursorPos()
{
    assert((bool) m_pInputMgr && "Error: m_pInputMgr is NULL in CGuiMgr::UpdateCursorPos");

    sdword x, y;
    m_pInputMgr->GetCursorPos(x, y);
    m_vCursorPos.Set(x * m_fWidthScale, y * m_fHeightScale);
}

void CGuiMgr::OnWndSize(word wWidth, word wHeight)
{
    m_fWidthScale = ((float) m_wVScreenWidth) / wWidth;
    m_fHeightScale = ((float) m_wVScreenHeight) / wHeight;
}

bool CGuiMgr::OnMouseMove(sdword sdwX, sdword sdwY, sdword sdwZ)
{
    UpdateCursorPos();

    if(m_InputGrabbers.size())
    {
        m_InputGrabbers.back()->OnMouseMove(sdwX * m_fWidthScale, sdwY * m_fHeightScale, sdwZ);
        return true;
    }
    
    if(m_pRootObject)
    {
        return m_pRootObject->OnMouseMove(sdwX * m_fWidthScale, sdwY * m_fHeightScale, sdwZ);
    }

    return false;
}

bool CGuiMgr::OnMouseKeyDown(byte bKey)
{
    if(m_InputGrabbers.size())
    {
        m_InputGrabbers.back()->OnMouseKeyDown(bKey);
        return true;
    }
    
    if(m_pRootObject)
    {
        return m_pRootObject->OnMouseKeyDown(bKey);
    }

    return false;
}

bool CGuiMgr::OnMouseKeyUp(byte bKey)
{
    if(m_InputGrabbers.size())
    {
        m_InputGrabbers.back()->OnMouseKeyUp(bKey);
        return true;
    }

    if(m_pRootObject)
    {
        return m_pRootObject->OnMouseKeyUp(bKey);
    }

    return false;
}

bool CGuiMgr::OnChar(char c)
{
    if(m_InputGrabbers.size())
    {
        m_InputGrabbers.back()->OnChar(c);
        return true;
    }

    if(m_pRootObject)
    {
        return m_pRootObject->OnChar(c);
    }

    return false;
}

bool CGuiMgr::OnKeyDown(dword dwKey)
{
    if(m_InputGrabbers.size())
    {
        m_InputGrabbers.back()->OnKeyDown(dwKey);
        return true;
    }

    if(m_pRootObject)
    {
        return m_pRootObject->OnKeyDown(dwKey);
    }

    return false;
}

bool CGuiMgr::OnKeyUp(dword dwKey)
{
    if(m_InputGrabbers.size())
    {
        m_InputGrabbers.back()->OnKeyUp(dwKey);
        return true;
    }

    if(m_pRootObject)
    {
        return m_pRootObject->OnKeyUp(dwKey);
    }

    //ToDo: Look for shortcut

    return false;
}
