/****************************************************************************\
*                                                                            *
* 28.03.2006                                                                 *
* TQTNode - Header File                                                      *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __TQTNODE_H__
#define __TQTNODE_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include <Math/TAABox3.h>
#include <Math/TSphere.h>
#include <Memory/TSmartPointer.h>
#include "TSceneNode.h"

#include "GraphStates/IStateNode.h"

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
/*
    Children are arranged like this:
    /-------\
    | 1 | 2 |
    |---+---|
    | 3 | 4 |
    \-------/


/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
template<class T>
class TQTNode : public TSceneNode<T>
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        TQTNode() {Clear();}

        bool Create(float fMinX, float fMinZ, float fSizeX, float fSizeZ, float fMinY, float fMaxY, word wGraphDepth, TQTNode<T> *pParent = NULL);
        void Render(const CFrustum &frustum, bool bVerify);

        void AddObject(const TSmartPointer<T> &pObject);
        bool RemoveObject(const TSmartPointer<T> &pObject);
        void PickObject(const CFrustum &frustum, const CVector3f &vRayOrigin, const CVector3f &vRayDir,
                        TSmartPointer<IRepresentation> &pRepresentation, float &fHitDistance, CVector3f &vHitPos, bool bVerify);

        virtual void ObjectChanged(const TSmartPointer<T> &pObject);

		void SetStateNode(TSmartPointer<IStateNode> pStateNode){m_pStateNode = pStateNode;}
		TSmartPointer<IStateNode> & getStateNode(){return m_pStateNode;}

		void Destroy();
        
        virtual ~TQTNode() {Destroy();}
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        void Clear();
        sbyte FitsInto(const TSmartPointer<T> &pObject);      //Returns 0-3 if the box fits into a direct child,
                                                              //-1 if it fits into this node and -2 if it doesn't fit at all (AND this node has a parent!)
        //Private copy constructor and assignment operator to prevent mistakes:
        TQTNode(const TQTNode &);
        TQTNode &operator =(const TQTNode &);
        
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        CAABox3f m_BoundingBox;
        CSpheref m_BoundingSphere;
        typedef std::vector< TSmartPointer<T> > QTElementList;
        QTElementList m_Elements;
        TQTNode<T> *m_pChildren;
        TQTNode<T> *m_pParent;

		TSmartPointer<IStateNode> m_pStateNode;

        //Debug: render counter
    public:
        static dword ms_dwCount;
};

template<class T>
dword TQTNode<T>::ms_dwCount = 0;

template<class T>
bool TQTNode<T>::Create(float fMinX, float fMinZ, float fSizeX, float fSizeZ, float fMinY, float fMaxY, word wGraphDepth, TQTNode<T> *pParent)
{
    assert(m_pChildren == NULL && "Error: TQTNode<T> already created");

    m_BoundingBox.SetSafePointPoint(fMinX, fMinY, fMinZ, fMinX + fSizeX, fMaxY, fMinZ + fSizeZ);

    //Needed later anyway (for chlidren.Create)
    fSizeX /= 2.0;
    fSizeZ /= 2.0;

    m_BoundingSphere.m_vCenter.m_X = fMinX + fSizeX;
    m_BoundingSphere.m_vCenter.m_Y = (fMinY + fMaxY) / 2.0f;
    m_BoundingSphere.m_vCenter.m_Z = fMinZ + fSizeZ;
    m_BoundingSphere.m_Radius = sqrt(fSizeX  * fSizeX + ((fMinY - fMaxY) * (fMinY - fMaxY)) / 4.0f + fSizeZ * fSizeZ);
    m_pParent = pParent;

    if(wGraphDepth > 1)
    {
        --wGraphDepth;
        m_pChildren = new TQTNode<T>[4];
        if(!m_pChildren[0].Create(fMinX,          fMinZ,           fSizeX, fSizeZ, fMinY, fMaxY, wGraphDepth, this))
            return false;
        if(!m_pChildren[1].Create(fMinX + fSizeX, fMinZ,           fSizeX, fSizeZ, fMinY, fMaxY, wGraphDepth, this))
            return false;
        if(!m_pChildren[2].Create(fMinX,          fMinZ + fSizeZ, fSizeX, fSizeZ, fMinY, fMaxY, wGraphDepth, this))
            return false;
        if(!m_pChildren[3].Create(fMinX + fSizeX, fMinZ + fSizeZ, fSizeX, fSizeZ, fMinY, fMaxY, wGraphDepth, this))
            return false;
    }

    return true;
}

#include <gl/GL.h> //Debug

template<class T>
void TQTNode<T>::Render(const CFrustum &frustum, bool bVerify)
{
/*
    //Debug: Render bounding box
    CAABox3f box = m_BoundingBox;
    glBegin(GL_LINES);
    glColor3f(0.0f, 1.0f, 0.0f);
    glVertex3f(box.m_MinX, box.m_MinY, box.m_MinZ);
    glVertex3f(box.m_MaxX, box.m_MinY, box.m_MinZ);

    glVertex3f(box.m_MinX, box.m_MaxY, box.m_MinZ);
    glVertex3f(box.m_MaxX, box.m_MaxY, box.m_MinZ);

    glVertex3f(box.m_MinX, box.m_MinY, box.m_MinZ);
    glVertex3f(box.m_MinX, box.m_MaxY, box.m_MinZ);

    glVertex3f(box.m_MinX, box.m_MinY, box.m_MaxZ);
    glVertex3f(box.m_MinX, box.m_MaxY, box.m_MaxZ);

    glVertex3f(box.m_MinX, box.m_MinY, box.m_MinZ);
    glVertex3f(box.m_MinX, box.m_MinY, box.m_MaxZ);

    glVertex3f(box.m_MinX, box.m_MaxY, box.m_MinZ);
    glVertex3f(box.m_MinX, box.m_MaxY, box.m_MaxZ);

    glVertex3f(box.m_MaxX, box.m_MinY, box.m_MinZ);
    glVertex3f(box.m_MaxX, box.m_MaxY, box.m_MinZ);

    glVertex3f(box.m_MaxX, box.m_MinY, box.m_MinZ);
    glVertex3f(box.m_MaxX, box.m_MinY, box.m_MaxZ);

    glVertex3f(box.m_MaxX, box.m_MaxY, box.m_MinZ);
    glVertex3f(box.m_MaxX, box.m_MaxY, box.m_MaxZ);

    glVertex3f(box.m_MaxX, box.m_MinY, box.m_MaxZ);
    glVertex3f(box.m_MaxX, box.m_MaxY, box.m_MaxZ);

    glVertex3f(box.m_MinX, box.m_MinY, box.m_MaxZ);
    glVertex3f(box.m_MaxX, box.m_MinY, box.m_MaxZ);

    glVertex3f(box.m_MinX, box.m_MaxY, box.m_MaxZ);
    glVertex3f(box.m_MaxX, box.m_MaxY, box.m_MaxZ);
    glEnd();
    //End Debug
*/

/*
    //Debug: Render bounding sphere
    CSpheref sphere = m_BoundingSphere;
    glBegin(GL_LINES);
    glColor3f(0.0f, 0.0f, 1.0f);
    glVertex3f(sphere.m_vCenter.m_X - sphere.m_Radius, sphere.m_vCenter.m_Y, sphere.m_vCenter.m_Z);
    glVertex3f(sphere.m_vCenter.m_X + sphere.m_Radius, sphere.m_vCenter.m_Y, sphere.m_vCenter.m_Z);

    glVertex3f(sphere.m_vCenter.m_X, sphere.m_vCenter.m_Y - sphere.m_Radius, sphere.m_vCenter.m_Z);
    glVertex3f(sphere.m_vCenter.m_X, sphere.m_vCenter.m_Y + sphere.m_Radius, sphere.m_vCenter.m_Z);

    glVertex3f(sphere.m_vCenter.m_X, sphere.m_vCenter.m_Y, sphere.m_vCenter.m_Z - sphere.m_Radius);
    glVertex3f(sphere.m_vCenter.m_X, sphere.m_vCenter.m_Y, sphere.m_vCenter.m_Z + sphere.m_Radius);

    glEnd();
    //End Debug

*/
    bool bRender = true;
    if(bVerify)
    {
        sbyte val = frustum.SphereInFrustum(m_BoundingSphere);
        if(val == FRUSTUM_INSIDE)
        {
            bVerify = false;
        }
        else if(val == FRUSTUM_OUTSIDE || !frustum.BoxInFrustum(m_BoundingBox))
        {
            bRender = false;
        }
    }

    if(bRender)
//    if(frustum.BoxInFrustum(m_BoundingBox))
    {
        ++ms_dwCount;
        if(m_Elements.size())
        {
            for(QTElementList::iterator it = m_Elements.begin(); it != m_Elements.end(); ++it)
            {
                (*it)->Render(frustum, bVerify);
            }
        }
        if(m_pChildren)
        {
            m_pChildren[0].Render(frustum, bVerify);
            m_pChildren[1].Render(frustum, bVerify);
            m_pChildren[2].Render(frustum, bVerify);
            m_pChildren[3].Render(frustum, bVerify);
        }
    }
}

template<class T>
void TQTNode<T>::AddObject(const TSmartPointer<T> &pObject)
{
    sbyte target = FitsInto(pObject);

    switch(target)
    {
        case 0:
        case 1:
        case 2:
        case 3:
            m_pChildren[target].AddObject(pObject);
            break;
        case -2:
            m_pParent->AddObject(pObject);
            break;
        default:
            //None fitted, add it to myself
            pObject->SetParentNode(this);
            m_Elements.push_back(pObject);
    }
}

template<class T>
bool TQTNode<T>::RemoveObject(const TSmartPointer<T> &pObject)
{
    //Search for it:
    QTElementList::iterator it(find(m_Elements.begin(), m_Elements.end(), pObject));
    if(it != m_Elements.end())
    {
        (*it)->SetParentNode(NULL);
        m_Elements.erase(it);
        return true;
    }
    if(m_pChildren)
    {
        if(m_pChildren[0].RemoveObject(pObject))
            return true;
        if(m_pChildren[1].RemoveObject(pObject))
            return true;
        if(m_pChildren[2].RemoveObject(pObject))
            return true;
        if(m_pChildren[3].RemoveObject(pObject))
            return true;
    }
    return false;
}

template<class T>
void TQTNode<T>::PickObject(const CFrustum &frustum, const CVector3f &vRayOrigin, const CVector3f &vRayDir,
                             TSmartPointer<IRepresentation> &pRepresentation, float &fHitDistance, CVector3f &vHitPos, bool bVerify)
{
    bool bPickable = true;
    if(bVerify)
    {
        sbyte val = frustum.SphereInFrustum(m_BoundingSphere);
        if(val == FRUSTUM_INSIDE)
        {
            bVerify = false;
        }
        else if(val == FRUSTUM_OUTSIDE || !frustum.BoxInFrustum(m_BoundingBox))
        {
            bPickable = false;
        }
    }

    if(bPickable)
    {
        if(m_Elements.size())
        {
            float fDistance;
			CVector3f tmpPos;
            for(QTElementList::iterator it = m_Elements.begin(); it != m_Elements.end(); ++it)
            {
                if((*it)->IsPickable())
                {                   
                    if((*it)->GetBoundingSphere().RayIntersection(vRayOrigin, vRayDir, fDistance))
                    {
						//Possible hit. Lets make sure... ==> polygon level hit test
						if((*it)->RayIntersection(vRayOrigin, vRayDir, tmpPos, fDistance))
						{
							//Hit! Better than the old one?
							if(fDistance < fHitDistance)
							{
								vHitPos = tmpPos;
								pRepresentation = (*it);
								fHitDistance = fDistance;
							}
						}
                    }
                }
            }
        }
        if(m_pChildren)
        {
            m_pChildren[0].PickObject(frustum, vRayOrigin, vRayDir, pRepresentation, fHitDistance, vHitPos, bVerify);
            m_pChildren[1].PickObject(frustum, vRayOrigin, vRayDir, pRepresentation, fHitDistance, vHitPos, bVerify);
            m_pChildren[2].PickObject(frustum, vRayOrigin, vRayDir, pRepresentation, fHitDistance, vHitPos, bVerify);
            m_pChildren[3].PickObject(frustum, vRayOrigin, vRayDir, pRepresentation, fHitDistance, vHitPos, bVerify);
        }
    }
}

template<class T>
void TQTNode<T>::ObjectChanged(const TSmartPointer<T> &pObject)
{
    sbyte target = FitsInto(pObject);

    switch(target)
    {
        case 0:
        case 1:
        case 2:
        case 3:
            RemoveObject(pObject);
            m_pChildren[target].AddObject(pObject);
            break;
        case -2:
            RemoveObject(pObject);
            m_pParent->AddObject(pObject);
            break;
    }
}

template<class T>
void TQTNode<T>::Destroy()
{
    SAFE_DELETE_ARRAY(m_pChildren);
	m_pStateNode = NULL;
    Clear();
}

template<class T>
void TQTNode<T>::Clear()
{
    m_Elements.clear();
    m_pChildren = NULL;
    m_pParent = NULL;
	
}

template<class T>
sbyte TQTNode<T>::FitsInto(const TSmartPointer<T> &pObject)
{
    float fMinX, fMaxX;
    float fMinZ, fMaxZ;
    CBox3f box(pObject->GetBoundingBox());

    //Find the minimum and maximum values :-(
    float t;
    fMinX = fMaxX = box.m_vPoint.m_X;
    fMinZ = fMaxZ = box.m_vPoint.m_Z;

    t = box.m_vPoint.m_X + box.m_vA.m_X;
    if(t < fMinX)
    {
        fMinX = t;
    }
    else if(t > fMaxX)
    {
        fMaxX = t;
    }
    t = box.m_vPoint.m_Z + box.m_vA.m_Z;
    if(t < fMinZ)
    {
        fMinZ = t;
    }
    else if(t > fMaxZ)
    {
        fMaxZ = t;
    }

    t = box.m_vPoint.m_X + box.m_vB.m_X;
    if(t < fMinX)
    {
        fMinX = t;
    }
    else if(t > fMaxX)
    {
        fMaxX = t;
    }
    t = box.m_vPoint.m_Z + box.m_vB.m_Z;
    if(t < fMinZ)
    {
        fMinZ = t;
    }
    else if(t > fMaxZ)
    {
        fMaxZ = t;
    }

    t = box.m_vPoint.m_X + box.m_vC.m_X;
    if(t < fMinX)
    {
        fMinX = t;
    }
    else if(t > fMaxX)
    {
        fMaxX = t;
    }
    t = box.m_vPoint.m_Z + box.m_vC.m_Z;
    if(t < fMinZ)
    {
        fMinZ = t;
    }
    else if(t > fMaxZ)
    {
        fMaxZ = t;
    }

    t = box.m_vPoint.m_X + box.m_vA.m_X + box.m_vB.m_X;
    if(t < fMinX)
    {
        fMinX = t;
    }
    else if(t > fMaxX)
    {
        fMaxX = t;
    }
    t = box.m_vPoint.m_Z + box.m_vA.m_Z + box.m_vB.m_Z;
    if(t < fMinZ)
    {
        fMinZ = t;
    }
    else if(t > fMaxZ)
    {
        fMaxZ = t;
    }

    t = box.m_vPoint.m_X + box.m_vA.m_X + box.m_vC.m_X;
    if(t < fMinX)
    {
        fMinX = t;
    }
    else if(t > fMaxX)
    {
        fMaxX = t;
    }
    t = box.m_vPoint.m_Z + box.m_vA.m_Z + box.m_vC.m_Z;
    if(t < fMinZ)
    {
        fMinZ = t;
    }
    else if(t > fMaxZ)
    {
        fMaxZ = t;
    }

    t = box.m_vPoint.m_X + box.m_vC.m_X + box.m_vB.m_X;
    if(t < fMinX)
    {
        fMinX = t;
    }
    else if(t > fMaxX)
    {
        fMaxX = t;
    }
    t = box.m_vPoint.m_Z + box.m_vC.m_Z + box.m_vB.m_Z;
    if(t < fMinZ)
    {
        fMinZ = t;
    }
    else if(t > fMaxZ)
    {
        fMaxZ = t;
    }

    t = box.m_vPoint.m_X + box.m_vA.m_X + box.m_vB.m_X + box.m_vC.m_X;
    if(t < fMinX)
    {
        fMinX = t;
    }
    else if(t > fMaxX)
    {
        fMaxX = t;
    }
    t = box.m_vPoint.m_Z + box.m_vA.m_Z + box.m_vB.m_Z + box.m_vC.m_Z;
    if(t < fMinZ)
    {
        fMinZ = t;
    }
    else if(t > fMaxZ)
    {
        fMaxZ = t;
    }

    if(m_pChildren)
    {
        //Which child should it be?
        float fWidth = m_BoundingBox.SizeX()  / 2;
        float fHeight = m_BoundingBox.SizeZ()  / 2;
        if(fMinZ >= m_BoundingBox.m_MinZ &&
           fMaxZ <= m_BoundingBox.m_MinZ + fHeight)
        {
            //Is in child 0 or 1?
            if(fMinX >= m_BoundingBox.m_MinX &&
               fMaxX <= m_BoundingBox.m_MinX + fWidth)
            {
                return 0;
            }
            if(fMinX >= m_BoundingBox.m_MinX + fWidth &&
               fMaxX <= m_BoundingBox.m_MaxX)
            {
                return 1;
            }
        }
        if(fMinZ >= m_BoundingBox.m_MinZ + fHeight &&
           fMaxZ <= m_BoundingBox.m_MaxZ)
        {
            //Is in child 2 or 3?
            if(fMinX >= m_BoundingBox.m_MinX &&
               fMaxX <= m_BoundingBox.m_MinX + fWidth)
            {
                return 2;
            }
            if(fMinX >= m_BoundingBox.m_MinX + fWidth &&
               fMaxX <= m_BoundingBox.m_MaxX)
            {
                return 3;
            }
        }
    }

    //Is ist too large for myself?
    if(m_pParent && (fMinX <  m_BoundingBox.m_MinX ||
                     fMaxX >  m_BoundingBox.m_MaxX ||
                     fMinZ <  m_BoundingBox.m_MinZ ||
                     fMaxZ >  m_BoundingBox.m_MaxZ))
    {
        return -2;
    }

    return -1;
}

#endif
