/****************************************************************************\
*                                                                            *
* 21.02.2006                                                                 *
* CDirectInputMgr - Source File                                              *
*                                                                            *
* (c) Raphael Charwot                                                        *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#define DIRECTINPUT_VERSION         DIRECTINPUT_HEADER_VERSION
#include <dinput.h>
#include "CDirectInputMgr.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/
const dword DI_MOUSE_BUFFER = 32;
const dword DI_KEYBOARD_BUFFER = 16;

#ifndef SAFE_RELEASE
    #define SAFE_RELEASE(p) {if(p) {(p)->Release(); (p) = NULL;}}
#endif

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
bool CDirectInputMgr::Create(HINSTANCE hInstance, const TSmartPointer<IWindow> &pWindow)
{
    if(FAILED(DirectInput8Create(hInstance, DIRECTINPUT_VERSION,
                                 IID_IDirectInput8, (void **) &m_pDI, NULL)))
    {
        LOG(TL_ERROR, "Error: DirectInput8Create failed");
        return false;
    }

    if(FAILED(m_pDI->CreateDevice(GUID_SysMouse, &m_pDIMouse, NULL)))
    {
        LOG(TL_ERROR, "Error: CreateDevice failed for mouse device");
        return false;
    }

    if(FAILED(m_pDIMouse->SetDataFormat(&c_dfDIMouse2)))
    {
        LOG(TL_ERROR, "Error: SetDataFormat failed for mouse device");
        return false;
    }

    if(FAILED(m_pDIMouse->SetCooperativeLevel(pWindow->GetWindowHandle(), DISCL_NONEXCLUSIVE | DISCL_FOREGROUND)))
    {
        LOG(TL_ERROR, "Error: SetCooperativeLevel failed for mouse device");
        return false;
    }

    //Enable buffered input
    DIPROPDWORD dipdw;
    dipdw.diph.dwSize       = sizeof(DIPROPDWORD);
    dipdw.diph.dwHeaderSize = sizeof(DIPROPHEADER);
    dipdw.diph.dwObj        = 0;
    dipdw.diph.dwHow        = DIPH_DEVICE;
    dipdw.dwData            = DI_MOUSE_BUFFER;

    if(FAILED(m_pDIMouse->SetProperty( DIPROP_BUFFERSIZE, &dipdw.diph)))
    {
        LOG(TL_ERROR, "Error: SetProperty (set buffer size) failed for mouse device");
        return false;
    }

    if(FAILED(m_pDI->CreateDevice(GUID_SysKeyboard, &m_pDIKeyboard, NULL)))
    {
        LOG(TL_ERROR, "Error: CreateDevice faild for keyboard device");
        return false;
    }

    if(FAILED(m_pDIKeyboard->SetDataFormat(&c_dfDIKeyboard)))
    {
        LOG(TL_ERROR, "Error: SetDataFormat faild for keyboard device");
        return false;
    }

    if(FAILED(m_pDIKeyboard->SetCooperativeLevel(pWindow->GetWindowHandle(), DISCL_NONEXCLUSIVE | DISCL_FOREGROUND)))
    {
        LOG(TL_ERROR, "Error: SetCooperativeLevel faild for keyboard device");
        return false;
    }

    //Enable buffered input
    dipdw.diph.dwSize       = sizeof(DIPROPDWORD);
    dipdw.diph.dwHeaderSize = sizeof(DIPROPHEADER);
    dipdw.diph.dwObj        = 0;
    dipdw.diph.dwHow        = DIPH_DEVICE;
    dipdw.dwData            = DI_KEYBOARD_BUFFER;

    if(FAILED(m_pDIKeyboard->SetProperty( DIPROP_BUFFERSIZE, &dipdw.diph )))
    {
        LOG(TL_ERROR, "Error: SetProperty (set buffer size) failed for keyboard device");
        return false;
    }
    
    m_pDIMouse->Acquire();
    m_bVerifyMouse = true;
    m_pDIKeyboard->Acquire();
    m_bVerifyKeyboard = true;

    m_bShowCursor = true;
    m_bLockCursor = false;

    m_pWindow = pWindow;
    m_pWindow->RegisterHandler(this);

    return true;
}

void CDirectInputMgr::Update()
{
    assert(m_pDI && "Error: CDirectInputMgr::Update called before ::Create");

    UpdateMouse();
    UpdateKeyboard();
}

void CDirectInputMgr::GetCursorPos(sdword &sdwPosX, sdword &sdwPosY) const
{
    POINT p;
    ::GetCursorPos(&p);
    //Get Position in client space
    ::ScreenToClient(m_pWindow->GetWindowHandle(), &p);

    sdwPosX = p.x;
    sdwPosY = p.y;
}

void CDirectInputMgr::GetNormalizedCursorPos(float &fPosX, float & fPosY) const
{
    sdword sdwPosX, sdwPosY;
    word wWidth, wHeight;
    GetCursorPos(sdwPosX, sdwPosY);
    m_pWindow->GetSize(wWidth, wHeight);
    fPosX = float(sdwPosX) / wWidth;
    fPosY = float(sdwPosY) / wHeight;
}


void CDirectInputMgr::LockCursor(bool bLock)
{
    if(m_bLockCursor != bLock)
    {
        if(bLock)
        {
            RECT r;
            if(::GetClientRect(m_pWindow->GetWindowHandle(), &r))
            {
                POINT p;
                p.x = r.left;
                p.y = r.top;
                ::ClientToScreen(m_pWindow->GetWindowHandle(), &p);
                r.left = p.x;
                r.top = p.y;
                p.x = r.right;
                p.y = r.bottom;
                ::ClientToScreen(m_pWindow->GetWindowHandle(), &p);
                r.right = p.x;
                r.bottom = p.y;
                ::ClipCursor(&r);
            }
        }
        else
        {
            ::ClipCursor(NULL);
        }
        m_bLockCursor = bLock;
    }
}

void CDirectInputMgr::ShowCursor(bool bShow)
{
    if(m_bShowCursor != bShow)
    {
        ::ShowCursor(bShow);
        m_bShowCursor = bShow;
    }
}

void CDirectInputMgr::Destroy()
{
    LockCursor(false);
    ShowCursor(true);
    if(m_pWindow)
    {
        m_pWindow->UnregisterHandler(this);
        m_pWindow = NULL;
    }

    SAFE_RELEASE(m_pDIKeyboard);
    SAFE_RELEASE(m_pDIMouse);
    SAFE_RELEASE(m_pDI);

    Clear();        //Not necessary but safer
}

void CDirectInputMgr::OnWndChar(char c)
{
    HandlerListIterator j;

    for(j = m_InputHandlers.begin(); j != m_InputHandlers.end(); ++j)
    {
        if((*j)->OnChar(c))
            return;
    }
}

void CDirectInputMgr::OnWndActivate(bool bActive)
{
	if(bActive)
	{
		if(m_bLockCursor)
		{
			LockCursor(false);
			LockCursor(true);
		}
	}
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/
void CDirectInputMgr::UpdateMouse()
{
    DIDEVICEOBJECTDATA dodMouse[DI_MOUSE_BUFFER];
    dword              dwElements(DI_MOUSE_BUFFER);
    HRESULT            hr;

    hr = m_pDIMouse->GetDeviceData(sizeof(DIDEVICEOBJECTDATA),
                                   dodMouse, &dwElements, 0);
    if(FAILED(hr))
    {
        do
            hr = m_pDIMouse->Acquire();
        while(hr == DIERR_INPUTLOST);

        //Didn't own the mouse ==> Return and throw away the data
        m_bVerifyMouse = true;
        return;
    }
    byte bKey;
    sdword sdwX(0), sdwY(0), sdwZ(0);
    HandlerListIterator j;

    for(dword i = 0; i < dwElements; ++i) 
    {        
        switch(dodMouse[i].dwOfs)
        {
            case DIMOFS_BUTTON0:
                bKey = 0;
                break;
            case DIMOFS_BUTTON1:
                bKey = 1;
                break;
            case DIMOFS_BUTTON2:
                bKey = 2;
                break;
            case DIMOFS_BUTTON3:
                bKey = 3;
                break;
            case DIMOFS_BUTTON4:
                bKey = 4;
                break;
            case DIMOFS_BUTTON5:
                bKey = 5;
                break;
            case DIMOFS_BUTTON6:
                bKey = 6;
                break;
            case DIMOFS_BUTTON7:
                bKey = 7;
                break;
            case DIMOFS_X:
                sdwX += (sdword) dodMouse[i].dwData;
                break;
            case DIMOFS_Y:
                sdwY += (sdword) dodMouse[i].dwData;
                break;
            case DIMOFS_Z:
                sdwZ += (sdword) dodMouse[i].dwData;
                break;
            default:
                LOG(TL_ERROR | TL_STDOUT, "Warning: Unknown mouse event: %lx", dodMouse[i]);
        }

        switch(dodMouse[i].dwOfs)
        {
            case DIMOFS_BUTTON0:
            case DIMOFS_BUTTON1:
            case DIMOFS_BUTTON2:
            case DIMOFS_BUTTON3:
            case DIMOFS_BUTTON4:
            case DIMOFS_BUTTON5:
            case DIMOFS_BUTTON6:
            case DIMOFS_BUTTON7:
                for(j = m_InputHandlers.begin(); j != m_InputHandlers.end(); ++j)
                {
                    if(dodMouse[i].dwData & 0x80)
                    {
                        m_bMouseStatus[bKey] = true;
                        if((*j)->OnMouseKeyDown(bKey))
                            break;
                    }
                    else
                    {
                        m_bMouseStatus[bKey] = false;
                        if((*j)->OnMouseKeyUp(bKey))
                            break;
                    }
                }
        }
    }
    if(sdwX != 0 || sdwY != 0 || sdwZ != 0)
    {
        //Mouse hase moved
        for(j = m_InputHandlers.begin(); j != m_InputHandlers.end(); ++j)
        {
            if((*j)->OnMouseMove(sdwX, sdwY, sdwZ))
                break;
        }
    }

    if(hr == DI_BUFFEROVERFLOW || m_bVerifyMouse)
    {
        VerifyMouse();
    }
}

void CDirectInputMgr::UpdateKeyboard()
{
    DIDEVICEOBJECTDATA didod[DI_KEYBOARD_BUFFER];
    dword              dwElements(DI_KEYBOARD_BUFFER);
    HRESULT            hr;

    hr = m_pDIKeyboard->GetDeviceData(sizeof(DIDEVICEOBJECTDATA),
                                      didod, &dwElements, 0);
    if(FAILED(hr))
    {
        do
        hr = m_pDIKeyboard->Acquire();
        while(hr == DIERR_INPUTLOST);

        //Didn't own the keyboard ==> Return and throw away the data
        m_bVerifyKeyboard = true;
        return;
    }

    HandlerListIterator j;

    for(dword i = 0; i < dwElements; ++i) 
    {
        for(j = m_InputHandlers.begin(); j != m_InputHandlers.end(); ++j)
        {
            if(didod[i].dwData & 0x80)
            {
                m_bMouseStatus[didod[i].dwOfs] = true;
                if((*j)->OnKeyDown(didod[i].dwOfs))
                    break;
            }
            else
            {
                m_bMouseStatus[didod[i].dwOfs] = false;
                if((*j)->OnKeyUp(didod[i].dwOfs))
                    break;
            }
        }
    }

    if(hr == DI_BUFFEROVERFLOW || m_bVerifyKeyboard)
    {
        VerifyKeyboard();
    }
}

bool CDirectInputMgr::VerifyMouse()
{
    //Verify that all buttons are in the state that we believe they are
    DIMOUSESTATE2 dims;
    ZeroMemory(&dims, sizeof(dims));
    if(FAILED(m_pDIMouse->GetDeviceState(sizeof(dims), &dims))) 
    {
        m_bVerifyMouse = true;
        return false;
    }

    HandlerListIterator j;
    for(byte i = 0; i < 8; ++i)
    {
        if(dims.rgbButtons[i] & 0x80 && !m_bMouseStatus[i])
        {
            for(j = m_InputHandlers.begin(); j != m_InputHandlers.end(); ++j)
            {
                if((*j)->OnMouseKeyDown(i))
                    break;
            }
            m_bMouseStatus[i] = true;
        }
        else if(!(dims.rgbButtons[i] & 0x80) && m_bMouseStatus[i])
        {
            for(j = m_InputHandlers.begin(); j != m_InputHandlers.end(); ++j)
            {
                if((*j)->OnMouseKeyUp(i))
                    break;
            }
            m_bMouseStatus[i] = false;
        }
    }

    m_bVerifyMouse = false;

    return true;
}

bool CDirectInputMgr::VerifyKeyboard()
{
    //Verify that all buttons are in the state that we believe they are
    BYTE diks[256];
    ZeroMemory(diks, sizeof(diks[0]) * 256);
    if(FAILED(m_pDIKeyboard->GetDeviceState(sizeof(diks), &diks))) 
    {
        m_bVerifyKeyboard = true;
        return false;
    }

    HandlerListIterator j;
    for(dword i = 0; i < 256; ++i)
    {
        if(diks[i] & 0x80 && !m_bKeyboardStatus[i])
        {
            for(j = m_InputHandlers.begin(); j != m_InputHandlers.end(); ++j)
            {
                if((*j)->OnKeyDown(i))
                    break;
            }
            m_bKeyboardStatus[i] = true;
        }
        else if(!(diks[i] & 0x80) && m_bKeyboardStatus[i])
        {
            for(j = m_InputHandlers.begin(); j != m_InputHandlers.end(); ++j)
            {
                if((*j)->OnKeyUp(i))
                    break;
            }
            m_bKeyboardStatus[i] = false;
        }
    }

    m_bVerifyKeyboard = false;

    return true;
}

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
void CDirectInputMgr::Clear()
{
    m_pDI = NULL;
    m_pDIMouse = NULL;
    m_pDIKeyboard = NULL;

    m_pWindow = NULL;

    ZeroMemory(m_bMouseStatus, sizeof(m_bMouseStatus[0]) * 8);
    ZeroMemory(m_bKeyboardStatus, sizeof(m_bKeyboardStatus[0]) * 256);
    m_bVerifyMouse = true;
    m_bVerifyKeyboard = true;
}

