/****************************************************************************\
*                                                                            *
* 27.03.2006                                                                 *
* CTGALoader - Source File                                                   *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CTGALoader.h"
#include <cstdio>

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

//TGA File Header
typedef struct TGAHead{
	BYTE Header[12];					
} TGAHead;

//TGA STRUCT
typedef struct TGA{
	BYTE				header[6];		//First 6 Useful Bytes From The Header
	unsigned int		bytesPerPixel;	//Holds Number Of Bytes Per Pixel Used In The TGA File
	unsigned int		imageSize;		//Used To Store The Image Size When Setting Aside Ram
	unsigned int		temp;			//Temporary Variable
	unsigned int		type;	
	dword       		Height;			//Height of Image
	dword	        	Width;			//Width ofImage
	dword		        Bpp;			//Bits Per Pixel
} TGA;

//global
BYTE uTGAcompare[12] = {0,0, 2,0,0,0,0,0,0,0,0,0};	//Uncompressed TGA Header
BYTE cTGAcompare[12] = {0,0,10,0,0,0,0,0,0,0,0,0};	//Compressed TGA Header

//static
static TGAHead head; //TGA header
static TGA     tga;	 //TGA image data

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/

bool CTGALoader::LoadImage(std::string &filename,TSmartPointer<CImageInfo> imageInfo) 
{
	FILE * fTGA;															 
	fTGA = fopen(filename.c_str(), "rb");								             

	if(fTGA == NULL){
        LOG(TL_ERROR, "Could not open texture file: %s", filename.c_str());                    	
		return false;														
	}

	if(fread(&head, sizeof(TGAHead), 1, fTGA) == 0){
		LOG(TL_ERROR,"Could not read file header: %s", filename.c_str());		
		if(fTGA != NULL){
			fclose(fTGA);													
		}
		return false;														
	}

	bool compressed;

	if(memcmp(uTGAcompare, &head, sizeof(TGAHead)) == 0){																		
		compressed = false;	
	}else if(memcmp(cTGAcompare, &head, sizeof(TGAHead)) == 0){																		
		compressed = true;		
	}else{
		LOG(TL_ERROR,"Invalid Image - TGA file be type 2 or type 10 ");	
		fclose(fTGA);
		return false;																
	}

	if(fread(tga.header, sizeof(tga.header), 1, fTGA) == 0){										
		LOG(TL_ERROR,"Could not read info header");		
		if(fTGA != NULL){
			fclose(fTGA);													
		}
		return false;														
	}	

	//determine The TGA Width	(highbyte*256+lowbyte)
	imageInfo->m_Width  = tga.header[1] * 256 + tga.header[0];
	//determine The TGA Height	(highbyte*256+lowbyte)
	imageInfo->m_Height = tga.header[3] * 256 + tga.header[2];	
	//determine the alpha component
	imageInfo->m_bAlpha = (tga.header[4]==32);
	//determine bits per pixel
	imageInfo->m_Bpp = tga.header[4];


	//copy into local structure	
	tga.Width  = imageInfo->m_Width;
	tga.Height = imageInfo->m_Height;
	tga.Bpp	=    imageInfo->m_Bpp;			

	//check
	if((tga.Width <= 0) || (tga.Height <= 0) || ((tga.Bpp != 24) && (tga.Bpp !=32))){
		LOG(TL_ERROR,"Invalid texture information");	
		if(fTGA != NULL){
			fclose(fTGA);													
		}
		return false;														
	}

	//Compute the number of BYTES per pixel
	tga.bytesPerPixel = (tga.Bpp / 8);
	//Compute the total amout ofmemory needed to store data
	tga.imageSize = (tga.bytesPerPixel * tga.Width * tga.Height);

	//Allocate memory
	imageInfo->m_pData = new BYTE [tga.imageSize];										

	//If no space was allocated
	if(imageInfo->m_pData == NULL){
		LOG(TL_ERROR,"Could not allocate memory for image");	
		fclose(fTGA);														
		return false;														
	}

	if(compressed){
		return LoadCompressedTGA(fTGA,tga.Width,tga.Height,imageInfo->m_bAlpha,imageInfo->m_pData);
	}else{
        return LoadUncompressedTGA(fTGA,tga.Width,tga.Height,imageInfo->m_bAlpha,imageInfo->m_pData);	
	}

}

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/

//Load a uncompressed TGA
bool CTGALoader::LoadUncompressedTGA(FILE *fTGA, dword &width, dword &height, bool &alpha, BYTE *bits)	
{
	//Attempt to read image data
	if(fread(bits, 1, tga.imageSize, fTGA) != tga.imageSize){
		LOG(TL_ERROR,"Could not read image data");		
		delete [] bits;													
		fclose(fTGA);														
		return false;														
	}
	//Byte Swapping
	//for(int cswap = 0; cswap < (int)tga.imageSize; cswap += tga.bytesPerPixel){
		//bits[cswap] ^= bits[cswap+2] ^= bits[cswap] ^= bits[cswap+2];
	//}

	//Close file
	fclose(fTGA);															
	return true;	
}

//Load a compressed TGA
bool CTGALoader::LoadCompressedTGA(FILE *fTGA, dword &width, dword &height, bool &alpha, BYTE *bits)		
{ 
	
	//Nuber of pixels in the image
	unsigned int pixelcount	= tga.Height * tga.Width;
	//Current pixel being read
	unsigned int currentpixel = 0;
	//Current byte 
	unsigned int currentbyte = 0;
	//Storage for 1 pixel
	BYTE * colorbuffer = new BYTE [tga.bytesPerPixel];	  

	do{
		//Storage for "chunk" header
		BYTE chunkheader = 0;											

		// Read in the 1 byte header
		if(fread(&chunkheader, sizeof(BYTE), 1, fTGA) == 0){
			LOG(TL_ERROR,"Could not read RLE header");	
			if(fTGA != NULL){
				fclose(fTGA);												
			}
			if(bits != NULL){
				delete [] bits;											
			}
			return false;													
		}

		// If the header is < 128, it means the that is the number of RAW color packets minus 1
		// that follow the header
		if(chunkheader < 128){	
			// add 1 to get number of following color values
			chunkheader++;

			// Read RAW color values
			for(short counter = 0; counter < chunkheader; counter++){
				// Try to read 1 pixel
				if(fread(colorbuffer, 1, tga.bytesPerPixel, fTGA) != tga.bytesPerPixel){
					// IF we cant, display an error
					LOG(TL_ERROR,"Could not read image data");		

					if(fTGA != NULL){
						fclose(fTGA);													
					}

					if(colorbuffer != NULL){
						delete [] colorbuffer;											
					}

					if(bits != NULL){
						delete [] bits;												
					}

					return false;														
				}
				// write to memory
				// Flip R and B vcolor values around in the process 
				bits[currentbyte		] = colorbuffer[2];								
				bits[currentbyte + 1	] = colorbuffer[1];
				bits[currentbyte + 2	] = colorbuffer[0];

				// if its a 32 bpp image
				if(tga.bytesPerPixel == 4){
					// copy the 4th byte
					bits[currentbyte + 3] = colorbuffer[3];							
				}

				// Increase thecurrent byte by the number of bytes per pixel
				currentbyte += tga.bytesPerPixel;										
				// Increase current pixel by 1
				currentpixel++;															

				// Make sure we havent read too many pixels
				if(currentpixel > pixelcount)											
				{
					LOG(TL_ERROR,"Too many pixels read");			    

					if(fTGA != NULL)													
					{
						fclose(fTGA);													
					}	

					if(colorbuffer != NULL)												
					{
						delete [] colorbuffer;											
					}

					if(bits != NULL)													
					{
						delete [] bits;												
					}

					return false;														
				}
			}
		}
		//chunkheader > 128 RLE data, next color reapeated chunkheader - 127 times
		else																			
		{
			//Subteact 127 to get rid of the ID bit
			chunkheader -= 127;															
			
			//Attempt to read following color values
			if(fread(colorbuffer, 1, tga.bytesPerPixel, fTGA) != tga.bytesPerPixel)		
			{	
				LOG(TL_ERROR,"Could not read from file");			

				if(fTGA != NULL)														
				{
					fclose(fTGA);														
				}

				if(colorbuffer != NULL)													
				{
					delete [] colorbuffer;												
				}

				if(bits != NULL)														
				{
					delete [] bits;													
				}

				return false;															
			}

			// copy the color into the image data as many times as dictated by the header
			for(short counter = 0; counter < chunkheader; counter++)					
			{		
				// switch R and B bytes areound while copying
				bits[currentbyte		] = colorbuffer[2];								
				bits[currentbyte + 1	] = colorbuffer[1];
				bits[currentbyte + 2	] = colorbuffer[0];

				// If TGA images is 32 bpp
				if(tga.bytesPerPixel == 4)												
				{
					// Copy 4th byte
					bits[currentbyte + 3] = colorbuffer[3];							
				}

				// Increase current byte by the number of bytes per pixel
				currentbyte += tga.bytesPerPixel;										
				// Increase pixel count by 1
				currentpixel++;															

				// Make sure we havent written too many pixels
				if(currentpixel > pixelcount)											
				{
					LOG(TL_ERROR,"Too many pixels read");         		

					if(fTGA != NULL)													
					{
						fclose(fTGA);													
					}	

					if(colorbuffer != NULL)												
					{
						delete [] colorbuffer;											
					}

					if(bits != NULL)													
					{
						delete [] bits;												
					}

					return false;														
				}
			}
		}
	}

	// Loop while there are still pixels left
	while(currentpixel < pixelcount);													

	//Byte Swapping
	for(int cswap = 0; cswap < (int)tga.imageSize; cswap += tga.bytesPerPixel){
		bits[cswap] ^= bits[cswap+2] ^= bits[cswap] ^= bits[cswap+2];
	}

	SAFE_DELETE_ARRAY(colorbuffer);

	// Close the file
	fclose(fTGA);
	return true;																		
}