/****************************************************************************\
*                                                                            *
* 27.03.2006                                                                 *
* COGLModelGeometry - Source File                                            *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "COGLModelGeometry.h"
#include "COGLModelFactory.h"
#include "COGLRenderer.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/

bool COGLModelGeometry::Create(COGLRenderer *pRenderer, COGLModelFactory *pHost, CGeometryInfo *pGeoInfo)
{    
    m_pRenderer = pRenderer;
	m_pHost     = pHost;

    for(std::vector<SMaterial>::iterator itM = pGeoInfo->m_Materials.begin(); itM != pGeoInfo->m_Materials.end(); ++itM)
    {
        m_Materials.push_back(new COGLMaterial());
	}

    for(std::vector<STexture>::iterator itT = pGeoInfo->m_Textures.begin() ; itT != pGeoInfo->m_Textures.end() ; ++itT)
    {
		m_Textures.push_back(new COGLTexture());
	}

    for(std::vector<SGeoGroup *>::iterator itG = pGeoInfo->m_pGeoGroups.begin() ; itG != pGeoInfo->m_pGeoGroups.end() ; ++itG)
    {
		dword numVertices      = (dword) (*itG)->m_Vertices.size();
		dword numIndices       = (dword) (*itG)->m_Indices.size();
		bool  hasVC            =         (*itG)->m_Colors.size() > 0;
		byte  numTextureLayers = (byte)  (*itG)->m_bNumTexLayers;
		m_TriangleGroups.push_back(new COGLTriangleGroup(this,numVertices,numIndices,numTextureLayers,hasVC));
    }
    return true;
}


bool COGLModelGeometry::RayIntersection(const CVector3f &vPos, const CVector3f &vRot, const CVector3f &vOrigin, const CVector3f &vDir, CVector3f &vIntersectionPos, float &fIntersectionDistance) const
{
    fIntersectionDistance = cg_fMaxIntersectionDist;
	bool bHit = false;

	//Transforming origin und direction into local space
	CMatrix44 rotM;
	rotM.RotationZXY(-vRot);
	CVector3f relOrigin(rotM * (vOrigin - vPos));
	CVector3f relDir(rotM * vDir);

    for(std::vector<COGLTriangleGroup *>::const_iterator it = m_TriangleGroups.begin(); it != m_TriangleGroups.end(); ++it)
    {
        bHit = bHit | (*it)->RayIntersection(relOrigin, relDir, vIntersectionPos, fIntersectionDistance);
    }

	if(bHit)
	{
		rotM.RotationZXY(vRot);
		vIntersectionPos = (rotM * vIntersectionPos) + vPos;
	}

    return bHit;
}


void COGLModelGeometry::Render()
{
	m_pRenderer->Render(this);
}


const CSpheref COGLModelGeometry::GetBoundingSphere()
{
    CSpheref sphere;

    std::vector<COGLTriangleGroup *>::iterator it;
    dword dwNumVertices = 0;
    dword dwVertex;
    sphere.m_vCenter.Set(0.0f, 0.0f, 0.0f);
    //ber alle Vertices loopen und Center ausrechnen
    for(it = m_TriangleGroups.begin(); it != m_TriangleGroups.end(); ++it)
    {
        for(dwVertex = 0; dwVertex < (*it)->m_NumVertices; ++dwVertex)
        {
            sphere.m_vCenter += (*it)->m_pCoordArray.m_DataArray[dwVertex];
            ++dwNumVertices;
        }
    }
    sphere.m_vCenter /= (float) dwNumVertices;
    sphere.m_Radius = 0.0f;
    float fDistSqr;
    //ber alle Vertices loopen, jeden transformieren und maximale Distanz ausrechnen
    for(it = m_TriangleGroups.begin(); it != m_TriangleGroups.end(); ++it)
    {
        for(dwVertex = 0; dwVertex < (*it)->m_NumVertices; ++dwVertex)
        {
            fDistSqr = ((*it)->m_pCoordArray.m_DataArray[dwVertex] - sphere.m_vCenter).LengthSqr();
            if(fDistSqr > sphere.m_Radius)
                sphere.m_Radius = fDistSqr;
        }
    }
    sphere.m_Radius = (float) sqrt(sphere.m_Radius);

    return sphere;
}

const CAABox3f COGLModelGeometry::GetAABoundingBox()
{
    CAABox3f box;

    //ber alle Vertices loopen und die Distanzen in jede Richtung messen
    std::vector<COGLTriangleGroup *>::iterator it;
    dword dwVertex;
    CVector3f vec;
    bool bFirst = true;
    for(it = m_TriangleGroups.begin(); it != m_TriangleGroups.end(); ++it)
    {
        for(dwVertex = 0; dwVertex < (*it)->m_NumVertices; ++dwVertex)
        {
            vec = (*it)->m_pCoordArray.m_DataArray[dwVertex];
            if(bFirst)
            {
                box.m_MinX = box.m_MaxX = vec.m_X;
                box.m_MinY = box.m_MaxY = vec.m_Y;
                box.m_MinZ = box.m_MaxZ = vec.m_Z;
                bFirst = false;
            }
            else
            {
                if(vec.m_X < box.m_MinX)
                {
                    box.m_MinX = vec.m_X;
                }
                else if(vec.m_X > box.m_MaxX)
                {
                    box.m_MaxX = vec.m_X;
                }

                if(vec.m_Y < box.m_MinY)
                {
                    box.m_MinY = vec.m_Y;
                }
                else if(vec.m_Y > box.m_MaxY)
                {
                    box.m_MaxY = vec.m_Y;
                }

                if(vec.m_Z < box.m_MinZ)
                {
                    box.m_MinZ = vec.m_Z;
                }
                else if(vec.m_Z > box.m_MaxZ)
                {
                    box.m_MaxZ = vec.m_Z;
                }
            }
        }
    }

    return box;
}

COGLTriangleGroup * const COGLModelGeometry::getTriangleGroup(dword nmb){
	if( nmb >= 0 && nmb < (int)m_TriangleGroups.size() ){
		return m_TriangleGroups[nmb];
	}else{
		return NULL;
	}
}

COGLMaterial * const COGLModelGeometry::getMaterial(dword nmb){
	if( nmb >= 0 && nmb < (int)m_Materials.size() ){
		return m_Materials[nmb];
	}else{
		return NULL;
	}
}

COGLTexture * const COGLModelGeometry::getTexture(dword nmb){
	if( nmb >= 0 && nmb < (dword)m_Textures.size() ){
		return m_Textures[nmb];
	}else{
		return NULL;
	}
}

COGLModelGeometry::~COGLModelGeometry()
{
	m_pRenderer->Free(this);
	m_pRenderer = NULL;
	m_pHost = NULL;

    for(std::vector<COGLTriangleGroup *>::iterator it = m_TriangleGroups.begin(); it != m_TriangleGroups.end(); ++it)
    {
        SAFE_DELETE((*it));
    }
    for(std::vector<COGLMaterial *>::iterator it = m_Materials.begin(); it != m_Materials.end(); ++it)
    {
        SAFE_DELETE((*it));
    }
    for(std::vector<COGLTexture *>::iterator it = m_Textures.begin(); it != m_Textures.end(); ++it)
    {
        SAFE_DELETE((*it));
    }
	
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/

/****************************************************************************\
* COGLTEXTURE                                                                *
\****************************************************************************/
COGLTexture::COGLTexture(){
	m_glTexBufID = 0;
	m_target     = 0;
	m_minFilter  = 0;
	m_maxFilter  = 0;
	m_bMipMap    = false;
}

dword COGLTexture::getTexBufID(){
	return m_glTexBufID;
}

void COGLTexture::setTexBufID(dword ID){
	m_glTexBufID = ID;
}

const std::string & COGLTexture::getFilename(){
	return m_fileName;
}

void COGLTexture::setFileName(const std::string & fileName){
	m_fileName = fileName;
}