/****************************************************************************\
*                                                                            *
* 24.02.2006                                                                 *
* CClock - Source File                                                       *
*                                                                            *
* (c) Raphael Charwot                                                        *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <algorithm>
#include "CClock.h"
#include <Misc/IObserver.h>

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
CClock::CClock(const TSmartPointer<ITimeSource> &pTimeSource) : 
    m_pTimeSource (NULL),
    m_dCurrentTime (0),
    m_dFrameTime (0),
    m_dSourceLastValue (0)
{
    SetTimeSource(pTimeSource);
    SetFiltering(1);
    m_dFrameTime = GetPredictedFrameDuration();
}

void CClock::SetTimeSource(const TSmartPointer<ITimeSource> &pTimeSource)
{
    m_pTimeSource = pTimeSource;
    assert(bool(m_pTimeSource) && "Error: Time source is NULL in CClock::SetTimeSource");

    m_dSourceLastValue = m_pTimeSource->GetTime();
}

void CClock::FrameStep()
{
    AddToFrameHistory(GetExactLastFrameDuration());

    m_dCurrentTime += m_dFrameTime = GetPredictedFrameDuration();

    ObserverList::iterator it;
    for (it = m_Observers.begin(); it != m_Observers.end(); ++it)
        (*it)->Notify();
}

void CClock::SetFiltering(byte bFrameWindow, double dFrameDefault, double dFrameLimit)
{
    m_bFrameFilteringWindow = (bFrameWindow > 1) ? bFrameWindow : 1;
    m_FrameDurationHistory.clear();
    m_FrameDurationHistory.push_back(dFrameDefault);
    m_dFrameLimit = dFrameLimit;
}

void CClock::AddObserver(IObserver *pObserver)
{
    if (pObserver != NULL)
        m_Observers.push_back(pObserver);
}

void CClock::RemoveObserver(IObserver *pObserver)
{
    m_Observers.erase(std::remove(m_Observers.begin(),m_Observers.end(), pObserver), m_Observers.end());
}

CClock::~CClock()
{
    m_Observers.clear();
    m_pTimeSource = NULL;
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
double CClock::GetExactLastFrameDuration()
{
    double dSourceTime(m_pTimeSource->GetTime());

    double dFrameDuration = dSourceTime - m_dSourceLastValue;
    if(dFrameDuration > m_dFrameLimit)  //Ausreier! Wird ignoriert
        dFrameDuration = m_FrameDurationHistory.back();
    m_dSourceLastValue = dSourceTime;
    return dFrameDuration;
}

void CClock::AddToFrameHistory(double dExactFrameDuration)
{
    m_FrameDurationHistory.push_back(dExactFrameDuration);
    if (m_FrameDurationHistory.size() > m_bFrameFilteringWindow)
        m_FrameDurationHistory.pop_front();
}

double CClock::GetPredictedFrameDuration() const
{
    double dTotalFrameTime = 0;

    std::deque<double>::const_iterator it(m_FrameDurationHistory.begin());
    while(it != m_FrameDurationHistory.end())
    {
        dTotalFrameTime += *it;
        ++it;
    }
    return dTotalFrameTime/m_FrameDurationHistory.size();
}