/****************************************************************************\
*                                                                            *
* 15.03.2006                                                                 *
* CKillBirdApp - Source File                                                 *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CKillBirdApp.h"
#include "CSettings.h"
#include <Display/Windows/CW32Window.h>
#include "CEngineFactory.h"
#include <Display/CQTSceneGraph.h>
#include <Display/Gui/CGuiMgr.h>
#include <Timer/CW32TimeSource.h>

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/
#define SETTINGS_FILENAME "settings.ini"

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/
//Initialization stuff
bool CKillBirdApp::Create()
{
    //Settings laden
    CSettings::CreateInstance();
	if(!CSettings::InstancePtr()->Load(SETTINGS_FILENAME))
    {
        LOG(TL_ERROR, "Error: CSettings::Load(%s) failed in CKillBirdApp::Create", SETTINGS_FILENAME);
        return false;
	}

    //Fenster erstellen
    if(!CreateWindows())
    {
        LOG(TL_ERROR, "Error: CreateWindows() failed in CKillBirdApp::Create");
        return false;
    }

	//EngineFactory erstellen
	CEngineFactory::CreateInstance();

	//Renderer erstellen
    if(!SetupRenderEngine())
	{
		LOG(TL_ERROR, "Error: SetupRenderEngine() failed in CKillBirdApp::Create");
		return false;
	}

    //InputMgr erstellen
    TSmartPointer<IInputMgr> pInputMgr = CEngineFactory::InstancePtr()->GetInputMgr();
    if(!pInputMgr->Create(m_hInstance, m_pWindow))
    {
        LOG(TL_ERROR, "Error: pInputMgr->Create failed in CKillBirdApp::Create");
        return false;
    }

	//SoundMgr erstellen
	if(! CEngineFactory::InstancePtr()->GetSoundMgr()->Create()){
		LOG(TL_ERROR, "Error: ISoundMgr()->Create() failed in CKillBirdApp::Create");
		return false;
	}

	//Debug:
    SetPriority(100);
    pInputMgr->RegisterHandler(this);
    //End debug

    //Clock erstellen
    m_pClock = new CClock(new CW32TimeSource);
    m_pClock->SetFiltering(5);
	
    //GuiMgr erstellen
    TSmartPointer<CGuiMgr> pGuiMgr = CEngineFactory::InstancePtr()->GetGuiMgr();
    if(!pGuiMgr->Create(m_pWindow, pInputMgr, new CTimer(m_pClock), 1000, 1000))
    {
        LOG(TL_ERROR, "Error: pGuiMgr->Create failed in CKillBirdApp::Create");
        return false;
    }

    //PhysicMgr erstellen
    TSmartPointer<CPhysXMgr> pPhysXMgr = CEngineFactory::InstancePtr()->GetPhysicsMgr();
    if(!pPhysXMgr->Create())
    {
        LOG(TL_ERROR, "Error: pPhysXMgr->Create failed in CKillBirdApp::Create");
        return false;
    }

    if(!m_GameCtrl.Create(m_pClock))
    {
        LOG(TL_ERROR, "Error: m_GameCtrl.Create failed in CKillBirdApp::Create");
        return false;
    }

	m_pWindow->Show(true);
    m_bShutdown = false;
    pInputMgr->LockCursor(true);

	return true;
}

//Frame updating/drawing, etc.
bool CKillBirdApp::Update()
{
    m_pClock->FrameStep();
	if(!m_GameCtrl.Update())
        return false;

    return !m_bShutdown;
}

//Destruction stuff
void CKillBirdApp::Destroy()
{
    m_GameCtrl.Destroy();

    CEngineFactory::DestroyInstance();

    m_pClock = NULL;

	if(m_pWindow)
    {
        m_pWindow->UnregisterHandler(this);
        m_pWindow->Destroy();
        m_pWindow = NULL;
    }

    CSettings::DestroyInstance();
    Clear();
}

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/

//Resetting, nullifying, etc.
void CKillBirdApp::Clear()
{
    m_pWindow = NULL;
    m_bShutdown = true;
}

//Erstellt das Hauptfenster und das Konsolenfenster
bool CKillBirdApp::CreateWindows()
{
    std::string settings[4];
    sword swPosX, swPosY;
    if(!CSettings::InstancePtr()->GetValue("DISPLAY","ConsoleX",settings[0]) ||
       !CSettings::InstancePtr()->GetValue("DISPLAY","ConsoleY",settings[1]))
    {
        LOG(TL_ERROR, "Warning: Couldn't get Console position from \"settings.ini\"! Using default values.");
        swPosX = 0;
        swPosY = 0;
    }
    else
    {
        swPosX = (sword) atoi(settings[0].c_str());
        swPosY = (sword) atoi(settings[1].c_str());
    }

	//Konsolenfenster (fr Logging, etc.)
	if(!m_Console.Create("Kill Bird: Volume 1 - Console", 80, 25, swPosX, swPosY))
	{
		LOG(TL_ERROR, "Error: m_Console.Create failed in CKillBirdApp::CreateWindows");
		return false;
	}

	//Hauptfenster
    m_pWindow = new CW32Window;
	
	//Hauptfenster Parameter von File
	std::string title;
	int  hRes;
	int  vRes;
	bool fullscreen;
	
	int nmbValid = 0;
		
	if(CSettings::Instance().GetValue("DISPLAY","title",settings[0])){
		if(settings[0].size() > 0 && settings[0].size() < 50)
			title = settings[0];
			nmbValid++; 
	}
	if(CSettings::Instance().GetValue("DISPLAY","hRes",settings[1])){
		hRes = atoi(settings[1].c_str());
		if(hRes <= 1280 && hRes >= 640){
			nmbValid++;
		}
	}
	if(CSettings::Instance().GetValue("DISPLAY","vRes",settings[2])){
		vRes = atoi(settings[2].c_str());
		if(vRes <= 1024 && vRes >= 480){
			nmbValid++;
		}
	}
	if(CSettings::Instance().GetValue("DISPLAY","fullscreen",settings[3])){
		fullscreen = settings[3].compare("true") == 0  ? true : false;
		nmbValid++;
	}

    //Hauptfenster erstellen
	if(nmbValid == 4){
		
		//erstelle Fenster mit Settings vom file
		if(!m_pWindow->Create(m_hInstance,title,hRes,vRes,fullscreen,fullscreen ? 0 : 200,fullscreen ? 0 : 150, 32))
		{
			LOG(TL_ERROR, "Error: m_pWindow->Create failed in CKillBirdApp::CreateWindows");
			return false;
		}
	
	}else{
		
		//erstelle Fenster mit Default Settings
		LOG(TL_ERROR, "Warning: creating default Window - check \"settings.ini\" for missing/wrong Entries !");
			
		if(!m_pWindow->Create(m_hInstance,"Kill Bird: Volume 1 - Main",800, 600, false, 200, 150, 32))
		{
			LOG(TL_ERROR, "Error: m_pWindow->Create failed in CKillBirdApp::CreateWindows");
			return false;
		}

	}
   
    m_pWindow->RegisterHandler(this);

    return true;
}

bool CKillBirdApp::SetupRenderEngine()
{
	//Renderer erstellen
	if(!CEngineFactory::InstancePtr()->GetRenderer()->Create(m_pWindow))
    {
        LOG(TL_ERROR, "Error: IRenderer::Create failed in CKillBirdApp::SetupRenderEngine");
		return false;
	}

    CEngineFactory::InstancePtr()->GetRenderer()->SetBgColor(0x3FBFEF00);

    return true;
}

//Wird aufgerufen, wenn unser IWindow geschlossen wird
void CKillBirdApp::OnWndDestroy()
{
    m_bShutdown = true;
}

bool CKillBirdApp::OnKeyDown(dword dwKey)
{
    switch(dwKey)
    {
        case cg_dwKeyEsc:
            m_bShutdown = true;
            return true;
    }
//    LOG(TL_DEBUG, "Key down: %lx", dwKey);
    return false;
}
bool CKillBirdApp::OnKeyUp(dword dwKey)
{
//    LOG(TL_DEBUG, "Key Up: %lx", dwKey);
    return false;
}
