/****************************************************************************\
*                                                                            *
* 30.04.2006                                                                 *
* CAiBirdCtrl - Source File                                             *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CAiBirdCtrl.h"
#include "CEngineFactory.h"
#include "CBirdObj.h"
#include "CGameCtrl.h"
#include "Objects/Buildings/CBuildingObj.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/
const float cg_fAttackDistanceSqr = 2.0f;
const float cg_fTurnDistance = 25.0f;
const float cg_fTurnHeight = 18.0f;
const float cg_fTurnPointDistanceSqr = 2.0f;
const float cg_fAttackHeight = 2.0f;

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
CAiBirdCtrl::CAiBirdCtrl(CGameCtrl *pGameCtrl) : m_pGameCtrl(pGameCtrl), m_pNextTarget(NULL), m_State(EABS_ATTACKING),
    m_vRot(0.0f, 0.0f, 0.0f), m_vAttackPoint(0.0f, 0.0f, 0.0f)
{
}

void CAiBirdCtrl::Update(float dt)
{
	if(m_pNextTarget && m_pNextTarget->GetState() == EBS_DESTROYED)
	{
		m_pNextTarget->RemoveDeleteObserver(this);
		m_pNextTarget = NULL;
	}

	if(m_pNextTarget == NULL)
	{
		m_pNextTarget = m_pGameCtrl->GetRandomBuilding();
		if(!m_pNextTarget)
		{
			LOG(TL_DEBUG, "Warning: CAiBirdCtrl: No building found");
            m_vRot = m_pBird->GetRot();
            m_vRot.m_Y += MY_PI/2.5f;
			return;
		}
		m_pNextTarget->AddDeleteObserver(this);
	}
	CVector3f vDir;
	CVector3f vIn;
	switch(m_State)
	{
		case EABS_ATTACKING:
            m_vAttackPoint = GetAttackPoint();
            if((m_vAttackPoint - m_pBird->GetPos()).LengthSqr() <= cg_fAttackDistanceSqr)
			{
				vDir = m_pBird->GetIn();
				vDir.m_Y = 0.0f;
				if(abs(vDir.m_X) < 0.001f && abs(vDir.m_Z) < 0.001f)
				{
					vDir.m_X = 1.0f;
					vDir.m_Z = 0.0f;
				}
				else
				{
					vDir.Normalize();
				}
				m_vTurnPoint = m_pBird->GetPos() + vDir * cg_fTurnDistance;
				m_vTurnPoint.m_Y = m_pGameCtrl->GetTerrain()->HeightAt(m_vTurnPoint.m_X, m_vTurnPoint.m_Z) + cg_fTurnHeight;
				m_State = EABS_TO_TURN_POINT;
			}
			else
			{
				m_vRot.LookAlongRot(m_vAttackPoint - m_pBird->GetPos());
			}
			break;
		case EABS_TO_TURN_POINT:
			if((m_vTurnPoint - m_pBird->GetPos()).LengthSqr() <= cg_fTurnPointDistanceSqr)
			{
				//Close enough to the turning point, lets turn
				m_State = EABS_TURNING;
			}
			else
			{
				//Calculate necessary rotation:
				m_vRot.LookAlongRot(m_vTurnPoint - m_pBird->GetPos());
			}
			break;
		case EABS_TURNING:
            m_vAttackPoint = GetAttackPoint();
			vDir = m_vAttackPoint - m_pBird->GetPos();
			vDir.m_Y = 0.0f;
			float fLength = vDir.Length();
			if(fLength > 0.0001f)
			{
				vIn = m_pBird->GetIn();
				vIn.m_Y = 0.0f;
				vDir /= fLength;
				if(vIn.Dot(vDir) < 0.5f)
				{
					m_vRot.LookAlongRot(m_vAttackPoint - m_pBird->GetPos());
				}
				else
				{
					//Ok, enough turning! Lets attack!
					m_State = EABS_ATTACKING;
				}
			}
			break;
	}
}

void CAiBirdCtrl::GetDir(CVector3f &vDir)
{
    vDir = m_pBird->GetIn();
}

bool CAiBirdCtrl::GetRot(CVector3f &vRot)
{
	vRot.ShortestRot(m_pBird->GetRot(), m_vRot);

    return true;
}

bool CAiBirdCtrl::Shoot()
{
	if(m_pNextTarget && (m_vAttackPoint - m_pBird->GetPos()).LengthSqr() <= cg_fAttackDistanceSqr)
		return true;

	return false;
}

CAiBirdCtrl::~CAiBirdCtrl()
{
	if(m_pNextTarget)
	{
		m_pNextTarget->RemoveDeleteObserver(this);
		m_pNextTarget = NULL;
	}
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
const CVector3f CAiBirdCtrl::GetAttackPoint()
{
    CSpheref sphere = m_pNextTarget->GetBoundingSphere();
    CVector3f vPos(sphere.m_vCenter);
    vPos.m_Y += sphere.m_Radius;
    vPos.m_Y += cg_fAttackHeight;

    return vPos;
}

void CAiBirdCtrl::Notify()
{
	//Ok, our target has been destroyed
	m_pNextTarget = NULL;
}
