/****************************************************************************\
*                                                                            *
* 02.05.2006                                                                 *
* CBirdObj - Source File                                                   *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CBirdObj.h"
#include "CGameCtrl.h"
#include "CBirdRep.h"
#include <CEngineFactory.h>
#include "CBirdProjObj.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/
const float cg_fBirdSpeed = 10.0f;
const float cg_fBirdTurningSpeed = 0.7f;
const CVector3f cg_vBirdShootOffset = CVector3f(0.0f, -0.5f, 0.0f);
const float cg_fBirdProjectileSpeed = 2.0f;
const float cg_fDeadTime = 5.0f;
const float cg_fAttackDelay = 1.0f;

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
CBirdObj::CBirdObj(CGameCtrl *pGameCtrl, const TSmartPointer<IFunctor> &pBirdDestroyedCb) : ICharacterObj(pGameCtrl, EOT_BIRD), m_pBirdCtrl(NULL),
    m_fAttackDelay(0.0f), m_pBirdDestroyedCb(pBirdDestroyedCb)
{
    CBirdRep *pRep = new CBirdRep;
    if(!pRep->Create(pGameCtrl->GetSceneGraph(), this))
    {
        LOG(TL_ERROR, "Error: pRep->Create failed in CBirdObj::CBirdObj");
        return;
    }
    m_pRepresentation = pRep;
    CAABox3f box(m_pRepresentation->GetOriginalBoundingBox());
	//Debug: Shrink the box a bit
	box *= CVector3f(0.4f, 1.0f, 0.4f);
	//End debug
    ICharacterObj::SetCharacter(CEngineFactory::InstancePtr()->GetPhysicsMgr()->CreateAABoxCharacter(GetPos(), box.Size(), ECG_ENEMY));
	SetGravityFactor(1.0f);
	 //Keine Collision mit anderen enemies und dem player
	SetCollisionGroups(0xFFFFFFFF & ~((1 << ECG_ENEMY) | (1 << ECG_ENEMY_PROJECTILE) | (1 << ECG_PLAYER)));
	SetState(EBS_LIVING);
    SetHitpoints(5.0f);

	//Sounds laden
	m_pDieSound = CEngineFactory::InstancePtr()->GetSoundMgr()->LoadSound(eDECOMPRESSED_SAMPLE,"Data/sounds/bird_death.mp3");
}

void CBirdObj::CollidedWith(IGameObj *pOther)
{
	switch(pOther->GetType())
	{
		case EOT_ARROW:
            SetHitpoints(GetHitpoints() - 10.0f);
			break;
        case EOT_BUILDING:
		case EOT_PLAYER:
            SetState(EBS_DEAD);
            m_fStateTime = cg_fDeadTime;
            break;
		default:
			if(GetState() == EBS_FALLING)
			{
				SetState(EBS_DEAD);
			}
			break;
	}	
}

void CBirdObj::LevelDone()
{
	Destroy();
}

void CBirdObj::SetHitpoints(float fHitpoints)
{
    m_fHitpoints = fHitpoints;
    if(m_fHitpoints <= 0.0f)
    {
        if(GetState() == EBS_LIVING)
        {
            UseGravity(true);
            SetState(EBS_FALLING);
        }
    }
}

void CBirdObj::SetBirdCtrl(const TSmartPointer<IBirdCtrl> &pBirdCtrl)
{
	if(m_pBirdCtrl)
	{
		m_pBirdCtrl->SetBird(NULL);
	}
    m_pBirdCtrl = pBirdCtrl;
    if(m_pBirdCtrl)
    {
		m_pBirdCtrl->SetBird(this);
		if(m_pRepresentation)
		{
			((CBirdRep *)((IRepresentation *)m_pRepresentation))->Show(true);
		}
    }
    else
    {
		if(m_pRepresentation)
		{
			((CBirdRep *)((IRepresentation *)m_pRepresentation))->Show(false);
		}
    }
}

void CBirdObj::Destroy()
{
    SetBirdCtrl(NULL);
    ICharacterObj::Destroy();
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/
void CBirdObj::SetState(EBirdState state)
{
	if(m_State != state)
	{
		if(m_State == EBS_LIVING)
		{
			//We are dying
			float fDistance = (m_pGameCtrl->GetCameraPos() - GetPos()).LengthSqr();
			if(fDistance < cg_fMaxSoundDistance)
			{
				m_pDieSound->SetVolume(1.0f - (fDistance / cg_fMaxSoundDistance));
				m_pDieSound->Play();
			}
			if(m_pBirdDestroyedCb)
			{
				(*m_pBirdDestroyedCb)(); //Notify the battle state that there is one bird less
				m_pBirdDestroyedCb = NULL;
			}
		}
		m_State = state;
		m_fStateTime = 0;
	}
}

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
void CBirdObj::SubUpdate(float dt)
{
	m_fStateTime += dt;
    switch(GetState())
	{
		case EBS_LIVING:
			if(m_pBirdCtrl)
			{
				m_pBirdCtrl->Update(dt);
				CVector3f vec;
				m_pBirdCtrl->GetDir(vec);
				if(vec.m_X != 0.0f || vec.m_Y != 0.0f || vec.m_Z != 0.0f)
				{
					vec.Normalize();
					Move(vec * cg_fBirdSpeed, dt);
				}
				if(m_pBirdCtrl->GetRot(vec))
				{
					float fSize = vec.Length();
					if(fSize > 0.0001f)
					{
						vec /= fSize;
						RotateX(vec.m_X * dt * cg_fBirdTurningSpeed);
						RotateY(vec.m_Y * dt * cg_fBirdTurningSpeed);
						RotateZ(vec.m_Z * dt * cg_fBirdTurningSpeed);
					}
				}
				m_fAttackDelay -= dt;
				if(m_fAttackDelay <= 0.0f)
				{
					if(m_pBirdCtrl->Shoot())
					{
						m_fAttackDelay = cg_fAttackDelay;
						CProjectileObj *pProjectile = new CBirdProjObj(m_pGameCtrl);
						pProjectile->SetPos(GetPos() + cg_vBirdShootOffset);
						pProjectile->SetRot(m_TrObj.GetRot());
						pProjectile->AddVelocity(m_TrObj.GetIn() * cg_fBirdProjectileSpeed);
						m_pGameCtrl->AddObject(pProjectile);
					}
				}
			}
			break;
		case EBS_FALLING:
			RotateX(2 * dt); //Tumble down
			RotateY(2 * dt); //Tumble down
			break;
		case EBS_DEAD:
			if(m_fStateTime > cg_fDeadTime)
			{
				Destroy();
			}
			break;
	}
}
