/****************************************************************************\
*                                                                            *
* 29.04.2006                                                                 *
* CPlayerObj - Source File                                                   *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CPlayerObj.h"
#include "CGameCtrl.h"
#include "IPlayerCtrl.h"
#include <CEngineFactory.h>
#include "CProjectileObj.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/
const float cg_fPlayerSpeed = 10.0f;
const float cg_fPlayerRunFactor = 1.5f;
const CVector3f cg_vPlayerJump = CVector3f(0.0f, 8.0f, 0.0f);

const CVector3f cg_vPlayerShootOffset = CVector3f(0.0f, 0.21f, 0.0f);
const float cg_fProjectileSpeed = 70.0f;

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
CPlayerObj::CPlayerObj(CGameCtrl *pGameCtrl) : ICharacterObj(pGameCtrl, EOT_PLAYER), m_bVectorsValid(false),
    m_pPlayerCtrl(NULL), m_wState(EPS_NORMAL)
{
    ICharacterObj::SetCharacter(CEngineFactory::InstancePtr()->GetPhysicsMgr()->CreateCapsuleCharacter(GetPos(), 0.5f, 1.9f, ECG_PLAYER));
    UseGravity(true);
	SetGravityFactor(2.0f);
	//Keine Collision mit projectiles und Vgeln
	SetCollisionGroups(0xFFFFFFFF & ~((1 << ECG_PLAYER_PROJECTILE) | (1 << ECG_ENEMY_PROJECTILE) | (1 << ECG_ENEMY)));

	//Sounds laden
	m_pShootSound = CEngineFactory::InstancePtr()->GetSoundMgr()->LoadSound(eDECOMPRESSED_SAMPLE,"Data/sounds/bowshot.mp3");
	m_pShootSound->SetVolume(0.5f);
	m_pJumpSound = CEngineFactory::InstancePtr()->GetSoundMgr()->LoadSound(eDECOMPRESSED_SAMPLE,"Data/sounds/jump.mp3");
	m_pJumpSound->SetVolume(0.5f);
}

void CPlayerObj::SetPos(const CVector3f &vPos)
{
    ICharacterObj::SetPos(vPos);
}

void CPlayerObj::SetRot(const CVector3f &vRot)
{
    ICharacterObj::SetRot(vRot);
    m_bVectorsValid = false;
}

void CPlayerObj::LookAt(const CVector3f &vPoint, const CVector3f &vUp)
{
    ICharacterObj::LookAt(vPoint, vUp);
    m_bVectorsValid = false;
}

void CPlayerObj::RotateX(float angle)
{
    ICharacterObj::RotateX(angle);
    m_bVectorsValid = false;
}

void CPlayerObj::RotateY(float angle)
{
    ICharacterObj::RotateY(angle);
    m_bVectorsValid = false;
}

void CPlayerObj::RotateZ(float angle)
{
    ICharacterObj::RotateZ(angle);
    m_bVectorsValid = false;
}

const CVector3f &CPlayerObj::GetIn()
{
    if(!m_bVectorsValid)
        UpdateVectors();
    return m_vIn;
}

const CVector3f &CPlayerObj::GetUp()
{
    if(!m_bVectorsValid)
        UpdateVectors();
    return m_vUp;
}

const CVector3f &CPlayerObj::GetRight()
{
    if(!m_bVectorsValid)
        UpdateVectors();
    return m_vRight;
}

void CPlayerObj::CollidedWith(IGameObj *pOther)
{
	//ToDo:
}

const CSpheref CPlayerObj::GetBoundingSphere()
{
	//Eigene bounding sphere erstellen
	//Wird genutzt, um den Wald rund um die Startposition aufzurumen
	return CSpheref(GetPos() + CVector3f(0.0f, 1.0f, 0.0f), 1.0f);
}

void CPlayerObj::SetPlayerCtrl(IPlayerCtrl *pPlayerCtrl)
{
    if(m_pPlayerCtrl)
    {
        m_pPlayerCtrl->SetPlayer(NULL);
    }
    m_pPlayerCtrl = pPlayerCtrl;
    if(m_pPlayerCtrl)
    {
        m_pPlayerCtrl->SetPlayer(this);
    }
}

CPlayerObj::~CPlayerObj()
{
    m_pPlayerCtrl = NULL;
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/
void CPlayerObj::SetState(word wState)
{
    m_wState = wState;
    //ToDo: Notify representation
}

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
void CPlayerObj::SubUpdate(float dt)
{
    if(m_pPlayerCtrl)
    {
        CVector3f vec;
        m_pPlayerCtrl->GetDirection(vec);
        vec.m_Y = 0.0f; //Only XZ-Movement allowed

        //Are we moving?
        if(vec.m_X == 0.0f && vec.m_Z == 0.0f)
        {
            SetState(GetState() & ~EPS_MOVING);
        }
        else
        {
            vec.Normalize();
			vec *= cg_fPlayerSpeed;
			if(m_pPlayerCtrl->Run())
			{
				vec *= cg_fPlayerRunFactor;
			}
            SetState(GetState() | EPS_MOVING);
        }

        //Wanna jump?
        if(!(GetState() & EPS_JUMPING) && m_pPlayerCtrl->Jump())
        {
            SetState(GetState() | EPS_JUMPING);
			m_pJumpSound->Play();
        }

        //Are we jumping?
        if(GetState() & EPS_JUMPING)
        {
            vec += cg_vPlayerJump;
        }

        Move(vec, dt);

        //Do we have solid ground under our feet?
        if(!IsInAir())
        {
            SetState(GetState() & ~EPS_JUMPING);
        }

        if(m_pPlayerCtrl->GetRot(vec))
        {
            RotateX(vec.m_X);
            RotateY(vec.m_Y);
		}

		if(m_pPlayerCtrl->Shoot())
		{
			CProjectileObj *pProjectile = new CProjectileObj(m_pGameCtrl, EOT_ARROW, ECG_PLAYER_PROJECTILE, "Data/models/pfeil.fbx");
			pProjectile->SetPos(GetPos() + cg_vPlayerShootOffset + m_TrObj.GetIn());
			pProjectile->SetRot(m_TrObj.GetRot());
			pProjectile->AddVelocity(m_TrObj.GetIn() * cg_fProjectileSpeed);
			m_pGameCtrl->AddObject(pProjectile);
			m_pShootSound->Play();
		}
    }
}

void CPlayerObj::UpdateVectors()
{
    CMatrix44 tmp;
    tmp.RotationY(GetRot().m_Y);

    //Get the view vectors
    m_vRight.m_X = tmp._11;
    m_vRight.m_Y = tmp._21;
    m_vRight.m_Z = tmp._31;

    m_vUp.m_X = tmp._12;
    m_vUp.m_Y = tmp._22;
    m_vUp.m_Z = tmp._32;

    //...and invert the in vector
    m_vIn.m_X = -tmp._13;
    m_vIn.m_Y = -tmp._23;
    m_vIn.m_Z = -tmp._33;

    m_bVectorsValid = true;
}
