/****************************************************************************\
*                                                                            *
* 22.05.2006                                                                 *
* CProjectileObj - Source File                                               *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CProjectileObj.h"
#include "CProjectileRep.h"
#include "CGameCtrl.h"
#include <CEngineFactory.h>

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/
const float cg_fMaxStickTime = 30.0f;

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
CProjectileObj::CProjectileObj(CGameCtrl *pGameCtrl, EObjectType type, byte bCollisionGroup, const std::string &sModel) : ICollidableObj(pGameCtrl, type), m_fStateTime(0.0f), m_pStickObj(NULL)
{
	CProjectileRep *pRep = new CProjectileRep();
	if(!pRep->Create(pGameCtrl->GetSceneGraph(), sModel, this))
	{
		LOG(TL_ERROR, "Error: pRep->Create failed in CProjectileObj::CProjectileObj");
	}

	m_pRepresentation = (IRepresentation *)pRep;

	//Actor erstellen
	CEngineFactory::InstancePtr()->GetPhysicsMgr()->BeginActorCreation(GetTransformation(), bCollisionGroup, true, 5.0f);
	CAABox3f box(m_pRepresentation->GetOriginalBoundingBox());
	CMatrix44 mat;
	mat.Translation(box.Center());
	CEngineFactory::InstancePtr()->GetPhysicsMgr()->AddCube(mat, box.Size());
	SetActor(CEngineFactory::InstancePtr()->GetPhysicsMgr()->FinishActorCreation());
}

void CProjectileObj::SetPos(const CVector3f &vPos)
{
	m_vLastPos = vPos;
	ICollidableObj::SetPos(vPos);
}

void CProjectileObj::CollidedWith(IGameObj *pOther)
{
	if(!m_pStickObj && pOther)
	{
		if(pOther->GetType() != EOT_ARROW && pOther->GetType() != EOT_BIRDPROJECTILE)  //Prevent arrows from sticking together
		{
			m_fStateTime = 0.0f;
			m_pStickObj = pOther;
			//Bei dem objekt registrieren, damit wir merken, wenn es zerstrt wird
			pOther->AddDeleteObserver(this);
			LookAt(GetPos() * 2 - m_vLastPos);	//Richtung nochmal aktualisieren
			//Wir holen uns das Offset per ray intersection
			float fDistance;
			if(!pOther->RayIntersection(GetPos(), GetIn(), m_vStickOffset, fDistance))
			{
				//Fake ray direction to hit something
				CVector3f rayDir(pOther->GetBoundingSphere().m_vCenter - GetPos());
				rayDir.Normalize();
				//Setting the ray origin one meter further away to prevent missing the first poly (GetPos() - rayDir)
				if(!pOther->RayIntersection(GetPos(), rayDir, m_vStickOffset, fDistance))
				{
					m_vStickOffset = GetPos() - pOther->GetPos();
				}
				else
				{
					m_vStickOffset -= pOther->GetPos() - rayDir * 0.15f; //Move 0.15 meter into the object
				}
			}
			else
			{
				m_vStickOffset -= pOther->GetPos() - GetIn() * 0.15f; //Move 0.15 meter into the object
			}
			m_vStickOffsetRotation = pOther->GetRot();
			m_vStickRotation = GetRot() - pOther->GetRot();
		}
	}
}

void CProjectileObj::LevelDone()
{
	Destroy();
}

void CProjectileObj::Notify()
{
	m_pStickObj = NULL;
	Destroy();
}

CProjectileObj::~CProjectileObj()
{
	if(m_pStickObj)
	{
		m_pStickObj->RemoveDeleteObserver(this);
		m_pStickObj = NULL;
	}
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
void CProjectileObj::SubUpdate(float dt)
{
	m_fStateTime += dt;
	if(m_pStickObj)
	{
		if(GetActor())
		{
			SetActor(NULL);
		}
		CMatrix44 rot;
		rot.RotationZXY(m_pStickObj->GetRot() - m_vStickOffsetRotation);
		SetRot(m_pStickObj->GetRot() + m_vStickRotation);
		SetPos(rot * m_vStickOffset + m_pStickObj->GetPos());
		if(m_fStateTime >= cg_fMaxStickTime)
		{
			m_pStickObj->RemoveDeleteObserver(this);
			Destroy();
		}
	}
	else
	{
		LookAt(GetPos() * 2 - m_vLastPos);
	}
}